/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact: 
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openrdf.sesame.query.rdql;

import java.util.ArrayList;
import java.util.List;

import org.openrdf.sesame.query.rdql.model.SfwQuery;
import org.openrdf.sesame.query.rdql.model.TripleSelector;

public class RdqlOptimizer {

	// Static class
	private RdqlOptimizer() {
	}

	public static SfwQuery optimize(SfwQuery sfwQuery) {
		// Make all explicit joins implicit
		//_handleExplicitJoins(sfwQuery);

		// Reorder the selectors
		_reorderSelectors(sfwQuery);

		return sfwQuery;
	}

	private static void _reorderSelectors(SfwQuery sfwQuery) {
		List selectors = sfwQuery.getTripleSelectors();

		// TripleSelectors are ordered such that the selectors having
		// the most _fixed_ variables are evaluated first.

		List zeroVarsList = new ArrayList();
		List oneVarList = new ArrayList();
		List twoVarsList = new ArrayList();
		List threeVarsList = new ArrayList();

		int selectorCount = selectors.size();
		for (int i = 0; i < selectorCount; i++) {
			TripleSelector selector = (TripleSelector)selectors.get(i);

			int varCount = 0;

			if (!selector.getSubjectVar().hasValue()) {
				varCount++;
			}
			if (!selector.getPredicateVar().hasValue()) {
				varCount++;
			}
			if (!selector.getObjectVar().hasValue()) {
				varCount++;
			}

			switch (varCount) {
				case 0: zeroVarsList.add(selector); break;
				case 1: oneVarList.add(selector); break;
				case 2: twoVarsList.add(selector); break;
				case 3: threeVarsList.add(selector); break;
			}
		}

		List result = new ArrayList();
		result.addAll(zeroVarsList);
		result.addAll(oneVarList);
		result.addAll(twoVarsList);
		result.addAll(threeVarsList);

		sfwQuery.setTripleSelectors(result);
	}
}
