/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact: 
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openrdf.sesame.sail.query;

import java.io.IOException;
import java.util.Collection;

import org.openrdf.model.Value;

import org.openrdf.sesame.sail.RdfSource;
/**
 * @version $Revision: 1.4 $
 */
public class All implements BooleanExpr {

	protected ValueExpr _leftArg;
	protected QueryComponent _qc;
	protected int _operator;

	public All(ValueExpr leftArg, QueryComponent qc, int op) {
		_leftArg = leftArg;
		_qc = qc;
		_operator = op;
	}

	public ValueExpr getLeftArg() {
		return _leftArg;
	}

	public QueryComponent getRightArg() {
		return _qc;
	}

	public int getOperator() {
		return _operator;
	}

	public void getVariables(Collection variables) {
		_leftArg.getVariables(variables);
		_qc.getVariables(variables);
	}

	public boolean isTrue(RdfSource rdfSource)
		throws SailQueryException
	{
		ValueListCompare vlc = new ValueListCompare();

		_qc.evaluate(rdfSource, vlc);

		return vlc.isTrue();
	}

	public String toString() {
		return _leftArg + " " + _operator + " ALL " + _qc.toString();
	}

	class ValueListCompare implements QueryAnswerListener {

		private boolean _isTrue = true;

		public void queryAnswer(Value[] values)
			throws IOException
		{
			if (_isTrue &&
				!ValueCompare.isTrue(_leftArg.getValue(), values[0], _operator))
			{
				_isTrue = false;
			}
		}

		public void clear() {
			_isTrue = true;
		}

		public boolean isTrue() {
			return _isTrue;
		}
	}
}
