/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact:
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openrdf.sesame.sail.query;

import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

import org.openrdf.model.Value;

/**
 * A QueryAnswerListener that filters any duplicates from the query results
 * and forwards these filtered results to another QueryAnswerListener.
 **/
public class DuplicatesFilter implements QueryAnswerListener {

/*--------------------------+
| Variables                 |
+--------------------------*/

	/**
	 * Recorded query result.
	 **/
	protected Set _queryAnswers;

	/**
	 * The listener to forward the filtered query results to.
	 **/
	protected QueryAnswerListener _listener;

/*--------------------------+
| Constructors              |
+--------------------------*/

	/**
	 * Creates a new DuplicatesFilter.
	 *
	 * @param l The QueryAnswerListener that will receive the filtered
	 * query results.
	 **/
	public DuplicatesFilter(QueryAnswerListener l) {
		_listener = l;
		_queryAnswers = new HashSet();
	}

/*--------------------------+
| Methods                   |
+--------------------------*/

	public void clear() {
		_listener.clear();
		_queryAnswers.clear();
	}

	public void queryAnswer(Value[] values)
		throws IOException
	{
		QueryAnswer queryAnswer = new QueryAnswer(values);

		boolean newQA = _queryAnswers.add(queryAnswer);

		if (newQA) {
			// forward the result
			_listener.queryAnswer(values);
		}
	}
}
