/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact: 
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openrdf.sesame.sail.query;

import org.openrdf.sesame.sail.RdfSource;

/**
 * Determines the intersection of two sets of values that are produced
 * by queries.
 */
public class Intersect extends SetOperator {

	public Intersect(QueryComponent leftArg, QueryComponent rightArg, boolean all) {
		super(leftArg, rightArg, all);
	}

	// overrides QueryComponent.evaluate()
	public void evaluate(RdfSource rdfSource, QueryAnswerListener listener)
			throws SailQueryException
	{
		QueryAnswerCollector qac = new QueryAnswerCollector();
		_rightArg.evaluate(rdfSource, qac);
		
		// NOTE: INTERSECT ALL never used, as it is unsupported
		
		if (!_all) {
			listener = new DuplicatesFilter(listener);
		}
		
		IntersectFilter iflt = new IntersectFilter(qac.getQueryAnswers(), listener);
		_leftArg.evaluate(rdfSource, iflt);
	}

	public String toString() {
		return _leftArg.toString() + " INTERSECT " + _rightArg.toString();
	}
}
