/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact:
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.openrdf.sesame.sail.query;

import java.io.IOException;
import java.util.Collection;
import java.util.HashSet;

import org.openrdf.model.Value;

/**
 * Generic set operator. Concrete set operations are union,
 * intersection, and minus.
 *  
 * @author Jeen Broekstra
 */
public abstract class SetOperator extends QueryComponent {

	protected QueryComponent _leftArg;
	protected QueryComponent _rightArg;
	protected boolean _all;
	
	public SetOperator(QueryComponent leftArg, QueryComponent rightArg, boolean all) {
		_leftArg = leftArg;
		_rightArg = rightArg;
		_all = all;
	}
	
	public QueryComponent getLeftArg() {
		return _leftArg;
	}

	public QueryComponent getRightArg() {
		return _rightArg;
	}
	
	public void getVariables(Collection variables) {
		_leftArg.getVariables(variables);
		_rightArg.getVariables(variables);
	}
	
	public String[] getColumnHeaders() {
		return _leftArg.getColumnHeaders();
	}
	
	// implements QueryComponent._reportQueryAnswer(QueryAnswerListener)
	protected void _reportQueryAnswer(QueryAnswerListener listener)
		throws IOException
	{
		// TODO Auto-generated method stub
	}

	// inner class for collecting query answers (necessary for filtering)
	class QueryAnswerCollector implements QueryAnswerListener {

		private HashSet _queryAnswers = new HashSet();
		
		public void queryAnswer(Value[] values)
			throws IOException
		{
			QueryAnswer queryAnswer = new QueryAnswer(values);
			_queryAnswers.add(queryAnswer);
		}

		public void clear() {
			_queryAnswers.clear();
		}
		
		public HashSet getQueryAnswers() {
			return _queryAnswers;
		}
	}
}
