/*  OMM - Ontology Middleware Module
 *  Copyright (C) 2002 OntoText Lab, Sirma AI OOD
 *
 *  Contact:
 *	Sirma AI OOD, OntoText Lab.
 *	38A, Christo Botev Blvd.
 *  1000 Sofia, Bulgaria
 *	tel. +359(2)981 00 18
 *	fax. +359(2)981 90 58
 *	info@ontotext.com
 *
 * 	http://www.ontotext.com/
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.openrdf.sesame.sailimpl.omm.security;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.openrdf.model.Resource;
/**
 * <p>Title: Knowledge Control System</p>
 * <p> </p>
 * <p> </p>
 * <p>Company: OntoText Lab. Sirma AI. </p>
 * @author  borislav popov
 * @version 1.0
 */
public class PropertiesRestriction extends Restriction
{
  /** set of the properties that form the restriction */
  Set properties = new HashSet();

  /**
   * Create a Resource Restriction.
   * @param id id of the restriction.
   * @param name name of the restriction.
   * @param descr description of the restriction.
   */
  public PropertiesRestriction(int id, String name, String descr) {
    super(id, PROPERTIES, name, descr);
  }

  /**
   * Gets the properties that form this restriction.
   * @return the properties that form this restriction.
   */
  public Set getProperties(){
    return properties;
  }

  /**
   * Sets the properties to form this restriction.
   * @param properties the properties to form this restriction.
   * @throws NullParameterException if the parameter is null.
   */
  public void setProperties(Set properties) throws NullParameterException {
    if ( null == properties ) {
      throw new NullParameterException("[properties] should not be [null].");
    }
    this.properties = properties;
  }

  public ArrayList toSql(Map idByLiteral, Map idByRes) throws NullParameterException,SecurityException{
    if (idByRes == null )
      throw new NullParameterException("[ids by resources] map should not be [null].");
    if (idByLiteral == null )
      throw new NullParameterException("[ids by literals] map should not be [null].");

    // update query for the res_prop_restrs table
    ArrayList list = super.toSql(idByLiteral,idByRes);
    ArrayList propz = new ArrayList(properties);
    StringBuffer query = new StringBuffer();

    for ( int pi = 0 ; pi < propz.size() ; pi++) {
      Resource res = (Resource)propz.get(pi);
      Integer i = (Integer) idByRes.get(res);
      if ( i == null ) {

        throw new SecurityException("Id not found for resource : \n"+res);
      }

      int pid = i.intValue();

      query = new StringBuffer();
      query.append(SecuritySail.INSERT);
      query.append(SecuritySail.RES_PROP_RESTRS_TABLE);
      query.append(SecuritySail.VALUES);
      query.append("(").append(id);
      query.append(",");
      query.append(pid).append(");");

      list.add(query.toString());

    } // for resourcez

    return list;
  }// toSql()


} // PropertiesRestriction
