/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact: 
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openrdf.sesame.sailimpl.rdbms;

import java.sql.SQLException;
import java.sql.Types;

/**
 * Defines PostgreSQL specific SQL syntax. 
 *
 * @author Peter van 't Hof
 * @author Arjohn Kampman
 */
public class PostgreSQL extends RDBMS {

	/**
	 * Initializes PostgreSQL specific SQL syntax.
	 */
	public PostgreSQL() {
		super();

		// override types for label and name; pg maps CLOB to oid, not to text.
		LABEL = "text";
		LABEL_TYPE = Types.LONGVARCHAR;
		NAME = "text";
		NAME_TYPE = Types.LONGVARCHAR;
	}

	public void optimizeTable(String tableName)
		throws SQLException
	{
		executeUpdate("VACUUM ANALYZE " + tableName);
	}

	// Overrides RDBMS._clearTable()
	protected void _clearTable(String tableName)
		throws SQLException
	{
		executeUpdate("TRUNCATE TABLE " + tableName);
	}
}
