/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact:
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openrdf.util;


/**
 * An abstract super class for type-safe enumeration classes.
 *
 * @author Arjohn Kampman
 * @version $Revision: 1.8 $
 */

public abstract class Enumeration implements java.io.Serializable {

/*------------------+
| Static stuff      |
+------------------*/

	/**
	 * Returns an Enumeration object from the supplied Enumeration
	 * array for the specified value.
	 *
	 * @param values An array of Enumeration objects.
	 * @param value The String-representation of an Enumeration object.
	 * @return An Enumeration object, or <tt>null</tt> if the specified
	 * value could not be found in the supplied Enumeration array.
	 **/
	protected static Enumeration _forValue(Enumeration[] values, String value) {
		for (int i = 0; i < values.length; i++) {
			if (values[i].toString().equals(value)) {
				return values[i];
			}
		}

		return null;
	}

/*------------------+
| Non-static stuff  |
+------------------*/

	/**
	 * The String-representation of the RDFFormat object.
	 **/
	protected final String _value;

	/**
	 * Creates a new Enumeration object.
	 **/
	protected Enumeration(String value) {
		_value = value;
	}

	/**
	 * Returns the String-representation for this Enumeration.
	 **/
	public String toString() {
		return _value;
	}

	// Overrides Object.equals()
	public boolean equals(Object o) {
		if (o == this) {
			return true;
		}
		else if (o instanceof Enumeration) {
			return _value.equals(o.toString());
		}
		else {
			return false;
		}
	}

	// Overrides Object.hashCode()
	public int hashCode() {
		return _value.hashCode();
	}
}

