/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact:
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openrdf.util.http;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * FilePart represents a file uploaded through a multipart/form-data encoded
 * POST request.
 */
public class FilePart {

	protected String _name;
	protected String _contentType;
	protected byte[] _contents = null;

	/**
	 * Construct a new filepart. Content type defaults to "application/octet-stream"
	 * if not specified.
	 */
	public FilePart(String name, String contentType, byte[] contents) {
        if (contents == null) {
            throw new IllegalArgumentException("contents can not be null");
		}

		_name = name;
		_contentType = contentType == null ? "application/octet-stream" : contentType;
		_contents = contents;
	}

    /**
     * @return an InputStream on the contents of the file.
     * Note: a new stream is returned each time.
     */
    public InputStream getInputStream()
		throws IOException
	{
		return new ByteArrayInputStream(_contents);
    }

	public int size() {
		return _contents.length;
	}

	/**
	 * @return the content type of the uploaded file as determined from the request,
	 * or the filename extension.
	 */
    public String getContentType() {
        return _contentType;
    }

	/**
	 * @return the name of the uploaded file as specified in the request.
	 */
    public String getName() {
        return _name;
    }

	/**
	 * @return the contents of the uploaded file as specified in the request.
	 */
    public byte[] getBytes() {
        return _contents;
    }
}
