package com.mkdoc.jspider;

/*
    Created:    2004-12-10
    Issued:     2004-12-10
    Modified:   2005-07-12

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.core.logging.Log;

import net.javacoding.jspider.core.util.config.PropertySet;

import com.mkdoc.util.ApplicationContext;

/**
 *  Abstract base class for MKDoc JSpider plugins.
 *  @version    0.7 2005-07-12
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public abstract class AbstractMKDocPlugin {

    /**
     *  A reference to the logging implementation.
     *  @since  Version 0.3 2005-01-14
     */
    protected Log log;

    /**
     *  The plugin vendor.
     *  @since  Version 0.1 2004-12-10
     */
    protected static final String VENDOR = "http://www.mkdoc.com";

    /**
     *  Get the vendor identifier.
     *  @return The identifier for the plugin vendor.
     *  @since  Version 0.1 2004-12-10
     */
    public final String getVendor() {

        return VENDOR;
    }

    /**
     *  Change the <code>Log</code> implementation.
     *  @param  log The <code>Log</code> implementation to
     *          substitute, not <code>null</code>.
     *  @since  Version 0.3 2005-01-14
     */
    public final void setLog(final Log log) {

        if (log != null) {

            this.log = log;
        }
        else {

            throw new IllegalArgumentException(
                        "Null log not permitted.");
        }
    }

    /**
     *  Initialialise the plugin.
     *  <p>
     *    Not implemented.
     *  </p>
     *  @since  Version 0.5 2005-04-27
     */
    public void initialize() {

        // Empty
    }

    /**
     *  Shut down the plugin.
     *  <p>
     *    Not implemented.
     *  </p>
     *  @since  Version 0.5 2005-04-27
     */
    public void shutdown() {

        // Empty
    }

    /**
     *  Get the plugin application context.
     *  @param  propertySet The property set obtained from the JSpider engine.
     *  @return A plugin application context.
     *  @since  Version 0.7 2005-07-12
     */
    public static final ApplicationContext getApplicationContext(
                                               final PropertySet propertySet) {

        if (propertySet != null) {

            return new PluginApplicationContext(propertySet);
        }
        else {

            throw new IllegalArgumentException(
                        "Null property set not permitted.");
        }
    }

}
