package com.mkdoc.jspider;

/*
    Created:    2005-04-11
    Issued:     2005-10-28
    Modified:   2005-10-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.api.model.Decision;
import net.javacoding.jspider.api.model.FetchedResource;
import net.javacoding.jspider.api.model.Site;

import net.javacoding.jspider.core.SpiderContext;

import net.javacoding.jspider.core.model.DecisionInternal;

import net.javacoding.jspider.core.rule.impl.BaseRuleImpl;

import net.javacoding.jspider.core.storage.ResourceDAO;
import net.javacoding.jspider.core.storage.Storage;

import java.net.URL;

import java.util.Locale;

/**
 *  A rule for processing only RDF content types.
 *  @version    0.2 2005-10-28
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class RdfMimeTypeOnlyRule extends BaseRuleImpl {

    /**
     *  An array of accepted content types.
     *  @since  Version 0.1 2005-04-11
     */
    private static final String[] TYPES = new String[] {"application/rdf+xml",
                                                        "application/rss+xml"};

    /**
     *  Decide whether to process a given URL.
     *  @param  context The spidering context for the URL.
     *  @param  currentSite The site the URL belongs to.
     *  @param  url The URL to decide upon.
     *  @return A decision whether to accept this URL for processing.
     *  @since  Version 0.1 2005-04-11
     */
    public final Decision apply(final SpiderContext context,
                                final Site currentSite,
                                final URL url) {

        if ((context == null) ||
            (currentSite == null) ||
            (url == null)) {

            throw new IllegalArgumentException(
                        "Null parameters not permitted.");

        }

        Storage storage = context.getStorage();

        ResourceDAO resourceDAO = storage.getResourceDAO();

        FetchedResource resource = null;

        resource = (FetchedResource) resourceDAO.getResource(url);

        if (accept(resource.getMime())) {

            return new DecisionInternal(Decision.RULE_ACCEPT,
                                        "MIME type is '" +
                                        resource.getMime() +
                                        "'- resource accepted");
        }
        else {

            return new DecisionInternal(Decision.RULE_IGNORE,
                                        "MIME type is '" +
                                        resource.getMime() +
                                        "' - resource ignored");
        }
    }

    /**
     *  Check whether a MIME type is accepted by this rule.
     *  @param  mime    The MIME type declared by the origin server.
     *  @return <code>true</code> if the type is accepted.
     *  @since  Version 0.1 2005-04-11
     */
    protected final boolean accept(final String mime) {

        if (mime == null) {

            return false;
        }
        else {

            String lowerMime = mime.toLowerCase(Locale.UK);

            for (int i = 0; i < TYPES.length; i++) {

                if (lowerMime.indexOf(TYPES[i]) == 0) {

                    return true;
                }
            }

            return false;
        }
    }

}

