package com.mkdoc.query;

/*
    Created:    2005-08-02
    Issued:     2005-10-28
    Modified:   2005-10-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

/**
 *  Base class for <code>QueryContext</code> types.
 *  @version    0.1 2005-08-02
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public abstract class AbstractQueryContext implements QueryContext {

    /**
     *  The result page number.
     *  @since  Version 0.1 2005-08-02
     */
    private int page = 1;

    /**
     *  The maximum results per page.
     *  @since  Version 0.1 2005-08-02
     */
    private int limit = LIMIT;

    /**
     *  Get the requested page number for any results.
     *  @return The page number for any results
     *  @since  Version 0.1 2005-08-02
     */
    public final int getPageNumber() {

        return page;
    }

    /**
     *  Set the requested page number for any results.
     *  @param  page    The page number for any results
     *  @since  Version 0.1 2005-08-02
     */
    protected final void setPageNumber(final int page) {

        if (page > 0) {

            this.page = page;
        }
    }

    /**
     *  Get the maximum number of results to display per page.
     *  @return The maximum number of results to display per page.
     *  @since  Version 0.1 2005-08-02
     */
    public final int getResultsPerPage() {

        return limit;
    }

    /**
     *  Set the maximum number of results to display per page.
     *  @param  limit   The maximum number of results per page.
     *  @since  Version 0.1 2005-08-02
     */
    protected final void setResultsPerPage(final int limit) {

        if (limit > 0) {

            this.limit = limit;
        }
    }

}
