package com.mkdoc.query;

/*
    Created:    2005-07-14
    Issued:     2005-07-14
    Modified:   2005-10-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

/**
 *  Represent a query result that has no results.
 *  @version    0.3 2005-10-28
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public final class NullQueryResult implements QueryResult {

    /**
     *  Get the number of rows in the result set.
     *  @return The number of rows in the result set.
     *  @since  Version 0.1 2005-07-14
     */
    public int getRows() {

        return 0;
    }

    /**
     *  Get the number of columns in the result set.
     *  @return The number of columns in the result set.
     *  @since  Version 0.1 2005-07-14
     */
    public int getColumns() {

        return 0;
    }

    /**
     *  Get the item URI for a row.
     *  @param  row The row index.
     *  @return The item URI.
     *  @since  Version 0.2 2005-07-28
     */
    public String getItemUri(final int row) {

            throw new IllegalArgumentException("No results for this query.");
    }

    /**
     *  Get the item title for a row.
     *  @param  row The row index.
     *  @return The item title or a &quot;No title...&quot; statement.
     *  @since  Version 0.2 2005-07-28
     */
    public String getItemTitle(final int row) {

            throw new IllegalArgumentException(
                        "No results for this query.");
    }

    /**
     *  Get the item summary for a row.
     *  @param  row The row index.
     *  @return The item summary or a &quot;No summary...&quot; statement.
     *  @since  Version 0.2 2005-07-28
     */
    public String getItemSummary(final int row) {

            throw new IllegalArgumentException("No results for this query.");
    }

}
