package com.mkdoc.query;

/*
    Created:    2005-07-05
    Issued:     2005-07-06
    Modified:   2005-08-04

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

/**
 *  A query that serves as a cache key.
 *  @version    0.5 2005-08-02
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public interface Query extends QueryContext {

    /**
     *  The target URI variable name.
     *  @since  Version 0.3 2005-07-28
     */
    String VAR_URI = "targetUri";

    /**
     *  The target summary variable name.
     *  @since  Version 0.3 2005-07-28
     */
    String VAR_SUMMARY = "targetSummary";

    /**
     *  The target title variable name.
     *  @since  Version 0.3 2005-07-28
     */
    String VAR_TITLE = "targetTitle";

    /**
     *  Get an SeRQL query statement.
     *  @return An SeRQL query statement.
     *  @since  Version 0.1 2005-07-05
     */
    String getSeRQLQuery();

    /**
     *  Get a unique URI for the query.
     *  @return A unique URI for the query.
     *  @since  Version 0.2 2005-07-26
     */
    String getQueryUri();

    /**
     *  Get a request URI for a result set by page number.
     *  @param  page    The page for which to get a query URI.
     *  @return A unique URI for the query.
     *  @since  Version 0.5 2005-08-02
     */
    String getQueryUri(final int page);

    /**
     *  Get the unique result key for the query.
     *  @return A unique key for the query result in the form of a URI without
     *          output type or paging information.
     *  @since  Version 0.4 2005-08-01
     */
    String getResultKey();

}
