package com.mkdoc.query;

/*
    Created:    2005-07-26
    Issued:     2005-10-28
    Modified:   2005-10-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

/**
 *  Get query components for a servlet application.
 *  @version    0.2 2005-10-28
 *  @author     Philip Shaw <phil@mkdoc.com>
 *  @TODO       Implement an abstract factory pattern using ApplicationContext
 *              and an extended Query interface to identify types.
 */
public final class QueryFactory {

    /**
     *  Prevent instantiation.
     *  @since  Version 0.1 2005-07-26
     */
    private QueryFactory() {

        super();
    }

    /**
     *  Get a result renderer.
     *  @param  query   A query object for the results.
     *  @return A result renderer type appropriate for the request.
     *  @since  Version 0.1 2005-07-26
     */
    public static ResultRenderer getResultRenderer(final Query query) {

        if (query == null) {

            throw new IllegalArgumentException("Null query not permitted.");
        }
        else {

            String type = query.getOutputType();

            if (type.equals(QueryContext.TEXT)) {

                return new TextResultRenderer(query);
            }
            if (type.equals(QueryContext.RSS)) {

               return new RssResultRenderer(query);
            }

            return new XhtmlResultRenderer(query);
        }
    }

}
