package com.mkdoc.query;

/*
    Created:    2005-01-26
    Issued:     2005-01-26
    Modified:   2005-10-12

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.io.IOException;

import javax.servlet.ServletOutputStream;

import javax.servlet.jsp.JspWriter;

/**
 *  An interface for types that render query results.
 *  @version    0.6 2005-10-11
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public interface ResultRenderer {

    /**
     *  The full document output mode.
     *  @since  Version 0.1 2005-07-19
     */
    int FULL = 0;

    /**
     *  The document insertion output mode.
     *  @since  Version 0.1 2005-07-19
     */
    int INSERT = 1;

    /**
     *  Get the content type for the renderer.
     *  @return A content type for the rendered output.
     *  @since  Version 0.1 2005-01-26
     */
    String getContentType();

    /**
     *  Render a query results table to a servlet output stream in
     *  <code>FULL</code> mode.
     *  @param  outputStream    The output stream to which to write the
     *          results.
     *  @param  queryResult The query result to render.
     *  @exception  IOException If there is a problem writing results to the
     *          output stream.
     *  @since  Version 0.5 2005-09-12
     */
    void renderQueryResult(final ServletOutputStream outputStream,
                           final QueryResult queryResult)
        throws IOException;

    /**
     *  Render a query result to a servlet output stream.
     *  <p>
     *    Implementations may render output according to a particular markup
     *    language or content type.
     *  </p>
     *  @param  outputStream    The output stream to which to write the
     *          results.
     *  @param  queryResult The query result to render.
     *  @param  mode    The output mode, either
     *          <code>ResultRenderer.FULL</code> or
     *          <code>ResultRenderer.INSERT</code>.
     *  @exception  IOException If there is a problem writing results to the
     *          output stream.
     *  @since  Version 0.2 2005-07-14
     */
    void renderQueryResult(final ServletOutputStream outputStream,
                           final QueryResult queryResult,
                           final int mode) throws IOException;

    /**
     *  Render a query result to a servlet output stream in <code>FULL</code>
     *  mode.
     *  <p>
     *    Implementations may render output according to a particular markup
     *    language or content type.
     *  </p>
     *  @param  jspWriter    The output stream to which to write the results.
     *  @param  queryResult The query result to render.
     *  @exception  IOException If there is a problem writing results to the
     *          output stream.
     *  @since  Version 0.5 2005-09-12
     */
    void renderQueryResult(final JspWriter jspWriter,
                           final QueryResult queryResult) throws IOException;


    /**
     *  Render a query result to a servlet output stream.
     *  <p>
     *    Implementations may render output according to a particular markup
     *    language or content type.
     *  </p>
     *  @param  jspWriter    The output stream to which to write the results.
     *  @param  queryResult The query result to render.
     *  @param  mode    The output mode, either
     *          <code>ResultRenderer.FULL</code> or
     *          <code>ResultRenderer.INSERT</code>.
     *  @exception  IOException If there is a problem writing results to the
     *          output stream.
     *  @since  Version 0.3 2005-07-19
     */
    void renderQueryResult(final JspWriter jspWriter,
                           final QueryResult queryResult,
                           final int mode) throws IOException;

    /**
     *  Render a query result to a string.
     *  @param  queryResult The query result to render.
     *  @param  mode    The output mode, either
     *          <code>ResultRenderer.FULL</code> or
     *          <code>ResultRenderer.INSERT</code>.
     *  @return A formatted representation of the query results.
     *  @since  Version 0.2 2005-07-14
     */
    String renderQueryResult(final QueryResult queryResult,
                             final int mode);

}
