package com.mkdoc.query;

/*
    Created:    2005-01-26
    Issued:     2005-01-26
    Modified:   2005-10-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

/**
 *  Renders Sesame query results to plain text.
 *  @version    0.8 2005-10-28
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class TextResultRenderer extends AbstractResultRenderer
                                     implements ResultRenderer {

    /**
     *  The text content type.
     *  @since  Version 0.1 2005-01-26
     */
    public static final String CONTENT_TYPE = "text/plain";

    /**
     *  Construct a renderer for a specific query.
     *  @param  query   The query against which the results were returned.
     *  @since  Version 0.5 2005-07-26
     */
    public TextResultRenderer(final Query query) {

        super();

        if (query == null) {

            throw new IllegalArgumentException(
                        "Null query not permitted.");
        }
        else {

            this.query = query;
        }
    }

    /**
     *  Render query results to a string buffer.
     *  @param  queryResult The query results to render.
     *  @param  mode    The output mode, either
     *          <code>ResultRenderer.FULL</code> or
     *          <code>ResultRenderer.INSERT</code>.
     *  @return A text formatted tabular layout of the query results.
     *  @since  Version 0.1 2005-01-26
     */
    protected final StringBuffer getQueryResult(final QueryResult queryResult,
                                                final int mode) {

        StringBuffer buffer = new StringBuffer();

        if (queryResult == null) {

            throw new IllegalArgumentException(
                        "Null query result not permitted.");
        }
        else {

            int rows = queryResult.getRows();

            if (rows > 0) {

                int start = getStartIndex(queryResult);

                int end = getEndIndex(queryResult);

                if (rows > 1) {

                    buffer.append("Results ");
                }
                else {

                    buffer.append("Result ");
                }

                buffer.append(start + 1);

                if (rows > 1) {

                    buffer.append(" to ");
                    buffer.append(end);
                }

                buffer.append(" of ");
                buffer.append(String.valueOf(rows));
                buffer.append(".\n");

                buffer.append("Query results\n");

                for (int row = 0; row < rows; row++) {

                    buffer.append(queryResult.getItemTitle(row));
                    buffer.append("\n");
                    buffer.append(queryResult.getItemSummary(row));
                    buffer.append("\n");
                    buffer.append(queryResult.getItemUri(row));
                    buffer.append("\n\n");
                }
            }
            else {

                buffer.append("No results");
            }

            return buffer;
        }
    }

    /**
     *  Get the content type for the renderer.
     *  @return The MIME type <code>text/plain</code>.
     *  @since  Version 0.1 2005-01-26
     */
    public final String getContentType() {

        return CONTENT_TYPE;
    }

}
