package com.mkdoc.sax;

/*
    Created:    2005-02-14
    Issued:     2005-02-14
    Modified:   2005-05-31

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.io.PrintWriter;

import java.net.URL;

import com.mkdoc.store.FileStoreManager;

import org.xml.sax.SAXException;

/**
 *  Base class for content handlers that write N-Triples.
 *  @version    0.4 2005-05-31
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public abstract class AbstractTripleWriter
              extends AbstractRdfContentHandler {

    /**
     *  A manager reference.
     *  @since  Version 0.4 2005-05-31
     */
    protected FileStoreManager manager;

    /**
     *  The output stream for the metadata.
     *  @since  Version 0.1 2005-02-14
     */
    protected PrintWriter out;

    /**
     *  Write an initial status message.
     *  @exception  SAXException    If there is a problem getting an output
     *          for the parse.
     *  @since  Version 0.1 2005-02-14
     */
    public final void startDocument() throws SAXException {

        try {

            URL url = new URL(subject);

            out = new PrintWriter(manager.getOutputStream(url),
                                  true);
        }
        catch (Exception e) {

            throw new SAXException(e.getMessage());
        }

        out.println("# Metadata triples derived from " + subject);
    }

    /**
     *  Flush the output and close.
     *  @exception  SAXException    Never thrown in this
     *          implementation.
     *  @since  Version 0.1 2005-02-14
     */
    public final void endDocument() throws SAXException {

        if (out != null) {

            out.println();
            out.flush();
            out.close();
        }
    }

}
