package com.mkdoc.sax;

/*
    Created:    2005-02-28
    Issued:     2005-02-28
    Modified:   2005-05-26

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.schema.ApplicationProfile;

import org.xml.sax.ContentHandler;

/**
 *  Handler type for extracting RDF content.
 *  @version    0.3 2005-05-26
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public interface RdfContentHandler extends ContentHandler {

    /**
     *  Set the application profile for the handler.
     *  @param  profile The application profile to be used.
     *  @since  Version 1.5 2005-05-26
     */
    void setApplicationProfile(final ApplicationProfile profile);

    /**
     *  Set the subject of the RDF content handler.
     *  @param  uri The URI of the subject, such as the URL
     *          of an (X)HTML document.
     *  @since  Version 0.1 2005-02-28
     */
    void setSubject(final String uri);

    /**
     *  Get the current subject URI for the RDF handler.
     *  @return The URI of the RDF statement, which may be
     *          the URL of an (X)HTML document.
     *  @since  Version 0.1 2005-02-28
     */
    String getSubject();

    /**
     *  Expand a schema encoded value to a URI.
     *  @param  predicate   The encoded triple predicate to
     *          expand in the form &quot;DC.Title&quot; for
     *          instance.
     *  @return The expanded URI reference for the triple
     *          predicate or the original string if it is
     *          not matched.
     *  @since  Version 0.1 2005-02-28
     */
    String expandPredicate(final String predicate);

    /**
     *  Expand a space separated list of schema encoded values.
     *  <p>
     *    Intended to extract Dublin Core encoded values from HTML
     *    <code>link</code> elements' <code>rel</code> or <code>rev</code>
     *    attributes, which may be mixed with generic link types. Distinguishes
     *    properties that are <em>not</em> encoded metadata predicates and
     *    silently drops them.
     *  </p>
     *  @param  list    The encoded, space separated link list.
     *  @return An array of expanded URI references only.
     *  @since  Version 0.1 2005-02-28
     */
    String[] expandPredicates(final String list);

    /**
     *  Expand an encoding scheme to a URI.
     *  @param  scheme  The encoded triple component to
     *          expand in the form
     *          &quot;DCTERMS.W3CDTF&quot; for instance.
     *  @return The expanded URI reference for the triple
     *          component or the original string if it is
     *          not matched.
     *  @since  Version 0.1 2005-02-28
     */
    String expandScheme(final String scheme);

    /**
     *  Get a blank node.
     *  @return A unique blank node reference.
     *  @since  Version 0.1 2005-02-28
     */
    String getBNode();

    /**
     *  Get a blank node identifier.
     *  @return A unique blank node identifier.
     *  @since  Version 0.1 2005-02-28
     */
    String getBNodeId();
}
