package com.mkdoc.sax;

/*
    Created:    2004-12-10
    Issued:     2004-12-15
    Modified:   2004-12-15

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import org.xml.sax.helpers.XMLFilterImpl;

/**
 *  An XHTML <code>meta</code> element filter.
 *  <p>
 *    Provides a gate for stripping out non-metadata
 *    content to simplify processing downstream. Only
 *    XHTML <code>meta</code> element content is passed.
 *  </p>
 *  <p>
 *    All other events pass directly to the relevant
 *    superclass handlers, which do nothing.
 *  </p>
 *  @version    0.1 2004-10-12
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class XhtmlMetaFilter extends XMLFilterImpl {

    /**
     *  Ignore irrelevant elements.
     *  @param  uri The namespace URI for the element.
     *  @param  localName   The local element name.
     *  @param  qName   The qualified element name.
     *  @param  atts    A set of attributes.
     *  @exception  SAXException    The underlying
     *          <code>ContentHandler</code> may throw
     *          this exception handling this event.
     *  @since  Version 0.1 2004-12-10
     */
    public final void startElement(final String uri,
                                   final String localName,
                                   final String qName,
                                   final Attributes atts)
        throws SAXException {

        if (localName.equals("meta")) {

            super.startElement(uri, localName, qName, atts);
        }
    }

}
