package com.mkdoc.sax;

/*
    Created:    2005-02-15
    Issued:     2005-02-16
    Modified:   2005-05-31

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.store.FileStoreManager;

import java.net.URL;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

/**
 *  A composite XHTML metadata triple writer.
 *  @version    0.4 2005-05-31
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class XhtmlTripleWriter extends AbstractTripleWriter
                            implements ContentHandler {

    /**
     *  A handler for XHTML <code>meta</code> elements.
     *  @since  Version 0.1 2005-02-15
     */
    private final ElementTripleWriter metaWriter =
                                                new MetaTripleWriter(profile);

    /**
     *  A handler for XHTML <code>link</code> elements.
     *  @since  Version 0.1 2005-02-15
     */
    private final ElementTripleWriter linkWriter =
                                                new LinkTripleWriter(profile);

    /**
     *  Construct with a subject manager and initial URI.
     *  @param  manager A subject manager interface to write the triples to.
     *  @param  url An initial URI subject.
     *  @since  Version 0.1 2005-02-15
     */
    public XhtmlTripleWriter(final FileStoreManager manager,
                             final URL url) {

        if (manager != null) {

            if (url != null) {

                this.manager = manager;

                subject = url.toExternalForm();
            }
            else {

                throw new IllegalArgumentException(
                            "Null URI subject not permitted.");
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null manager not permitted.");
        }
    }

    /**
     *  Extract metadata from <code>meta</code> and <code>link</code> elements.
     *  @param  uri The namespace URI for the element.
     *  @param  localName   The local element name.
     *  @param  qName   The qualified element name.
     *  @param  attributes    A set of attributes.
     *  @exception  SAXException    If the element is not
     *          a <code>meta</code> element.
     *  @since  Version 0.1 2005-02-14
     */
    public final void startElement(final String uri,
                                   final String localName,
                                   final String qName,
                                   final Attributes attributes)
        throws SAXException {

        if (metaWriter.acceptElement(uri, localName)) {

            metaWriter.printTriples(subject, attributes, out);
        }
        else {

            if (linkWriter.acceptElement(uri, localName)) {

                linkWriter.printTriples(subject, attributes, out);
            }
            else {

                out.close();

                throw new SAXException("Cannot process element '" +
                                       qName + "'.");
            }
        }
    }

}
