package com.mkdoc.schema;

/*
    Created:    2005-05-24
    Issued:     2005-05-24
    Modified:   2005-07-25

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.util.Map;

/**
 *  A profile for a specific metadata application.
 *  @version    0.6 2005-07-25
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public interface ApplicationProfile extends Resolver {

    /**
     *  Get a mapping of schemas with their default prefixes.
     *  @return A map in which lower case default schema prefixes are keys to
     *          the schemas themselves.
     *  @since  Version 0.4 2005-06-10
     */
    Map getSchemaMap();

    /**
     *  Get the schema property for a given predicate.
     *  @param  predicate   The predicate to look up.
     *  @return The schema property for the predicate or <code>null</code> if no
     *          mapping exists.
     *  @since  Version 0.4 2005-06-10
     */
    SchemaProperty getPredicateProperty(final String predicate);

    /**
     *  Get the primary search property for the schema.
     *  @return The primary search property for the schema.
     *  @since  Version 0.5 2005-06-17
     */
    SchemaProperty getPrimarySearchProperty();

    /**
     *  Get the title property for the schema.
     *  @return The resource title property for the schema.
     *  @since  Version 0.6 2005-07-25
     */
    SchemaProperty getTitleProperty();

    /**
     *  Get the summary property for the schema.
     *  @return The resource summary property for the schema.
     *  @since  Version 0.6 2005-07-25
     */
    SchemaProperty getSummaryProperty();

}
