package com.mkdoc.schema;

/*
    Created:    2005-05-25
    Issued:     2005-05-27
    Modified:   2005-07-25

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

/**
 *  A pure Dublin Core application profile.
 *  <p>
 *    Interim version adapts a DublinCoreElements instance.
 *  </p>
 *  @version    0.5 2005-07-25
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class DublinCoreProfile extends AbstractApplicationProfile
                                    implements ApplicationProfile {

    /**
     *  Set up the schema mapping.
     *  @since  Version 0.1 2005-05-25
     */
    public DublinCoreProfile() {

        addSchema(new DublinCoreElements());

        addSchema(new DublinCoreTerms());
    }

    /**
     *  Template method for falling back to profile-specific algorithms.
     *  @param  predicate   A predicate that has not been matched using default
     *          schema prefixes.
     *  @return A predicate URI or <code>null</code> if not matched.
     *  @since  Version 0.2 2005-05-27
     */
    protected final String expandPredicateProfile(final String predicate) {

        if (predicate != null) {

            Schema schema = getPrimarySchema(DCTERMS);

            return schema.expandPredicate(predicate);
        }
        else {

            throw new IllegalArgumentException(
                        "Null predicate not permitted.");
        }
    }

    /**
     *  Template method for falling back to profile-specific algorithms.
     *  @param  scheme  A scheme that has not been matched using default
     *          schema prefixes.
     *  @return A scheme URI or <code>null</code> if not matched.
     *  @since  Version 0.2 2005-05-27
     */
    protected final String expandSchemeProfile(final String scheme) {

        if (scheme != null) {

            Schema schema = getPrimarySchema(DCTERMS);

            return schema.expandScheme(scheme);
        }
        else {

            throw new IllegalArgumentException(
                        "Null scheme not permitted.");
        }

    }

    /**
     *  Get the primary search property for the schema.
     *  @return The primary search property for the schema.
     *  @since  Version 0.4 2005-06-17
     */
    public final SchemaProperty getPrimarySearchProperty() {

        return getPredicateProperty("dc.subject");
    }

    /**
     *  Get the title property for the schema.
     *  @return The resource title property for the schema.
     *  @since  Version 0.5 2005-07-25
     */
    public final SchemaProperty getTitleProperty() {

        return getPredicateProperty("dc.title");
    }

    /**
     *  Get the summary property for the schema.
     *  @return The resource summary property for the schema.
     *  @since  Version 0.5 2005-07-25
     */
    public final SchemaProperty getSummaryProperty() {

        return getPredicateProperty("dc.description");
    }

}
