package com.mkdoc.schema;

/*
    Created:    2004-12-22
    Issued:     2004-12-22
    Modified:   2005-06-17

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.util.Collection;

/**
 *  An interface for metadata schema.
 *  <p>
 *    XHTML metadata schema are expected to have a prefix
 *    to distinguish vocabularies, e.g. &quot;DC&quot; for
 *    Dublin Core.
 *  </p>
 *  @version    0.9 2005-06-17
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public interface Schema extends Resolver {

    /**
     *  Get the base URI for the metadata schema.
     *  @return The namespace URI for the metadata schema.
     *  @since  Version 0.2 2004-12-23
     */
    String getUri();

    /**
     *  Get the default namespace prefix for the schema.
     *  @return The default namepace prefix for the schema.
     *  @since  Version 0.3 2005-03-01
     */
    String getDefaultPrefix();

    /**
     *  Get the HTML metadata prefixes recognised by this schema.
     *  @return An array of one or more schema prefixes in mixed case.
     *  @since  Version 0.6 2005-05-27
     */
    String[] getAllPrefixes();

    /**
     *  Signal whether the schema provides element qualifiers.
     *  @return <code>true</code> if the schema
     *  @since  Version 0.4 2005-03-10
     */
    boolean providesQualifiers();

    /**
     *  Check whether the schema provides qualifiers for a specific schema.
     *  @param  uri The URI of another schema to check for extensions.
     *  @return <code>true</code> if the schema provieds qualifiers for the
     *        schema signified by the URI.
     *  @since  Version 0.4 2005-03-10
     */
    boolean providesQualifiersFor(final String uri);

    /**
     *  Get a collection of schema property predicates for this schema.
     *  @return A collection of predicate names.
     *  @since  Version 0.7 2005-06-10
     */
    Collection getPredicates();

    /**
     *  Get the schema property for a given predicate.
     *  @param  name    The predicate name without prefix.
     *  @return The schema property for the predicate or <code>null</code> if no
     *          mapping exists.
     *  @since  Version 0.8 2005-06-16
     */
    SchemaProperty getNamedPredicateProperty(final String name);

    /**
     *  Get the common name for the schema.
     *  @return The common name for the schema.
     *  @since  Version 0.9 2005-06-17
     */
    String getName();
}
