package com.mkdoc.servlet;

/*
    Created:    2005-07-12
    Issued:     2005-10-18
    Modified:   2005-11-02

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.util.AbstractApplicationContext;
import com.mkdoc.util.ApplicationContext;
import com.mkdoc.util.ApplicationContextException;

import java.io.File;

import javax.servlet.ServletContext;

/**
 *  A servlet application context.
 *  @version    0.4 2005-10-18
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class ServletApplicationContext extends AbstractApplicationContext
                                            implements ApplicationContext {

    /**
     *  The default store file path, relative to the application context path.
     *  @since  Version 0.3 2005-10-14
     */
    public static final String STORE_PATH = "/WEB-INF/rdf";

    /**
     *  Provide a wrapper for a servlet context.
     *  @since  Version 0.1 2005-07-12
     */
    private final ServletContext context;

    /**
     *  Provide a wrapper for a servlet context.
     *  @param  context A context obtained from the servlet container.
     *  @since  Version 0.1 2005-07-12
     */
    public ServletApplicationContext(final ServletContext context) {

        super();

        if (context == null) {

            throw new IllegalArgumentException(
                        "Null servlet context not permitted.");
        }
        else {

            this.context = context;
        }
    }

    /**
     *  Get a named parameter value.
     *  @param  name    The name of the parameter to look up.
     *  @return The value of the named parameter, or <code>null</code> if not
     *          found.
     *  @since  Version 0.1 2005-07-12
     */
    public final String getStringParameter(final String name) {

        return context.getInitParameter(name);
    }

    /**
     *  Get a named parameter value.
     *  @param  name    The name of the parameter to look up.
     *  @return A reference to the class of the named parameter, or
     *          <code>null</code> if there is a problem loading the class.
     *  @since  Version 0.1 2005-07-12
     */
    public final Class getClassParameter(final String name) {

        String className = context.getInitParameter(name);

        if (className == null) {

            return null;
        }
        else {

            try {

                return Class.forName(className);
            }
            catch (Exception e) {

                return null;
            }
        }
    }

    /**
     *  Get a named parameter value.
     *  @param  name    The name of the parameter to look up.
     *  @return The boolean value of named parameter, or <code>false</code> if
     *          not found.
     *  @since  Version 0.1 2005-07-12
     */
    public final boolean getBooleanParameter(final String name) {

        return "true".equalsIgnoreCase(context.getInitParameter(name));
    }

    /**
     *  Get the default store directory.
     *  @return A reference to the default output directory.
     *  @since  Version 0.2 2005-10-14
     */
    protected final File getDefaultStoreDirectory() {

        String realPath = context.getRealPath(STORE_PATH);

        if (realPath == null) {

            throw new ApplicationContextException(
                        "Cannot obtain a real path for the store " +
                        STORE_PATH);
        }

        return new File(realPath);
    }

}
