package com.mkdoc.sesame;

/*
    Created:    2005-01-26
    Issued:     2005-01-26
    Modified:   2005-07-22

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.openrdf.sesame.query.QueryResultsTable;

import java.io.IOException;

import javax.servlet.ServletOutputStream;

/**
 *  An interface for types that render Sesame query results.
 *  @version    0.1 2005-01-26
 *  @author     Philip Shaw <phil@mkdoc.com>
 *  @deprecated Use com.mkdoc.query.ResultRenderer
 */
public interface ResultsRenderer {

    /**
     *  Get the content type for the renderer.
     *  @return A content type for the rendered output.
     *  @since  Version 0.1 2005-01-26
     */
    String getContentType();

    /**
     *  Render a query results table to a servlet output stream.
     *  <p>
     *    Implementations may render output according to a particular markup
     *    language or content type.
     *  </p>
     *  @param  outputStream    The output stream to which to write the
     *          results.
     *  @param  table   The query results table to render.
     *  @exception  IOException If there is a problem writing results to the
     *          output stream.
     *  @since  Version 0.1 2005-01-26
     */
    void renderResults(final ServletOutputStream outputStream,
                       final QueryResultsTable table) throws IOException;

    /**
     *  Render a query results table to a string.
     *  @param  table   The query results table to render.
     *  @return A formatted representation of the query results.
     *  @since  Version 0.1 2005-01-26
     */
    String renderResults(final QueryResultsTable table);

}
