package com.mkdoc.util;

/*
    Created:    2005-10-04
    Issued:     2005-10-05
    Modified:   2005-10-14

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.io.File;

/**
 *  Base superclass for application contexts.
 *  @version    0.2 2005-10-14
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public abstract class AbstractApplicationContext
                                                implements ApplicationContext {

    /**
     *  Get the store directory.
     *  @return A reference to the default output directory, or a custom
     *          location if configured.
     *  @since  Version 0.1 2005-10-04
     */
    public final File getStoreDirectory() {

        File defaultDir = getDefaultStoreDirectory();

        String customPath = getStringParameter(DIRECTORY);

        if (getBooleanParameter(ABSOLUTE)) {

            if (customPath == null) {

                return defaultDir;
            }
            else {

                return new File(customPath);
            }
        }
        else {

            if (customPath == null) {

                return defaultDir;
            }
            else {

                return new File(defaultDir,
                                customPath);
            }
        }
    }

    /**
     *  Get the reopsitory file path, if any.
     *  @return An absolute file path, or <code>null</code> if not found.
     *  @since  Version 0.2 2005-10-14
     */
    public final String getStorePath() {

        File directory = getStoreDirectory();

        String path = getStringParameter(REPOSITORY);

        if (path != null) {

            File store = new File(directory, path);

            return store.getAbsolutePath();
        }

        return null;
    }

    /**
     *  A template method to get a default store directory.
     *  @return A reference to the default output directory.
     *  @since  Version 0.2 2005-10-14
     */
    protected abstract File getDefaultStoreDirectory();

}
