package com.mkdoc.util;

/*
    Created:    2005-07-12
    Issued:     2005
    Modified:   2005-10-14

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.io.File;

/**
 *  An application context that contains configuration parameters.
 *  @version    0.4 2005-10-14
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public interface ApplicationContext {

    /**
     *  The absolute file path property name.
     *  @since  Version 0.1 2005-10-04
     */
    String ABSOLUTE = "output.absolute";

    /**
     *  The output file path property name.
     *  @since  Version 0.1 2005-10-04
     */
    String DIRECTORY = "output.folder";

    /**
     *  A repository file path associated with the application.
     *  @since  Version 0.3 2005-10-04
     */
    String REPOSITORY = "repository";

    /**
     *  Get a named parameter value.
     *  @param  name    The name of the parameter to look up.
     *  @return The value of the named parameter, or <code>null</code> if not
     *          found.
     *  @since  Version 0.1 2005-07-12
     */
    String getStringParameter(final String name);

    /**
     *  Get a named parameter value.
     *  @param  name    The name of the parameter to look up.
     *  @return A reference to the class of the named parameter, or
     *          <code>null</code> if there is a problem loading the class.
     *  @since  Version 0.1 2005-07-12
     */
    Class getClassParameter(final String name);

    /**
     *  Get a named parameter value.
     *  @param  name    The name of the parameter to look up.
     *  @return The boolean value of named parameter, or <code>false</code> if
     *          not found.
     *  @since  Version 0.1 2005-07-12
     */
    boolean getBooleanParameter(final String name);

    /**
     *  Get a (temporary) output folder associated with the application.
     *  @return A <code>File</code> reference for the named parameter, or
     *          <code>null</code> if not found.
     *  @since  Version 0.1 2005-07-12
     */
    File getStoreDirectory();

    /**
     *  Get the reopsitory file path, if any.
     *  @return An absolute file path, or <code>null</code> if not found.
     *  @since  Version 0.3 2005-10-04
     */
    String getStorePath();

}
