package com.mkdoc.io;

/*
    Created:    2004-11-05
    Issued:     2004-11-05
    Modified:   2005-01-07

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.io.File;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test the directory file filter.
 *  @version    0.1 2004-11-05
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class DirectoryFileFilterTest extends TestCase {

    /**
     *  A Java file filter to test.
     *  @since  Version 0.1 2004-11-05
     */
    private DirectoryFileFilter filter;

    /**
     *  A file type.
     *  @since  Version 0.1 2004-11-05
     */
    private static final String EXT_JAVA = "Example.java";

    /**
     *  A directory path.
     *  @since  Version 0.1 2004-11-05
     */
    private static final String DIR = System.getProperty("mkHome");

    /**
     *  Test class constructor.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2004-11-05
     */
    public DirectoryFileFilterTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2004-11-05
     */
    protected final void setUp() {

        filter = new DirectoryFileFilter();
    }

    /**
     *  Should accept a Java file extension.
     *  @since  Version 0.1 2004-11-05
     */
    public final void testAcceptTrue() {

        File dir = new File(DIR);

        assertTrue(filter.accept(dir));
    }

    /**
     *  Should not accept a text file extension.
     *  @since  Version 0.1 2004-11-05
     */
    public final void testAcceptFalse() {

        File file = new File(EXT_JAVA);

        assertFalse(filter.accept(file));
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2004-11-05
    protected void tearDown() {

    }
     */

    /**
     *  Get a test suite for this class.
     *  @return A test suite.
     *  @since  Version 0.1 2004-11-05
     */
    public static Test suite() {

        return new TestSuite(DirectoryFileFilterTest.class);
    }

    /**
     *  Run the test as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2004-11-05
     */
    public static void main(final String[] args) {

            junit.textui.TestRunner.run(suite());
    }

}
