package com.mkdoc.jsp;

/*
    Created:    2005-06-29
    Issued:     2005
    Modified:   2005-06-29

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.servlet.MockServletOutputStream;

import java.io.IOException;

import javax.servlet.jsp.JspWriter;

/**
 *  Mock JSP writer for testing.
 *  @version    0.1 2005-06-29
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockJspWriter extends JspWriter {

    /**
     *  The line separator.
     *  @since  Version 0.1 2005-06-29
     */
    private static final String newLine = System.getProperty("line.separator");

    /**
     *  Use a StringBuffer to simulate and report the output.
     *  @since  Version 0.1 2005-06-29
     */
    private StringBuffer buffer = new StringBuffer();

    /**
     *  Whether to throw an exception with the print methods.
     *  @since  Version 0.1 2005-06-29
     */
    private boolean failOnPrint = false;

    /**
     *  Default constructor.
     *  @since  Version 0.1 2005-06-29
     */
    public MockJspWriter() {

        super(0, false);
    }

    /**
     *  Get the output that has been written since the last call.
     *  @since  Version 0.1 2005-06-29
     */
    public final String getOut() {

        return buffer.toString();
    }

    /**
     *  Set print methods to throw an IOException.
     *  @since  Version 0.1 2005-06-29
     */
    public final void setFailOnPrint(boolean setting) {

        failOnPrint = setting;
    }


    /**
     *  Print a new line.
     *  @since  Version 0.1 2005-06-29
     */
    public final void newLine() throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws IOException.");
        }
        buffer.append(newLine);
    }

    /**
     *  Print a boolean.
     *  @since  Version 0.1 2005-06-29
     */
    public final void print(boolean b) throws java.io.IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws IOException.");
        }
        buffer.append(b);
    }

    /**
     *  Print a character.
     *  @since  Version 0.1 2005-06-29
     */
    public final void print(char c) throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws IOException.");
        }
        buffer.append(c);
    }

    /**
     *  Print an int.
     *  @since  Version 0.1 2005-06-29
     */
    public final void print(int i) throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws IOException.");
        }
        buffer.append(i);
    }

    /**
     *  Print a long.
     *  @since  Version 0.1 2005-06-29
     */
    public final void print(long l) throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws IOException.");
        }
        buffer.append(l);
    }

    /**
     *  Print a float.
     *  @since  Version 0.1 2005-06-29
     */
    public final void print(float f) throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws IOException.");
        }
        buffer.append(f);
    }

    /**
     *  Print a double.
     *  @since  Version 0.1 2005-06-29
     */
    public final void print(double d) throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws IOException.");
        }
        buffer.append(d);
    }

    /**
     *  Print a character array.
     *  @since  Version 0.1 2005-06-29
     */
    public final void print(char[] s) throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws IOException.");
        }
        buffer.append(s);
    }

    /**
     *  Print a string.
     *  @since  Version 0.1 2005-06-29
     */
    public final void print(java.lang.String s) throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws IOException.");
        }
        buffer.append(s);
    }

    /**
     *  Print an object, not implemented.
     *  @since  Version 0.1 2005-06-29
     */
    public final void print(java.lang.Object obj) throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws an IOException printing an object.");
        }

        buffer.append(obj);
    }

    /**
     *  Print a new line.
     *  @since  Version 0.1 2005-06-29
     */
    public final void println() throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws IOException.");
        }
        newLine();
    }

    /**
     *  Print a boolean followed by a new line.
     *  @since  Version 0.1 2005-06-29
     */
    public final void println(boolean x) throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws IOException.");
        }
        print(x);
        println();
    }

    /**
     *  Print a character followed by a new line.
     *  @since  Version 0.1 2005-06-29
     */
    public final void println(char x) throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws IOException.");
        }
        print(x);
        println();
    }

    /**
     *  Print an int followed by a new line.
     *  @since  Version 0.1 2005-06-29
     */
    public final void println(int x) throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws IOException.");
        }
        print(x);
        println();
    }

    /**
     *  Print a long followed by a new line.
     *  @since  Version 0.1 2005-06-29
     */
    public final void println(long x) throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws IOException.");
        }
        print(x);
        println();
    }

    /**
     *  Print a float followed by a new line.
     *  @since  Version 0.1 2005-06-29
     */
    public final void println(float x) throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws IOException.");
        }
        print(x);
        println();
    }

    /**
     *  Print a double followed by a new line.
     *  @since  Version 0.1 2005-06-29
     */
    public final void println(double x) throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws IOException.");
        }
        print(x);
        println();
    }

    /**
     *  Print a character array followed by a new line.
     *  @since  Version 0.1 2005-06-29
     */
    public final void println(char[] x) throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws IOException.");
        }
        print(x);
        println();
    }

    /**
     *  Print a string followed by a new line.
     *  @since  Version 0.1 2005-06-29
     */
    public final void println(java.lang.String x) throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws IOException.");
        }
        print(x);
        println();
    }

    /**
     *  Print an object followed by a new line, not implemented.
     *  @since  Version 0.1 2005-06-29
     */
    public final void println(java.lang.Object x) throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws IOException.");
        }
        print(x);
        println();
    }

    /**
     *  Clear, not implemented.
     *  @since  Version 0.1 2005-06-29
     */
    public final void clear() throws IOException {

        buffer = new StringBuffer();
    }

    /**
     *  Clear buffer, not implemented.
     *  @since  Version 0.1 2005-06-29
     */
    public final void clearBuffer() throws IOException {

        // Do nothing
    }

    /**
     *  Flush the output, not implemented.
     *  @since  Version 0.1 2005-06-29
     */
    public final void flush() throws IOException {

        // Do nothing
    }

    /**
     *  Close the output, not properly implemented.
     *  @since  Version 0.1 2005-06-29
     */
    public final void close() throws IOException {

        // Do nothing
    }

    /**
     *  Remaining bytes always zero.
     *  @since  Version 0.1 2005-06-29
     */
    public final int getRemaining() {

        return buffer.capacity();
    }

    /**
     *  Write a character array with offset.
     *  @since  Version 0.1 2005-06-29
     */
    public final void write(char[] c,int offset,int length) throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws an IOException writing a character array with offset.");
        }

        int size = c.length;

        if ((offset > -1) &&
            (offset < size) &&
            (length > -1) &&
            ((offset + length) <= size)) {

            for (int i = offset; i < offset + length; i++) {

                print(c[i]);
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Invalid offset or length. " +
                        "Offset: " + offset + ", " +
                        "length: " + length + ", " +
                        "array size: " + size + ".");
        }

    }

    /**
     *  Write a character array.
     *  @since  Version 0.1 2005-06-29
     */
    public void write(char[] cbuf) throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws an IOException writing a character array with offset.");
        }
        print(cbuf);
    }

    /**
     *  Write an integer.
     *  @since  Version 0.1 2005-06-29
     */
    public void write(int c) throws IOException {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws an IOException writing a character array with offset.");
        }
        print(c);
    }

    /**
     *  Write a string.
     *  @since  Version 0.1 2005-06-29
     */
    public void write(String str) throws IOException  {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws an IOException writing a character array with offset.");
        }
        print(str);
    }

    public void write(String str,
                      int off,
                      int len) throws IOException  {

        if (failOnPrint) {

            throw new IOException("MockJspWriter throws an IOException writing a character array with offset.");
        }

        char[] charArray = str.toCharArray();

        write(charArray, off, len);
    }

}
