package com.mkdoc.jsp;

/*
    Created:    2005-06-29
    Issued:     2005
    Modified:   2005-10-19

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.util.Enumeration;
import java.util.Hashtable;

import javax.servlet.Servlet;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;

import javax.servlet.jsp.el.ExpressionEvaluator;
import javax.servlet.jsp.el.VariableResolver;

/**
 *  A mock page context for testing.
 *  @version    0.2 2005-10-18
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockPageContext extends PageContext {

    /**
     *  A null reference to reset the page context.
     *  @since  Version 0.1 2005-06-29
     */
    protected static final Object NULL = null;

    /**
     *  The servlet request object.
     *  @since  Version 0.1 2005-06-29
     */
    protected ServletRequest request;

    /**
     *  The servlet response object.
     *  @since  Version 0.1 2005-06-29
     */
    protected ServletResponse response;

    /**
     *  A stored exception.
     *  @since  Version 0.1 2005-06-29
     */
    protected Throwable throwable;

    /**
     *  The servlet response path.
     *  @since  Version 0.1 2005-06-29
     */
    protected String path;

    /**
     *  An error page.
     *  @since  Version 0.1 2005-06-29
     */
    protected String errorPage;

    /**
     *  A servlet that represents the whole page.
     *  @since  Version 0.1 2005-06-29
     */
    protected Servlet servlet;

    /**
     *  The servlet configuration object.
     *  @since  Version 0.1 2005-06-29
     */
    protected ServletConfig servletConfig;

    /**
     *  The servlet context object.
     *  @since  Version 0.1 2005-06-29
     */
    protected ServletContext servletContext;

    /**
     *  The servlet context object.
     *  @since  Version 0.1 2005-06-29
     */
    protected HttpSession httpSession;

    /**
     *  The JSP output writer.
     *  @since  Version 0.1 2005-06-29
     */
    protected JspWriter out;

    /**
     *  The variable resolver.
     *  @since  Version 0.1 2005-06-29
     */
    protected VariableResolver resolver;

    /**
     *  The expression evaluator.
     *  @since  Version 0.1 2005-06-29
     */
    protected ExpressionEvaluator evaluator;

    /**
     *  A set of page attributes.
     *  @since  Version 0.1 2005-06-29
     */
    protected Hashtable attributes = new Hashtable();

    /**
     *  Get the servlet response.
     *  @return The servlet response object, or <code>null</code> if not set.
     *  @since  Version 0.1 2005-06-29
     */
    public final ServletResponse getResponse() {

        return response;
    }

    /**
     *  Set the servlet response.
     *  @param  response    The servlet response object.
     *  @since  Version 0.1 2005-06-29
     */
    public final void setResponse(final ServletResponse response) {

        this.response = response;
    }

    /**
     *  Get the servlet request.
     *  @return The servlet request object, or <code>null</code> if not set.
     *  @since  Version 0.1 2005-06-29
     */
    public final ServletRequest getRequest() {

        return request;
    }

    /**
     *  Set the servlet request.
     *  @param  request The servlet request object.
     *  @since  Version 0.1 2005-06-29
     */
    public final void setRequest(final ServletRequest request) {

        this.request = request;
    }

    /**
     *  Forward the servlet request.
     *  @param  urlPath The path to forward the request to.
     *  @since  Version 0.1 2005-06-29
     */
    public final void forward(final String urlPath) {

        path = urlPath;
    }

    /**
     *  Include a servlet resource.
     *  @param  urlPath The path to the resource to include.
     *  @since  Version 0.1 2005-06-29
     */
    public final void include(final String urlPath) {

        path = urlPath;
    }

    /**
     *  Include a servlet resource.
     *  @param  urlPath The path to the resource to include.
     *  @param  flush   Whether to flush the output stream.
     *  @since  Version 0.1 2005-06-29
     */
    public final void include(final String urlPath,
                              final boolean flush) {

        path = urlPath;
    }

    /**
     *  Get a stored exception.
     *  @return Any exception associated with the context.
     *  @since  Version 0.1 2005-06-29
     */
    public final Exception getException() {

        return (Exception) throwable;
    }

    /**
     *  Get a servlet representation of the page.
     *  @return The JSP servlet object.
     *  @since  Version 0.1 2005-06-29
     */
    public final Object getPage() {

        return (Object) servlet;
    }

    /**
     *  Get the servlet configuration.
     *  @return The servlet configuration.
     *  @since  Version 0.1 2005-06-29
     */
    public final ServletConfig getServletConfig() {

        return servletConfig;
    }

    /**
     *  Get the servlet context.
     *  @return The servlet context.
     *  @since  Version 0.1 2005-06-29
     */
    public final ServletContext getServletContext() {

        return servletContext;
    }

    /**
     *  Set the servlet context.
     *  @param  servletContext  The servlet context for the page context.
     *  @since  Version 0.2 2005-10-18
     */
    public final void setServletContext(final ServletContext servletContext) {

        this.servletContext = servletContext;
    }

    /**
     *  Get the HTTP session reference.
     *  @return Any HTTP session associated with the context.
     *  @since  Version 0.1 2005-06-29
     */
    public final HttpSession getSession() {

        if (request == null) {

            return null;
        }
        else {

            try {

                HttpServletRequest httpRequest = (HttpServletRequest) request;

                return httpRequest.getSession();
            }
            catch (ClassCastException cce) {

                return null;
            }
        }
    }

    /**
     *  Handle a page exception.
     *  @param  exception   The page exception to handle.
     *  @since  Version 0.1 2005-06-29
     */
    public final void handlePageException(final Exception exception) {

        throwable = exception;
    }

    /**
     *  Handle a page exception.
     *  @param  throwable   The page exception to handle.
     *  @since  Version 0.1 2005-06-29
     */
    public final void handlePageException(final Throwable throwable) {

        this.throwable = throwable;
    }

    /**
     *  Initialise the page context.
     *  @param  servlet The JSP servlet instance.
     *  @param  request The servlet request object.
     *  @param  response    The servlet response object.
     *  @param  errorPageURL    The error page for the JSP.
     *  @param  needsSession    Whether a session is mandatory.
     *  @param  bufferSize  The size of the buffer for the page.
     *  @param  autoFlush   Whether to automatically flush the output.
     *  @since  Version 0.1 2005-06-29
     */
    public final void initialize(final Servlet servlet,
                                 final ServletRequest request,
                                 final ServletResponse response,
                                 final String errorPageURL,
                                 final boolean needsSession,
                                 final int bufferSize,
                                 final boolean autoFlush) {

        this.servlet = servlet;
        this.request = request;
        this.response = response;
        this.errorPage = errorPage;
    }

    /**
     *  Release the page context and reset the initial field values.
     *  @since  Version 0.1 2005-06-29
     */
    public final void release() {

        this.servlet = (Servlet) NULL;
        this.request = (ServletRequest) NULL;
        this.response = (ServletResponse) NULL;
        this.errorPage = (String) NULL;

        throwable = (Throwable) NULL;
        path = (String) NULL;
        out = (JspWriter) NULL;
    }

    /**
     *  Get the JSP output stream.
     *  @return The output stream for the JSP.
     *  @since  Version 0.1 2005-06-29
     */
    public final JspWriter getOut() {

        return out;
    }

    /**
     *  Set the JSP output stream.
     *  @param  out A JSP writer to assign to the context.
     *  @since  Version 0.1 2005-06-29
     */
    public final void setOut(final JspWriter out) {

        this.out = out;
    }

    /**
     *  Get the variable resolver.
     *  @return A variable resolver for the page context.
     *  @since  Version 0.1 2005-06-29
     */
    public final VariableResolver getVariableResolver() {

        return resolver;
    }

    /**
     *  Get the expression evaluator.
     *  @return The expression evaluator for the page context.
     *  @since  Version 0.1 2005-06-29
     */
    public final ExpressionEvaluator getExpressionEvaluator() {

        return evaluator;
    }

    /**
     *  Get the scope of an attribute.
     *  @param  name    The name of the attribute to check.
     *  @return This implementation always returns 1.
     *  @since  Version 0.1 2005-06-29
     */
    public final int getAttributesScope(final String name) {

        return 1;
    }

    /**
     *  Get the attribute names in scope.
     *  @param  scope   The scope to check for attributes.
     *  @return This implementation returns all attribute names.
     *  @since  Version 0.1 2005-06-29
     */
    public final Enumeration getAttributeNamesInScope(final int scope) {

        synchronized (attributes) {

            return attributes.keys();
        }
    }

    /**
     *  Remove an attribute.
     *  @param  name    The name of the attribute to remove.
     *  @param  scope   The scope from which to remove the attribute.
     *  @since  Version 0.1 2005-06-29
     */
    public final void removeAttribute(final String name,
                                      final int scope) {

        synchronized (attributes) {

            attributes.remove(name);
        }
    }

    /**
     *  Remove an attribute.
     *  @param  name    The attribute to remove.
     *  @since  Version 0.1 2005-06-29
     */
    public final void removeAttribute(final String name) {

        removeAttribute(name, 1);
    }

    /**
     *  Find an attribute by name.
     *  @param  name    The attribute name to find.
     *  @return The object reference for the attribute.
     *  @since  Version 0.1 2005-06-29
     */
    public final Object findAttribute(final String name) {

        synchronized (attributes) {

            return attributes.get(name);
        }
    }

    /**
     *  Get an attribute by name and scope.
     *  @param  name    The name of the attribute to get.
     *  @param  scope   The scope within which to look-up the attribute.
     *  @return The object reference for the attribute.
     *  @since  Version 0.1 2005-06-29
     */
    public final Object getAttribute(final String name,
                                     final int scope) {

        synchronized (attributes) {

            return attributes.get(name);
        }
    }

    /**
     *  Get an attribute by name.
     *  @param  name    The name of the attribute to get.
     *  @return The object reference for the attribute.
     *  @since  Version 0.1 2005-06-29
     */
    public final Object getAttribute(final String name) {

        return getAttribute(name, 1);
    }

    /**
     *  Set an attribute by name and scope.
     *  @param  name    The name of the attribute to set.
     *  @param  attribute   The value to assign to the attribute.
     *  @param  scope   The scope to assign the attribute.
     *  @since  Version 0.1 2005-06-29
     */
    public final void setAttribute(final String name,
                                   final Object attribute,
                                   final int scope) {

        synchronized (attributes) {

            attributes.put(name, attribute);
        }
    }

    /**
     *  Set an attribute by name.
     *  @param  name    The name of the attribute to set.
     *  @param  attribute   The value to assign to the attribute.
     *  @since  Version 0.1 2005-06-29
     */
    public final void setAttribute(final String name,
                                   final Object attribute) {

        setAttribute(name, attribute, 1);
    }

}
