package com.mkdoc.jspider;

/*
    Created:    2005-05-31
    Issued:     2005-05-31
    Modified:   2005-11-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.store.MockStoreManager;

import com.mkdoc.util.ApplicationContext;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>AbstractStoreManagerPlugin</code>.
 *  @version    0.3 2005-11-01
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class AbstractStoreManagerPluginTest extends TestCase {

    /**
     *  A mock plugin instance to test.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-05-31
     */
    protected MockStoreManagerPlugin mockPlugin;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-
     */
    public AbstractStoreManagerPluginTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-05-31
     */
    protected final void setUp() {

        mockPlugin = new MockStoreManagerPlugin();
    }

    /**
     *  Check shutdown shuts down the subject manager.
     *  @since  Version 0.1 2005-05-31
     */
    public final void testShutdown() {

        MockStoreManager testManager = new MockStoreManager();

        MockPropertySet propertySet = new MockPropertySet();

        ApplicationContext context = null;

        context = AbstractMKDocPlugin.getApplicationContext(propertySet);

        testManager.initialize(context);

        mockPlugin.manager = testManager;

        assertFalse(testManager.isShutdown());

        mockPlugin.shutdown();

        assertTrue(testManager.isShutdown());
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-05-31
     */
    public static Test suite() {

        return new TestSuite(AbstractStoreManagerPluginTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-05-31
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
