package com.mkdoc.jspider;

/*
    Created:    2004-11-26
    Issued:     2004-11-26
    Modified:   2005-10-21

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.api.event.resource.ResourceFetchedEvent;
import net.javacoding.jspider.api.event.resource.ResourceDiscoveredEvent;

import com.mkdoc.store.AbstractFileStoreManager;
import com.mkdoc.store.MockVirtualFileStoreManager;
import com.mkdoc.store.Manager;
import com.mkdoc.store.StoreManagerFactory;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;

import java.net.URL;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>JTidyFileWriterPlugin</code>.
 *  @version    0.8 2005-10-21
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class JTidyFileWriterPluginTest extends TestCase {

    /**
     *  An expected <code>IllegalArgumentException</code> message.
     *  @since  Version 0.1 2004-11-26
     */
    private static final String EXPECTED_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  The store manager class name.
     *  @since  Version 0.1 2004-11-26
     */
    protected static final String CLASS_NAME =
    "com.mkdoc.store.HtmlFileStoreManager";

    /**
     *  The project source directory.
     *  @since  Version 0.1 2004-11-26
     */
    protected static final String HOME = System.getProperty("mkHome");

    /**
     *  The project test source directory.
     *  @since  Version 0.1 2004-11-26
     */
    protected static final String TEST = System.getProperty("testDir") +
                                         File.separator +
                                         "html";

    /**
     *  The default JSpider output directory.
     *  @since  Version 0.1 2004-11-26
     */
    protected static final String OUTPUT = "output";

    /**
     *  A custom output directory.
     *  @since  Version 0.2 2004-12-03
     */
    protected static final String CUSTOM = "custom";

    /**
     *  The test source directory path.
     *  @since  Version 0.1 2004-11-26
     */
    protected static final String PATH_TEST = HOME +
                                              File.separator +
                                              TEST;

    /**
     *  An invalid source file for testing.
     *  @since  Version 0.1 2004-11-26
     */
    protected static final String INVALID = "invalid-Basic.html";

    /**
     *  A file reference for invalid source for testing.
     *  @since  Version 0.1 2004-11-26
     */
    protected static final File SRC_INVALID = new File(PATH_TEST +
                                                       File.separator +
                                                       INVALID);

    /**
     *  An valid source file for testing.
     *  @since  Version 0.1 2004-11-26
     */
    protected static final String VALID = "valid-Basic.html";

    /**
     *  A file reference for valid source for testing.
     *  @since  Version 0.1 2004-11-26
     */
    protected static final File SRC_VALID = new File(PATH_TEST +
                                                     File.separator +
                                                     VALID);

    /**
     *  The default JSpider output path.
     *  @since  Version 0.1 2004-11-26
     */
    protected static final String PATH_OUT = HOME +
                                             File.separator +
                                             OUTPUT;


    /**
     *  The custom output path.
     *  @since  Version 0.2 2004-12-03
     */
    protected static final String PATH_CUSTOM = HOME +
                                                File.separator +
                                                OUTPUT +
                                                File.separator +
                                                CUSTOM;

    /**
     *  A <code>File</code> representation of the default
     *  output path for comparisons.
     *  @since  Version 0.1 2004-11-26
     */
    protected static final File DIR_OUT = new File(PATH_OUT);

    /**
     *  A <code>File</code> representation of the home
     *  directory for comparisons.
     *  @since  Version 0.1 2004-11-26
     */
    protected static final File DIR_HOME = new File(HOME);

    /**
     *  A <code>File</code> representation of the custom
     *  output directory for comparisons.
     *  @since  Version 0.2 2004-12-03
     */
    protected static final File DIR_CUSTOM = new File(PATH_CUSTOM);

    /**
     *  A test domain.
     *  @since  Version 0.2 2004-12-03
     */
    protected static final String DOMAIN = System.getProperty("testDomain");


    /**
     *  A test domain URL.
     *  @since  Version 0.2 2004-12-03
     */
    protected static final String URL_DOMAIN = "http://" + DOMAIN;

    /**
     *  An expected output directory for the test domain.
     *  @since  Version 0.2 2004-12-03
     */
    protected static final String PATH_DOMAIN = PATH_CUSTOM +
                                                File.separator +
                                                DOMAIN;

    /**
     *  A File reference for the expected output directory
     *  for the test domain.
     *  @since  Version 0.2 2004-12-03
     */
    protected static final File DIR_DOMAIN = new File(PATH_DOMAIN);

    /**
     *  A test domain root directory URL.
     *  @since  Version 0.2 2004-12-03
     */
    protected static final String URL_ROOT = URL_DOMAIN + "/";

    /**
     *  A root index file.
     *  @since  Version 0.2 2004-12-03
     */
    protected static final String INDEX = "index.html";

    /**
     *  A test domain root index URL.
     *  @since  Version 0.2 2004-12-03
     */
    protected static final String ROOT_INDEX = URL_ROOT + INDEX;

    /**
     *  A valid source URL.
     *  @since  Version 0.2 2004-12-03
     */
    protected static final String URL_VALID = URL_ROOT + VALID;

    /**
     *  An invalid source URL.
     *  @since  Version 0.2 2004-12-03
     */
    protected static final String URL_INVALID = URL_ROOT + INVALID;


    /**
     *  A file reference for the actual root index.
     *  @since  Version 0.2 2004-12-03
     */
    protected static final File FILE_INDEX = new File(PATH_DOMAIN +
                                                           File.separator +
                                                           INDEX);

    /**
     *  A file reference for invalid output file.
     *  @since  Version 0.2 2004-12-03
     */
    protected static final File OUT_INVALID = new File(PATH_DOMAIN +
                                                            File.separator +
                                                            INVALID);

    /**
     *  A file reference for valid output file.
     *  @since  Version 0.2 2004-12-03
     */
    protected static final File OUT_VALID = new File(PATH_DOMAIN +
                                                          File.separator +
                                                          VALID);

    /**
     *  A pre-configured plugin to use in multiple tests.
     *  @since  Version 0.2 2004-12-03
     */
    protected JTidyFileWriterPlugin commonPlugin;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2004-
     */
    public JTidyFileWriterPluginTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @exception  ClassNotFoundException  Should not be thrown.
     *  @since  Version 0.1 2004-
     */
    protected final void setUp() throws ClassNotFoundException {

        MockPropertySet propertySet = new MockPropertySet();

        Class managerClass = Class.forName(CLASS_NAME);

        propertySet.setClass(StoreManagerFactory.MANAGER_KEY,
                             managerClass);

        propertySet.setBoolean(AbstractFileStoreManager.ABSOLUTE, true);
        propertySet.setString(AbstractFileStoreManager.FOLDER, PATH_CUSTOM);

        commonPlugin = new JTidyFileWriterPlugin(propertySet);
    }

    /**
     *  Constructor should not accept a null argument.
     *  @since  Version 0.1 2004-11-26
     */
    public final void testConstructorNullPropertySet() {

        try {

            new JTidyFileWriterPlugin(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Constructor should log an error with no manager configured.
     *  @since  Version 0.1 2004-11-26
     */
    public final void testConstructorNoManagerConfigured() {

        JTidyFileWriterPlugin plugin = null;

        plugin = new JTidyFileWriterPlugin(new MockPropertySet());

        Manager testManager = plugin.manager;

        assertNull(testManager);
    }

    /**
     *  With null event should throw an
     *  IllegalArgumentException.
     *  @since  Version 0.1 2004-11-26
     */
    public final void testNotifyNullEvent() {

        ResourceDiscoveredEvent nullEvent = null;

        try {

            commonPlugin.notify(nullEvent);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  With wrong type should ignore.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2004-11-26
     */
    public final void testNotifyWrongType()
        throws IOException {

        MockFetchedResource resource = new MockFetchedResource();

        resource.setUrl(new URL(URL_DOMAIN));

        InputStream fileInStream = new FileInputStream(SRC_INVALID);

        resource.setInputStream(fileInStream);

        ResourceDiscoveredEvent rde = new ResourceDiscoveredEvent(resource);

        commonPlugin.notify(rde);

        assertFalse("Should not output file to path for this event type'" +
                    FILE_INDEX.getCanonicalPath() + "'",
                    FILE_INDEX.exists());

    }

    /**
     *  Should use the default <code>index.html</code> name
     *  for the output file.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2004-11-26
     */
    public final void testNotifyResourceNoFile()
        throws IOException {

        MockFetchedResource resource = new MockFetchedResource();

        resource.setUrl(new URL(URL_DOMAIN));

        InputStream fileInStream = new FileInputStream(SRC_INVALID);

        resource.setInputStream(fileInStream);

        ResourceFetchedEvent rfe = new ResourceFetchedEvent(resource);

        commonPlugin.notify(rfe);

        assertTrue("Should output file to path '" +
                   FILE_INDEX.getCanonicalPath() + "'",
                   FILE_INDEX.exists());
    }

    /**
     *  Should use the file part of the URL for the output file name.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2004-11-26
     */
    public final void testNotifyResourceWithFile()
        throws IOException {

        MockFetchedResource resource = new MockFetchedResource();

        resource.setUrl(new URL(URL_VALID));

        InputStream fileInStream = new FileInputStream(SRC_INVALID);

        resource.setInputStream(fileInStream);

        ResourceFetchedEvent rfe = new ResourceFetchedEvent(resource);

        commonPlugin.notify(rfe);

        assertTrue("Should output file to path '" +
                   OUT_VALID.getCanonicalPath() + "'",
                   OUT_VALID.exists());
    }

    /**
     *  Test parse with a null input stream.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.6 2005-05-10
     */
    public final void testParseNullInputStream() throws IOException {

        try {

            commonPlugin.parse(null, new URL(URL_VALID));

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test parse with a null URL.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.6 2005-05-10
     */
    public final void testParseNullUrl() throws IOException {

        try {

            InputStream fileInStream = new FileInputStream(SRC_INVALID);

            commonPlugin.parse(fileInStream, null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test parse throws IOException.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.6 2005-05-10
     */
    public final void testParseThrowsIOException() throws IOException {

        MockVirtualFileStoreManager badManager = null;

        badManager = new MockVirtualFileStoreManager();

        badManager.setFailOnGetOutput(true);

        commonPlugin.manager = badManager;

        MockLog log = new MockLog();

        commonPlugin.setLog(log);

        InputStream fileInStream =  null;

        try {

            fileInStream = new FileInputStream(SRC_INVALID);

            commonPlugin.parse(fileInStream, new URL(URL_VALID));

            String message = log.getError();

            assertTrue("Should log the error, actual log was '" +
                       message + "'.",
                       message.indexOf("Could not write output") > -1);
        }
        finally {

            fileInStream.close();
        }
    }

    /**
     *  Get description, no check for literal content.
     *  @since  Version 0.1 2004-11-26
     */
    public final void testGetDescription() {

        String description = commonPlugin.getDescription();

        assertTrue((description != null) &&
                   (!description.equals("")));
    }

    /**
     *  Get version, no check for literal content.
     *  @since  Version 0.1 2004-11-26
     */
    public final void testGetVersion() {

        String version = commonPlugin.getVersion();

        assertTrue((version != null) &&
                   (!version.equals("")));
    }

    /**
     *  Get Name.
     *  @since  Version 0.1 2004-11-26
     */
    public final void testGetName() {

        String name = commonPlugin.getName();

        assertTrue((name != null) &&
                   (name.equals("JTidy file writer plugin for JSpider")));
    }

    /**
     *  Tear down a test scenario as required.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2004-12-03
     */
    protected final void tearDown() throws IOException {

        if (FILE_INDEX.exists()) {

            assertTrue("Could not delete root index file '" +
                       FILE_INDEX.getCanonicalPath() + "'",
                       FILE_INDEX.delete());
        }
        if (OUT_VALID.exists()) {

            assertTrue("Could not delete valid output file '" +
                       OUT_VALID.getCanonicalPath() + "'",
                       OUT_VALID.delete());
        }
        if (OUT_INVALID.exists()) {

            assertTrue("Could not delete invalid output file '" +
                       OUT_INVALID.getCanonicalPath() + "'",
                       OUT_INVALID.delete());
        }
        if (DIR_DOMAIN.exists()) {

            assertTrue("Could not delete the test domain output directory '" +
                       DIR_DOMAIN.getCanonicalPath() + "'",
                       DIR_DOMAIN.delete());
        }
        if (DIR_CUSTOM.exists()) {

            assertTrue("Could not delete the custom output directory '" +
                       DIR_CUSTOM.getCanonicalPath() + "'",
                       DIR_CUSTOM.delete());
        }
    }

    /**
     *  Create a test suite for this test case.
     *  @return A test coverage decorator.
     *  @since  Version 0.1 2004-
     */
    public static Test suite() {

        return new TestSuite(JTidyFileWriterPluginTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2004-
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
