package com.mkdoc.jspider;

/*
    Created:    2004-12-17
    Issued:     2004-12-17
    Modified:   2005-11-02

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.api.event.resource.ResourceFetchedEvent;
import net.javacoding.jspider.api.event.resource.ResourceDiscoveredEvent;

import com.mkdoc.sax.RdfContentHandler;

import com.mkdoc.schema.UKeGMS;
import com.mkdoc.schema.Schema;
import com.mkdoc.schema.UKeGMSProfile;

import com.mkdoc.store.MockFileStoreManager;
import com.mkdoc.store.MockVirtualFileStoreManager;
import com.mkdoc.store.AbstractFileStoreManager;
import com.mkdoc.store.StoreManagerFactory;

import com.mkdoc.util.ApplicationContext;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;

import java.net.URL;

import org.xml.sax.ContentHandler;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>MetaTripleWriterPlugin</code>.
 *  @version    1.1 2005-11-02
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MetaTripleWriterPluginTest extends TestCase {

    /**
     *  An expected <code>IllegalArgumentException</code> message.
     *  @since  Version 0.6 2005-05-10
     */
    private static final String EXPECTED_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  The store manager class name.
     *  @since  Version 0.5 2005-04-29
     */
    protected static final String CLASS_NAME =
    "com.mkdoc.store.TripleFileStoreManager";

    /**
     *  The project source directory.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String HOME = System.getProperty("mkHome");

    /**
     *  The project test source directory.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String TEST = System.getProperty("testDir") +
                                         File.separator +
                                         "html";

    /**
     *  The application profile property key name.
     *  @since  Version 0.7 2005-05-26
     */
    public static final String PROFILE = "profile";

    /**
     *  The default JSpider output directory.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String OUTPUT = "output";

    /**
     *  A custom output directory.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String CUSTOM = "custom";

    /**
     *  The test source directory path.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String PATH_TEST = HOME +
                                              File.separator +
                                              TEST;

    /**
     *  An invalid source file for testing.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String INVALID = "invalid-Basic.html";

    /**
     *  A file reference for invalid source for testing.
     *  @since  Version 0.5 2005-04-29
     */
    protected static final File SRC_INVALID = new File(PATH_TEST +
                                                       File.separator +
                                                       INVALID);

    /**
     *  An irreparably invalid source file for testing.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String BROKEN = "invalid-Irreparable.html";

    /**
     *  An invalid output file for testing.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String INVALID_NT = "invalid-Basic.nt";

    /**
     *  An irreparably invalid output file for testing.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String BROKEN_NT = "invalid-Irreparable.nt";

    /**
     *  A file reference for an irreparably invalid source for testing.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final File SRC_BROKEN = new File(PATH_TEST +
                                                      File.separator +
                                                      BROKEN);

    /**
     *  An valid source file for testing.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String VALID = "valid-Basic.html";

    /**
     *  An valid output file for testing.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String VALID_NT = "valid-Basic.nt";


    /**
     *  A file reference for valid source for testing.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final File SRC_VALID = new File(PATH_TEST +
                                                     File.separator +
                                                     VALID);

    /**
     *  The default JSpider output path.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String PATH_OUT = HOME +
                                             File.separator +
                                             OUTPUT;


    /**
     *  The custom output path.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String PATH_CUSTOM = HOME +
                                                File.separator +
                                                OUTPUT +
                                                File.separator +
                                                CUSTOM;

    /**
     *  A <code>File</code> representation of the default
     *  output path for comparisons.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final File DIR_OUT = new File(PATH_OUT);

    /**
     *  A <code>File</code> representation of the home
     *  directory for comparisons.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final File DIR_HOME = new File(HOME);

    /**
     *  A <code>File</code> representation of the custom
     *  output directory for comparisons.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final File DIR_CUSTOM = new File(PATH_CUSTOM);

    /**
     *  A test domain.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String DOMAIN = System.getProperty("testDomain");


    /**
     *  A test domain URL.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String URL_DOMAIN = "http://" + DOMAIN;

    /**
     *  An expected output directory for the test domain.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String PATH_DOMAIN = PATH_CUSTOM +
                                                File.separator +
                                                DOMAIN;

    /**
     *  A File reference for the expected output directory
     *  for the test domain.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final File DIR_DOMAIN = new File(PATH_DOMAIN);

    /**
     *  A test domain root directory URL.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String U_ROOT = URL_DOMAIN + "/";

    /**
     *  A root index source file.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String INDEX = "index.html";

    /**
     *  A root index output file.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String INDEX_NT = "index.nt";

    /**
     *  A test domain root index URL.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String U_ROOT_IDX = U_ROOT + INDEX;

    /**
     *  A valid source URL.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String URL_VALID = U_ROOT + VALID;

    /**
     *  An invalid source URL.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String URL_INVALID = U_ROOT + INVALID;


    /**
     *  An irreparably invalid source URL.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String URL_BROKEN = U_ROOT + BROKEN;

    /**
     *  A file reference for the actual root index.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final File ROOT_INDEX = new File(PATH_DOMAIN +
                                                      File.separator +
                                                      INDEX_NT);

    /**
     *  A file reference for invalid output file.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final File OUT_INVALID = new File(PATH_DOMAIN +
                                                       File.separator +
                                                       INVALID_NT);

    /**
     *  A file reference for invalid output file.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final File OUT_BROKEN = new File(PATH_DOMAIN +
                                                      File.separator +
                                                      BROKEN_NT);

    /**
     *  A file reference for valid output file.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final File OUT_VALID = new File(PATH_DOMAIN +
                                                          File.separator +
                                                          VALID_NT);

    /**
     *  A pre-configured plugin to use in multiple tests.
     *  @since  Version 0.1 2004-12-17
     */
    protected MetaTripleWriterPlugin commonPlugin;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2004-
     */
    public MetaTripleWriterPluginTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @exception  ClassNotFoundException  Should not be thrown.
     *  @since  Version 0.1 2004-
     */
    protected final void setUp() throws ClassNotFoundException {

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setClass(StoreManagerFactory.MANAGER_KEY,
                             Class.forName(CLASS_NAME));

        propertySet.setBoolean(AbstractFileStoreManager.ABSOLUTE, true);
        propertySet.setString(AbstractFileStoreManager.FOLDER, PATH_CUSTOM);

        commonPlugin = new MetaTripleWriterPlugin(propertySet);

    }

    /**
     *  Test get content handler with profile loaded.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  ClassNotFoundException Should not be thrown by this test.
     *  @exception  InstantiationException Should not be thrown by this test.
     *  @exception  IllegalAccessException Should not be thrown by this test.
     *  @since  Version 0.7 2005-05-26
     */
    public final void testGetContentHandlerProfileLoaded()
        throws IOException, ClassNotFoundException,
        InstantiationException, IllegalAccessException {

        String eGMS = "com.mkdoc.schema.UKeGMSProfile";

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setClass(PROFILE, Class.forName(eGMS));

        propertySet.setClass(StoreManagerFactory.MANAGER_KEY,
                             Class.forName(CLASS_NAME));

        propertySet.setBoolean(AbstractFileStoreManager.ABSOLUTE, true);
        propertySet.setString(AbstractFileStoreManager.FOLDER, PATH_CUSTOM);


        ApplicationContext context = null;

        context = commonPlugin.getApplicationContext(propertySet);

        commonPlugin.loadApplicationProfile(context);

        MockFileStoreManager manager = new MockFileStoreManager();

        URL url = new URL(U_ROOT_IDX);

        ContentHandler handler = commonPlugin.getContentHandler(manager,
                                                                url);

        RdfContentHandler rdfHandler = (RdfContentHandler) handler;

        String predicate = "eGMS.Addressee";

        String expanded = rdfHandler.expandPredicate(predicate);

        Schema schema = new UKeGMS();

        assertTrue(expanded.equals(schema.expandPredicate(predicate)));

    }

    /**
     *  Test get content handler without profile loaded.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  ClassNotFoundException Should not be thrown by this test.
     *  @exception  InstantiationException Should not be thrown by this test.
     *  @exception  IllegalAccessException Should not be thrown by this test.
     *  @since  Version 0.7 2005-05-26
     */
    public final void testGetContentHandlerProfileNotLoaded()
        throws IOException, ClassNotFoundException,
        InstantiationException, IllegalAccessException {


        MockFileStoreManager manager = new MockFileStoreManager();

        URL url = new URL(U_ROOT_IDX);

        ContentHandler handler = commonPlugin.getContentHandler(manager,
                                                                url);

        RdfContentHandler rdfHandler = (RdfContentHandler) handler;

        String predicate = "eGMS.Addressee";

        String expanded = rdfHandler.expandPredicate(predicate);

        Schema schema = new UKeGMS();

        assertFalse(expanded.equals(schema.expandPredicate(predicate)));

        assertTrue(expanded.equals(predicate));

    }

    /**
     *  Test get content handler, null subject manager.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.7 2005-05-26
     */
    public final void testGetContentHandlerNullSubjectManager()
        throws IOException {

        try {

            commonPlugin.getContentHandler(
                                        null,
                                        new URL("http://www.remote.example/"));

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test get content handler, null URL.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.7 2005-05-26
     */
    public final void testGetContentHandlerNullUrl()
        throws IOException {

        try {

            commonPlugin.getContentHandler(new MockFileStoreManager(),
                                           null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }


    /**
     *  Constructor should not accept a null argument.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testConstructorNullPropertySet() {

        try {

            new MetaTripleWriterPlugin(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test constructor with custom profile exception.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @exception  ClassNotFoundException  Should not be thrown.
     *  @since  Version 0.3 2005-04-11
     */
    public final void testConstructorCustomProfileException()
        throws IOException, ClassNotFoundException {

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setBoolean(AbstractFileStoreManager.ABSOLUTE, false);
        propertySet.setString(AbstractFileStoreManager.FOLDER, CUSTOM);
        propertySet.setClass(PROFILE, Class.forName("java.lang.String"));

        propertySet.setClass(StoreManagerFactory.MANAGER_KEY,
                             Class.forName(CLASS_NAME));

        MetaTripleWriterPlugin plugin = null;

        plugin = new MetaTripleWriterPlugin(propertySet);

        assertTrue((plugin != null) &&
                   (plugin.profile == null));
    }

    /**
     *  Test constructor with no custom profile exception.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @exception  ClassNotFoundException  Should not be thrown.
     *  @since  Version 0.7 2005-05-26
     */
    public final void testConstructorCustomProfileSuccess()
        throws IOException, ClassNotFoundException {

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setBoolean(AbstractFileStoreManager.ABSOLUTE, false);
        propertySet.setString(AbstractFileStoreManager.FOLDER, CUSTOM);
        propertySet.setClass(PROFILE,
                             Class.forName("com.mkdoc.schema.UKeGMSProfile"));

        propertySet.setClass(StoreManagerFactory.MANAGER_KEY,
                             Class.forName(CLASS_NAME));

        MetaTripleWriterPlugin plugin = null;

        plugin = new MetaTripleWriterPlugin(propertySet);

        assertTrue((plugin != null) &&
                   (plugin.profile != null) &&
                   (plugin.profile instanceof UKeGMSProfile));
    }

    /**
     *  Test constructor with no custom profile exception.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @exception  ClassNotFoundException  Should not be thrown.
     *  @since  Version 1.1 2005-11-02
     */
    public final void testConstructorNoStoreManager()
        throws IOException, ClassNotFoundException {

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setBoolean(AbstractFileStoreManager.ABSOLUTE, false);
        propertySet.setString(AbstractFileStoreManager.FOLDER, CUSTOM);
        propertySet.setClass(PROFILE,
                             Class.forName("com.mkdoc.schema.UKeGMSProfile"));

        MetaTripleWriterPlugin plugin = null;

        plugin = new MetaTripleWriterPlugin(propertySet);

        assertNull(plugin.manager);

        plugin.shutdown();
    }

    /**
     *  With null event should throw an
     *  IllegalArgumentException.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testNotifyNullEvent() {

        ResourceDiscoveredEvent nullEvent = null;

        try {

            commonPlugin.notify(nullEvent);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  With wrong type should ignore.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testNotifyWrongType()
        throws IOException {

        MockFetchedResource resource = new MockFetchedResource();

        resource.setUrl(new URL(URL_DOMAIN));

        InputStream fileInStream = new FileInputStream(SRC_INVALID);

        resource.setInputStream(fileInStream);

        ResourceDiscoveredEvent rde = new ResourceDiscoveredEvent(resource);

        commonPlugin.notify(rde);

        assertFalse("Should not output file to path for this event type'" +
                    ROOT_INDEX.getCanonicalPath() + "'",
                    ROOT_INDEX.exists());

    }

    /**
     *  Should use the default <code>index.html</code> name
     *  for the output file.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testNotifyResourceNoFile()
        throws IOException {

        MockFetchedResource resource = new MockFetchedResource();

        resource.setUrl(new URL(URL_DOMAIN));

        InputStream fileInStream = new FileInputStream(SRC_INVALID);

        resource.setInputStream(fileInStream);

        ResourceFetchedEvent rfe = new ResourceFetchedEvent(resource);

        commonPlugin.notify(rfe);

        assertTrue("Should output file to path '" +
                   ROOT_INDEX.getCanonicalPath() + "'",
                   ROOT_INDEX.exists());

    }

    /**
     *  Should use the file part of the URL for the output file name.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testNotifyResourceWithFile()
        throws IOException {

        MockFetchedResource resource = new MockFetchedResource();

        resource.setUrl(new URL(URL_VALID));

        InputStream fileInStream = new FileInputStream(SRC_INVALID);

        resource.setInputStream(fileInStream);

        ResourceFetchedEvent rfe = new ResourceFetchedEvent(resource);

        commonPlugin.notify(rfe);

        assertTrue("Should output file to path '" +
                   OUT_VALID.getCanonicalPath() + "'",
                   OUT_VALID.exists());
    }

    /**
     *  Test parse with a null input stream.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.6 2005-05-10
     */
    public final void testParseNullInputStream() throws IOException {

        try {

            commonPlugin.parse(null, new URL(URL_VALID));

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test parse with a null URL.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.6 2005-05-10
     */
    public final void testParseNullUrl() throws IOException {

        try {

            InputStream fileInStream = new FileInputStream(SRC_INVALID);

            commonPlugin.parse(fileInStream, null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test parse throws IOException.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.6 2005-05-10
     */
    public final void testParseThrowsIOException() throws IOException {

        MockVirtualFileStoreManager badManager = null;

        badManager = new MockVirtualFileStoreManager();

        badManager.setFailOnGetOutput(true);

        commonPlugin.manager = badManager;

        MockLog log = new MockLog();

        commonPlugin.setLog(log);

        InputStream fileInStream =  null;

        try {

            fileInStream = new FileInputStream(SRC_INVALID);

            commonPlugin.parse(fileInStream, new URL(URL_VALID));

            String message = log.getError();

            assertTrue("Should log the error, actual log was '" +
                       message + "'.",
                       message.indexOf("Could not write output") > -1);
        }
        finally {

            fileInStream.close();
        }
    }

    /**
     *  Test parse with shutdown manager.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 1.1 2005-11-02
     */
    public final void testParseManagerShutdown() throws IOException {

        MockVirtualFileStoreManager manager = null;

        manager = new MockVirtualFileStoreManager();

        commonPlugin.manager = manager;

        MockLog log = new MockLog();

        commonPlugin.setLog(log);

        commonPlugin.shutdown();

        InputStream fileInStream =  null;

        try {

            fileInStream = new FileInputStream(SRC_VALID);

            commonPlugin.parse(fileInStream, new URL(URL_VALID));

            String message = log.getError();

            assertTrue("Should log the error, actual log was '" +
                       message + "'.",
                       message.indexOf("Could not write output") > -1);
        }
        finally {

            fileInStream.close();
        }
    }

    /**
     *  Get description, no check for literal content.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testGetDescription() {

        String description = commonPlugin.getDescription();

        assertTrue((description != null) &&
                   (!description.equals("")));

    }

    /**
     *  Get version, no check for literal content.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testGetVersion() {

        String version = commonPlugin.getVersion();

        assertTrue((version != null) &&
                   (!version.equals("")));
    }

    /**
     *  Get Name.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testGetName() {

        String name = commonPlugin.getName();

        assertTrue((name != null) &&
                   (name.equals(
                    "XHTML meta triple writer plugin for JSpider")));
    }

    /**
     *  Tear down a test scenario as required.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2004-12-03
     */
    protected final void tearDown() throws IOException {

        if (ROOT_INDEX.exists()) {

            assertTrue("Could not delete root index file '" +
                       ROOT_INDEX.getCanonicalPath() + "'",
                       ROOT_INDEX.delete());
        }
        if (OUT_VALID.exists()) {

            assertTrue("Could not delete valid output file '" +
                       OUT_VALID.getCanonicalPath() + "'",
                       OUT_VALID.delete());
        }
        if (OUT_INVALID.exists()) {

            assertTrue("Could not delete invalid output file '" +
                       OUT_INVALID.getCanonicalPath() + "'",
                       OUT_INVALID.delete());
        }
        if (OUT_BROKEN.exists()) {

            assertTrue("Could not delete irreparable output file '" +
                       OUT_BROKEN.getCanonicalPath() + "'",
                       OUT_BROKEN.delete());
        }
        if (DIR_DOMAIN.exists()) {

            assertTrue("Could not delete the test domain output directory '" +
                       DIR_DOMAIN.getCanonicalPath() + "'",
                       DIR_DOMAIN.delete());
        }
        if (DIR_CUSTOM.exists()) {

            assertTrue("Could not delete the custom output directory '" +
                       DIR_CUSTOM.getCanonicalPath() + "'",
                       DIR_CUSTOM.delete());
        }
    }

    /**
     *  Create a test suite for this test case.
     *  @return A test coverage decorator.
     *  @since  Version 0.1 2004-12-17
     */
    public static Test suite() {

        return new TestSuite(MetaTripleWriterPluginTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2004-12-17
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
