package com.mkdoc.jspider;

/*
    Created:    2004-12-09
    Issued:     2004-12-09
    Modified:   2005-03-29

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.core.logging.Log;

/**
 *  Mock JSpider <code>Log</code> implementation for testing.
 *  @version    0.2 2005-03-29
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockLog implements Log {

    /**
     *  Store for debug messages.
     *  @since  Version 0.1 2004-12-09
     */
    private StringBuffer debug = new StringBuffer();

    /**
     *  Store for error messages.
     *  @since  Version 0.1 2004-12-09
     */
    private StringBuffer error = new StringBuffer();

    /**
     *  Store for info messages.
     *  @since  Version 0.1 2004-12-09
     */
    private StringBuffer info = new StringBuffer();

    /**
     *  Store for fatal messages.
     *  @since  Version 0.1 2004-12-09
     */
    private StringBuffer fatal = new StringBuffer();

    /**
     *  Store for trace messages.
     *  @since  Version 0.1 2004-12-09
     */
    private StringBuffer trace = new StringBuffer();

    /**
     *  Store for warn messages.
     *  @since  Version 0.1 2004-12-09
     */
    private StringBuffer warn = new StringBuffer();

    /**
     *  Whether debug logging is enabled.
     *  @since  Version 0.1 2004-12-09
     */
    private boolean debugEnabled = true;

    /**
     *  Whether error logging is enabled.
     *  @since  Version 0.1 2004-12-09
     */
    private boolean errorEnabled = true;

    /**
     *  Whether info logging is enabled.
     *  @since  Version 0.1 2004-12-09
     */
    private boolean infoEnabled = true;

    /**
     *  Whether fatal logging is enabled.
     *  @since  Version 0.1 2004-12-09
     */
    private boolean fatalEnabled = true;

    /**
     *  Whether trace logging is enabled.
     *  @since  Version 0.1 2004-12-09
     */
    private boolean traceEnabled = true;

    /**
     *  Whether warn logging is enabled.
     *  @since  Version 0.1 2004-12-09
     */
    private boolean warnEnabled = true;

    /**
     *  Check whether debug logging is enabled.
     *  @return <code>true</code> if debug logging is
     *          enabled.
     *  @since  Version 0.1 2004-12-09
     */
    public final boolean isDebugEnabled() {

        return debugEnabled;
    }

    /**
     *  Set debug logging status.
     *  @param  enable  Whether logging is enabled.
     *  @since  Version 0.1 2004-12-09
     */
    public final void setDebugEnabled(final boolean enable) {

        debugEnabled = enable;
    }

    /**
     *  Check whether error logging is enabled.
     *  @return <code>true</code> if error logging is
     *          enabled.
     *  @since  Version 0.1 2004-12-09
     */
    public final boolean isErrorEnabled() {

        return errorEnabled;
    }

    /**
     *  Set error logging status.
     *  @param  enable  Whether logging is enabled.
     *  @since  Version 0.1 2004-12-09
     */
    public final void setErrorEnabled(final boolean enable) {

        errorEnabled = enable;
    }

    /**
     *  Check whether fatal logging is enabled.
     *  @return <code>true</code> if fatal logging is
     *          enabled.
     *  @since  Version 0.1 2004-12-09
     */
    public final boolean isFatalEnabled() {

        return fatalEnabled;
    }

    /**
     *  Set fatal logging status.
     *  @param  enable  Whether logging is enabled.
     *  @since  Version 0.1 2004-12-09
     */
    public final void setFatalEnabled(final boolean enable) {

        fatalEnabled = enable;
    }

    /**
     *  Check whether info logging is enabled.
     *  @return <code>true</code> if info logging is
     *          enabled.
     *  @since  Version 0.1 2004-12-09
     */
    public final boolean isInfoEnabled() {

        return infoEnabled;
    }

    /**
     *  Set info logging status.
     *  @param  enable  Whether logging is enabled.
     *  @since  Version 0.1 2004-12-09
     */
    public final void setInfoEnabled(final boolean enable) {

        infoEnabled = enable;
    }

    /**
     *  Check whether trace logging is enabled.
     *  @return <code>true</code> if trace logging is
     *          enabled.
     *  @since  Version 0.1 2004-12-09
     */
    public final boolean isTraceEnabled() {

        return traceEnabled;
    }

    /**
     *  Set trace logging status.
     *  @param  enable  Whether logging is enabled.
     *  @since  Version 0.1 2004-12-09
     */
    public final void setTraceEnabled(final boolean enable) {

        traceEnabled = enable;
    }

    /**
     *  Check whether warn logging is enabled.
     *  @return <code>true</code> if warn logging is
     *          enabled.
     *  @since  Version 0.1 2004-12-09
     */
    public final boolean isWarnEnabled() {

        return warnEnabled;
    }

    /**
     *  Set warn logging status.
     *  @param  enable  Whether logging is enabled.
     *  @since  Version 0.1 2004-12-09
     */
    public final void setWarnEnabled(final boolean enable) {

        warnEnabled = enable;
    }

    /**
     *  Log a trace message.
     *  @param  object  An object from which to generate
     *          a log.
     *  @param  throwable   An object from which to get
     *          a detail message.
     *  @since  Version 0.1 2004-12-09
     */
    public final void trace(final Object object,
                            final Throwable throwable) {

        if (traceEnabled) {

            trace.append(object);

            if (throwable != null) {

                trace.append(throwable.getMessage());
            }
        }
    }

    /**
     *  Log a trace message.
     *  @param  object  An object from which to generate
     *          a log.
     *  @since  Version 0.1 2004-12-09
     */
    public final void trace(final Object object) {

        trace(object, null);
    }

    /**
     *  Get the trace log.
     *  @return The contents of the log.
     *  @since  Version 0.1 2004-12-09
     */
    public final String getTrace() {

        return trace.toString();
    }

    /**
     *  Log a debug message.
     *  @param  object  An object from which to generate
     *          a log.
     *  @param  throwable   An object from which to get
     *          a detail message.
     *  @since  Version 0.1 2004-12-09
     */
    public final void debug(final Object object,
                            final Throwable throwable) {

        if (debugEnabled) {

            debug.append(object);

            if (throwable != null) {

                debug.append(throwable.getMessage());
            }
        }
    }

    /**
     *  Log a debug message.
     *  @param  object  An object from which to generate
     *          a log.
     *  @since  Version 0.1 2004-12-09
     */
    public final void debug(final Object object) {

        debug(object, null);
    }

    /**
     *  Get the debug log.
     *  @return The contents of the log.
     *  @since  Version 0.1 2004-12-09
     */
    public final String getDebug() {

        return debug.toString();
    }

    /**
     *  Log an info message.
     *  @param  object  An object from which to generate
     *          a log.
     *  @param  throwable   An object from which to get
     *          a detail message.
     *  @since  Version 0.1 2004-12-09
     */
    public final void info(final Object object,
                            final Throwable throwable) {

        if (infoEnabled) {

            info.append(object);

            if (throwable != null) {

                info.append(throwable.getMessage());
            }
        }
    }

    /**
     *  Log an info message.
     *  @param  object  An object from which to generate
     *          a log.
     *  @since  Version 0.1 2004-12-09
     */
    public final void info(final Object object) {

        info(object, null);
    }

    /**
     *  Get the info log.
     *  @return The contents of the log.
     *  @since  Version 0.1 2004-12-09
     */
    public final String getInfo() {

        return info.toString();
    }

    /**
     *  Log a warn message.
     *  @param  object  An object from which to generate
     *          a log.
     *  @param  throwable   An object from which to get
     *          a detail message.
     *  @since  Version 0.1 2004-12-09
     */
    public final void warn(final Object object,
                            final Throwable throwable) {

        if (warnEnabled) {

            warn.append(object);

            if (throwable != null) {

                warn.append(throwable.getMessage());
            }
        }
    }

    /**
     *  Log a warn message.
     *  @param  object  An object from which to generate
     *          a log.
     *  @since  Version 0.1 2004-12-09
     */
    public final void warn(final Object object) {

        warn(object, null);
    }

    /**
     *  Get the warn log.
     *  @return The contents of the log.
     *  @since  Version 0.1 2004-12-09
     */
    public final String getWarn() {

        return warn.toString();
    }

    /**
     *  Log an error message.
     *  @param  object  An object from which to generate
     *          a log.
     *  @param  throwable   An object from which to get
     *          a detail message.
     *  @since  Version 0.1 2004-12-09
     */
    public final void error(final Object object,
                            final Throwable throwable) {

        if (errorEnabled) {

            error.append(object);

            if (throwable != null) {

                error.append(throwable.getMessage());
            }
        }
    }

    /**
     *  Log an error message.
     *  @param  object  An object from which to generate
     *          a log.
     *  @since  Version 0.1 2004-12-09
     */
    public final void error(final Object object) {

        error(object, null);
    }

    /**
     *  Get the error log.
     *  @return The contents of the log.
     *  @since  Version 0.1 2004-12-09
     */
    public final String getError() {

        return error.toString();
    }

    /**
     *  Log a fatal message.
     *  @param  object  An object from which to generate
     *          a log.
     *  @param  throwable   An object from which to get
     *          a detail message.
     *  @since  Version 0.1 2004-12-09
     */
    public final void fatal(final Object object,
                            final Throwable throwable) {

        if (fatalEnabled) {

            fatal.append(object);

            if (throwable != null) {

                fatal.append(throwable.getMessage());
            }
        }
    }

    /**
     *  Log a fatal message.
     *  @param  object  An object from which to generate
     *          a log.
     *  @since  Version 0.1 2004-12-09
     */
    public final void fatal(final Object object) {

        fatal(object, null);
    }

    /**
     *  Get the fatal log.
     *  @return The contents of the log.
     *  @since  Version 0.1 2004-12-09
     */
    public final String getFatal() {

        return fatal.toString();
    }

}
