package com.mkdoc.jspider;

/*
    Created:    2005-04-12
    Issued:     2005-04-12
    Modified:   2005-10-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.api.model.Folder;
import net.javacoding.jspider.api.model.Site;

import net.javacoding.jspider.api.model.Resource;
import net.javacoding.jspider.api.model.ResourceReference;

import net.javacoding.jspider.core.event.impl.ResourceParsedErrorEvent;
import net.javacoding.jspider.core.event.impl.ResourceParsedOkEvent;
import net.javacoding.jspider.core.event.impl.URLFoundEvent;
import net.javacoding.jspider.core.event.impl.URLSpideredErrorEvent;
import net.javacoding.jspider.core.event.impl.URLSpideredOkEvent;

import net.javacoding.jspider.core.storage.ResourceDAO;
import net.javacoding.jspider.core.storage.exception.InvalidStateTransitionException;


import java.net.URL;

import java.util.Hashtable;
import java.util.Map;

/**
 *  A mock <code>ResourceDAO</code> for testing, <em>minimal</em>.
 *  @version    0.2 2005-10-28
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockResourceDAO implements ResourceDAO {

    /**
     *  A map of resources by URL.
     *  @since  Version 0.1 2005-04-12
     */
    private final Map resources = new Hashtable();

    /**
     *  Register a URL.
     *  @param  resource    The resource to register.
     *  @param  url The URL against which the resource is keyed.
     *  @since  Version 0.1 2005-04-12
     */
    public final void registerResource(final Resource resource,
                                       final URL url) {

        resources.put(url.toExternalForm(), resource);
    }

    /**
     *  Register a URL.
     *  @param  url The URL to register.
     *  @return The URL assigned to the URL.
     *  @since  Version 0.1 2005-04-12
     */
    public final Resource registerURL(final URL url) {

        MockResource resource = new MockResource();

        resource.setUrl(url);

        resources.put(url.toExternalForm(), resource);

        return resource;
    }

    /**
     *  Register a URL referrer.
     *  @param  url The original URL to add the referrer to.
     *  @param  referrer    The referrer to the original URL.
     *  @since  Version 0.1 2005-04-12
     */
    public final void registerURLReference(final URL url,
                                           final URL referrer) {

        // Empty
    }

    /**
     *  Get all resources.
     *  @return An array of all resources in storage.
     *  @since  Version 0.2 2005-10-28
     */
    public final Resource[] getAllResources() {

        return new Resource[0];
    }

    /**
     *  Get referring resources.
     *  @param  resource    The resource to check for referrers.
     *  @return An array of all resources that refer to the given resource.
     *  @since  Version 0.2 2005-10-28
     */
    public final Resource[] getRefereringResources(final Resource resource) {

        return new Resource[0];
    }

    /**
     *  Get referenced resources.
     *  @param  resource    The resource to check for references.
     *  @return An array of all the resources referenced by the given resource.
     *  @since  Version 0.2 2005-10-28
     */
    public final Resource[] getReferencedResources(final Resource resource) {

        return new Resource[0];
    }

    /**
     *  Get incoming resources.
     *  @param  resource    The resource to check for incoming resource
     *          references.
     *  @return An array of resource references attached to the given resource.
     *  @since  Version 0.2 2005-10-28
     */
    public final ResourceReference[] getIncomingReferences(
                                                     final Resource resource) {

        return new ResourceReference[0];
    }

    /**
     *  Get outgoing references.
     *  @param  resource    The resource to check for outgoing references.
     *  @return An array of resource references from the given resource.
     *  @since  Version 0.2 2005-10-28
     */
    public final ResourceReference[] getOutgoingReferences(
                                                     final Resource resource) {

        return new ResourceReference[0];
    }

    /**
     *  Get root resources for a site.
     *  @param  site    The site to check for root resources.
     *  @return An array of resources at the root of the site.
     *  @since  Version 0.2 2005-10-28
     */
    public final Resource[] getRootResources(final Site site) {

        return new Resource[0];
    }

    /**
     *  Get resources by site.
     *  @param  site    The site to check for resources.
     *  @return An array of all resources for the site.
     *  @since  Version 0.2 2005-10-28
     */
    public final Resource[] getBySite(final Site site) {

        return new Resource[0];
    }

    /**
     *  Find by folder.
     *  @param  folder  A folder reference to check for resources.
     *  @return An array of resources in the given folder.
     *  @since  Version 0.2 2005-10-28
     */
    public final Resource[] findByFolder(final Folder folder) {

        return new Resource[0];
    }

    /**
     *  Get a resource by URL.
     *  @param  url The URL by which the resource is indexed.
     *  @return A resource reference for the given URL.
     *  @since  Version 0.2 2005-10-28
     */
    public final Resource getResource(final URL url) {

        return (Resource) resources.get(url.toExternalForm());
    }

    /**
     *  Set spidered.
     *  @param  url The URL against which to record the spider event.
     *  @param  event   The spider event to record.
     *  @since  Version 0.2 2005-10-28
     */
    public final void setSpidered(final URL url,
                                  final URLSpideredOkEvent event) {

        // Empty
    }

    /**
     *  Ignore for parsing.
     *  @since  Version 0.2 2005-10-28
     *  @param  url The URL that will be ignored.
     *  @exception  InvalidStateTransitionException If the registered resource
     *          was not known or not in an appropriate state before this method
     *          was called.
     */
    public final void setIgnoredForParsing(final URL url)
        throws InvalidStateTransitionException {

        // Empty
    }


    /**
     *  Set ignored for fetching.
     *  @param  url The URL against which to record the URL found event.
     *  @param  event   The spider event to record.
     *  @exception  InvalidStateTransitionException If the registered resource
     *          was not known or not in an appropriate state before this method
     *          was called.
     *  @since  Version 0.2 2005-10-28
     */
    public final void setIgnoredForFetching(final URL url,
                                            final URLFoundEvent event)
        throws InvalidStateTransitionException {

        // Empty
    }

    /**
     *  Set forbidden.
     *  @param  url The URL against which to record the URL found event.
     *  @param  event   The spider event to record.
     *  @exception  InvalidStateTransitionException If the registered resource
     *          was not known or not in an appropriate state before this method
     *          was called.
     *  @since  Version 0.2 2005-10-28
     */
    public final void setForbidden(final URL url,
                                   final URLFoundEvent event)
        throws InvalidStateTransitionException {

        // Empty
    }

    /**
     *  Set an error.
     *  @param  url The URL against which to record the parse error event.
     *  @param  event   The parse event to record.
     *  @exception  InvalidStateTransitionException If the registered resource
     *          was not known or not in an appropriate state before this method
     *          was called.
     *  @since  Version 0.2 2005-10-28
     */
    public final void setError(final URL url,
                               final ResourceParsedErrorEvent event)
        throws InvalidStateTransitionException {

        // Empty
    }

    /**
     *  Set parsed.
     *  @param  url The URL against which to record the parse success event.
     *  @param  event   The parse event to record.
     *  @exception  InvalidStateTransitionException If the registered resource
     *          was not known or not in an appropriate state before this method
     *          was called.
     *  @since  Version 0.2 2005-10-28
     */
    public final void setParsed(final URL url,
                                final ResourceParsedOkEvent event)
        throws InvalidStateTransitionException {

        // Empty
    }

    /**
     *  Set an error.
     *  @param  url The URL against which to record the spider error event.
     *  @param  event   The spider event to record.
     *  @exception  InvalidStateTransitionException If the registered resource
     *          was not known or not in an appropriate state before this method
     *          was called.
     *  @since  Version 0.2 2005-10-28
     */
    public final void setError(final URL url,
                               final URLSpideredErrorEvent event)
        throws InvalidStateTransitionException {

        // Empty
    }

}
