package com.mkdoc.jspider;

/*
    Created:    2005-04-11
    Issued:     2005-04-11
    Modified:   2005-10-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.api.model.Cookie;
import net.javacoding.jspider.api.model.Site;

import net.javacoding.jspider.core.dispatch.EventDispatcher;

import net.javacoding.jspider.core.Agent;
import net.javacoding.jspider.core.SpiderContext;

import net.javacoding.jspider.core.rule.Ruleset;
import net.javacoding.jspider.core.rule.impl.RuleSetImpl;

import net.javacoding.jspider.core.storage.Storage;

import net.javacoding.jspider.spi.Rule;

import java.io.InputStream;

import java.net.URL;
import java.net.URLConnection;

import java.util.Hashtable;
import java.util.Map;
import java.util.Vector;


/**
 *  Mock <code>SpiderContext</code> type.
 *  <p>
 *    Implemented in skeleton form as far as required to date.
 *  </p>
 *  @version    0.2 2005-10-28
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockSpiderContext implements SpiderContext {

    /**
     *  The JSpider agent interface.
     *  @since  Version 0.1 2005-04-11
     */
    private Agent agent;

    /**
     *  The base URL.
     *  @since  Version 0.1 2005-04-11
     */
    private URL baseUrl;

    /**
     *  The event dispatcher for the context.
     *  @since  Version 0.1 2005-04-11
     */
    private EventDispatcher dispatcher;

    /**
     *  The set of parser rules.
     *  @since  Version 0.1 2005-04-11
     */
    private Ruleset parserRuleset = new RuleSetImpl(Ruleset.RULESET_GENERAL,
                                                    new Vector());

    /**
     *  The set of spider rules.
     *  @since  Version 0.1 2005-04-11
     */
    private Ruleset spiderRuleset = new RuleSetImpl(Ruleset.RULESET_GENERAL,
                                                    new Vector());

    /**
     *  A map of site spider rules.
     *  @since  Version 0.1 2005-04-11
     */
    private Map spiderRules = new Hashtable();

    /**
     *  A map of site robots rules.
     *  @since  Version 0.1 2005-04-11
     */
    private Map robotRules = new Hashtable();

    /**
     *  A map of site parser rules.
     *  @since  Version 0.1 2005-04-11
     */
    private Map parserRules = new Hashtable();

    /**
     *  The robots.txt rule.
     *  @since  Version 0.1 2005-04-11
     */
    private Rule robotsRule;

    /**
     *  The storage reference.
     *  @since  Version 0.1 2005-04-11
     */
    private Storage storage = new MockStorage();

    /**
     *  Set the cookies for the context.
     *  @param  site    The site to set cookies for
     *  @param  cookies The cookie array to set.
     *  @since  Version 0.1 2005-04-11
     */
    public final void setCookies(final Site site,
                                 final Cookie[] cookies) {

        // Empty
    }

    /**
     *  Pre-handle the URL.
     *  @param  connection  The URL connection to pre-process.
     *  @param  site    The site to associate with the URL.
     *  @since  Version 0.1 2005-04-11
     */
    public final void preHandle(final URLConnection connection,
                                final Site site) {

        // Empty
    }

    /**
     *  Post-handle the URL.
     *  @param  connection  The URL connection to post-process.
     *  @param  site    The site associated with the URL.
     *  @since  Version 0.1 2005-04-11
     */
    public final void postHandle(final URLConnection connection,
                                 final Site site) {

        // Empty
    }

    /**
     *  Get the storage reference.
     *  @return The storage implementation for the context.
     *  @since  Version 0.1 2005-04-11
     */
    public final Storage getStorage() {

        return storage;
    }

    /**
     *  Set the storage reference.
     *  @param  storage The storage implementation to assign to the context.
     *  @since  Version 0.2 2005-10-28
     */
    public final void setStorage(final Storage storage) {

        this.storage = storage;
    }

    /**
     *  Get the agent reference.
     *  @return The user agent reference for the context.
     *  @since  Version 0.1 2005-04-11
     */
    public final Agent getAgent() {

        return agent;
    }

    /**
     *  Set the agent reference.
     *  @param  agent   The user agent to assign to the context.
     *  @since  Version 0.1 2005-04-11
     */
    public final void setAgent(final Agent agent) {

        this.agent = agent;
    }

    /**
     *  Get the base URL.
     *  @return The base URL for the spider context.
     *  @since  Version 0.1 2005-04-11
     */
    public final URL getBaseURL() {

        return baseUrl;
    }

    /**
     *  Get the event dispatcher.
     *  @return The event dispatcher for the context.
     *  @since  Version 0.1 2005-04-11
     */
    public final EventDispatcher getEventDispatcher() {

        return dispatcher;
    }

    /**
     *  Get the general spider rules.
     *  @return The general spider rule set.
     *  @since  Version 0.1 2005-04-11
     */
    public final Ruleset getGeneralSpiderRules() {

        return spiderRuleset;
    }

    /**
     *  Get the general parser rules.
     *  @return The general parser rule set.
     *  @since  Version 0.1 2005-04-11
     */
    public final Ruleset getGeneralParserRules() {

        return parserRuleset;
    }

    /**
     *  Get site specific spider rules.
     *  @param  site    The site for which to get the spidering rule set.
     *  @return The spidering rules for the given site.
     *  @since  Version 0.1 2005-04-11
     */
    public final Ruleset getSiteSpiderRules(final Site site) {

        return (Ruleset) spiderRules.get(site);
    }

    /**
     *  Get site specific spider rules.
     *  @param  site    The site for which to get the parsing rule set.
     *  @return The parse rules for the given site.
     *  @since  Version 0.1 2005-04-11
     */
    public final Ruleset getSiteParserRules(final Site site) {

        return (Ruleset) parserRules.get(site);
    }

    /**
     *  Throttle a site.
     *  @param  site    A site whose request rate should be throttled.
     *  @since  Version 0.1 2005-04-11
     */
    public final void throttle(final Site site) {

        // Empty
    }

    /**
     *  Register a robots.txt file for a site.
     *  @param  site    The site for which to register the robots.txt policy.
     *  @param  inStream    The input stream for the robots.txt policy.
     *  @since  Version 0.1 2005-04-11
     */
    public final void registerRobotsTXT(final Site site,
                                        final InputStream inStream) {

        // Empty
    }

    /**
     *  Register a robots.txt error for a site.
     *  @param  site    The site for which to register a robots.txt error.
     *  @since  Version 0.1 2005-04-11
     */
    public final void registerRobotsTXTError(final Site site) {

        // Empty
    }

    /**
     *  Register robots.txt for a site has been skipped.
     *  @param  site    The site against which to register a skipped robots.txt
     *          policy.
     *  @since  Version 0.1 2005-04-11
     */
    public final void registerRobotsTXTSkipped(final Site site) {

        // Empty
    }

    /**
     *  Register a new site.
     *  @param  site    The site to register.
     *  @since  Version 0.1 2005-04-11
     */
    public final void registerNewSite(final Site site) {

        // Empty
    }

    /**
     *  Get a robots.txt rule for a site.
     *  @param  site    The site for which to get the robots.txt policy.
     *  @return The robots.txt handling rule for the given site.
     *  since   Veraion 0.1 2005-04-11
     */
    public final Rule getSiteRobotsTXTRule(final Site site) {

        return robotsRule;
    }

    /**
     *  Get the user agent identifier.
     *  @return The user agent identifier.
     *  @since  Version 0.1 2005-04-11
     */
    public final String getUserAgent() {

        return "MockSpiderContext user agent";
    }

    /**
     *  Check whether a proxy is in use.
     *  @return <code>true</code> if a proxy should be used.
     *  @since  Version 0.1 2005-04-11
     */
    public final boolean getUseProxy() {

        return false;
    }

}
