package com.mkdoc.jspider;

/*
    Created:    2005-04-12
    Issued:     2005-04-12
    Modified:   2005-10-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.api.model.Summary;

import net.javacoding.jspider.core.storage.Storage;
import net.javacoding.jspider.core.storage.CookieDAO;
import net.javacoding.jspider.core.storage.DecisionDAO;
import net.javacoding.jspider.core.storage.EMailAddressDAO;
import net.javacoding.jspider.core.storage.ResourceDAO;
import net.javacoding.jspider.core.storage.SiteDAO;

/**
 *  A mock <code>Storage</code> for testing.
 *  <p>
 *    Implementated as far as necessary for testing.
 *  </p>
 *  @version    0.2 2005-10-28
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockStorage implements Storage {

    /**
     *  The <code>Cookie</code> DAO.
     *  @since  Version 0.1 2005-04-12
     */
    private CookieDAO cookieDAO;

    /**
     *  The <code>Decision</code> DAO.
     *  @since  Version 0.1 2005-04-12
     */
    private DecisionDAO decisionDAO;

    /**
     *  The <code>EMailAddress</code> DAO.
     *  @since  Version 0.1 2005-04-12
     */
    private EMailAddressDAO emailAddressDAO;

    /**
     *  The <code>Resource</code> DAO.
     *  @since  Version 0.1 2005-04-12
     */
    private ResourceDAO resourceDAO;

    /**
     *  The <code>Site</code> DAO.
     *  @since  Version 0.1 2005-04-12
     */
    private SiteDAO siteDAO;

    /**
     *  The <code>Summary</code>.
     *  @since  Version 0.1 2005-04-12
     */
    private Summary summary;

    /**
     *  Get a <code>Cookie</code> DAO.
     *  @return A cookie data access object.
     *  @since  Version 0.1 2005-04-12
     */
    public final CookieDAO getCookieDAO() {

        return cookieDAO;
    }

    /**
     *  Get a <code>Decision</code> DAO.
     *  @return A decision data access object.
     *  @since  Version 0.1 2005-04-12
     */
    public final DecisionDAO getDecisionDAO() {

        return decisionDAO;
    }

    /**
     *  Get a <code>EMailAddres</code> DAO.
     *  @return An email data access object.
     *  @since  Version 0.1 2005-04-12
     */
    public final EMailAddressDAO getEMailAddressDAO() {

        return emailAddressDAO;
    }

    /**
     *  Get a <code>Resource</code> DAO.
     *  @return A resource data access object.
     *  @since  Version 0.1 2005-04-12
     */
    public final ResourceDAO getResourceDAO() {

        return resourceDAO;
    }

    /**
     *  Set the <code>Resource</code> DAO.
     *  @param  resourceDAO Assign a resource DAO interface.
     *  @since  Version 0.2 2005-10-28
     */
    public final void setResourceDAO(final ResourceDAO resourceDAO) {

        this.resourceDAO = resourceDAO;
    }

    /**
     *  Get a <code>Site</code> DAO.
     *  @return A site data access object.
     *  @since  Version 0.1 2005-04-12
     */
    public final SiteDAO getSiteDAO() {

        return siteDAO;
    }

    /**
     *  Get the storage <code>Summary</code>.
     *  @return A summary of the storage system.
     *  @since  Version 0.1 2005-04-12
     */
    public final Summary getSummary() {

        return summary;
    }

}
