package com.mkdoc.jspider;

/*
    Created:    2005-05-31
    Issued:     2005-05-31
    Modified:   2005-11-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.sax.RdfContentHandler;
import com.mkdoc.sax.XhtmlStoreWriter;

import com.mkdoc.store.StoreManager;

import java.io.InputStream;

import java.net.URL;

/**
 *  A mock subject manager plugin for testing.
 *  @version    0.2 2005-11-01
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockStoreManagerPlugin extends AbstractStoreManagerPlugin {

    /**
     *  The plugin name.
     *  @since  Version 0.1 2005-05-31
     */
    private String name;

    /**
     *  The plugin description.
     *  @since  Version 0.1 2005-05-31
     */
    private String description;

    /**
     *  The plugin version.
     *  @since  Version 0.1 2005-05-31
     */
    private String version;

    /**
     *  The most recent parse URL source.
     *  @since  Version 0.1 2005-05-31
     */
    private URL url;

    /**
     *  Get a pre-configured content handler.
     *  @param  manager A subject manager interface the content handler will
     *          write the document metadata to.
     *  @param  url An initial URI subject.
     *  @return An <code>RdfContentHandler</code> instance with custom
     *          <code>Schema</code> if configured.
     *  @since  Version 0.1 2005-05-31
     */
    protected final RdfContentHandler getContentHandler(
                                                  final StoreManager manager,
                                                  final URL url) {

        return new XhtmlStoreWriter(manager, url);
    }

    /**
     *  Get the plugin name.
     *  @return The plugin name.
     *  @since  Version 0.1 2005-05-31
     */
    public final String getName() {

        return name;
    }

    /**
     *  Set the plugin name.
     *  @param  name    The plugin name.
     *  @since  Version 0.1 2005-05-31
     */
    public final void setName(final String name) {

        this.name = name;
    }

    /**
     *  Get the plugin description.
     *  @return The plugin description.
     *  @since  Version 0.1 2005-05-31
     */
    public final String getDescription() {

        return description;
    }

    /**
     *  Set the plugin description.
     *  @param  description The plugin description.
     *  @since  Version 0.1 2005-05-31
     */
    public final void setDescription(final String description) {

        this.description = description;
    }

    /**
     *  Get the plugin version.
     *  @return The plugin version.
     *  @since  Version 0.1 2005-05-31
     */
    public final String getVersion() {

        return version;
    }

    /**
     *  Set the plugin version.
     *  @param  version The plugin version.
     *  @since  Version 0.1 2005-05-31
     */
    public final void setVersion(final String version) {

        this.version = version;
    }

    /**
     *  Add a resource to the repository from an input stream.
     *  @param  inStream The input stream source for the metadata.
     *  @param  url The URL of the source document.
     *  @since  Version 0.1 2005-05-31
     */
    protected final void parse(final InputStream inStream,
                               final URL url) {

        this.url = url;
    }

    /**
     *  Get the last parsed URL.
     *  @return The most recent URL passed to the parse method, may be
     *          <code>null</code>.
     *  @since  Version 0.1 2005-05-31
     */
    public final URL getParsedUrl() {

        return url;
    }

}
