package com.mkdoc.jspider;

/*
    Created:    2005-03-29
    Issued:     2005-03-29
    Modified:   2005-11-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.api.event.resource.ResourceDiscoveredEvent;

import org.openrdf.sesame.Sesame;

import org.openrdf.sesame.config.ConfigurationException;
import org.openrdf.sesame.config.RepositoryConfig;
import org.openrdf.sesame.config.SailConfig;

import org.openrdf.sesame.repository.local.LocalService;
import org.openrdf.sesame.repository.SesameRepository;

import org.openrdf.sesame.sailimpl.memory.RdfSchemaRepositoryConfig;

import com.mkdoc.io.MockInputStream;

import com.mkdoc.sax.RdfContentHandler;

import com.mkdoc.schema.UKeGMS;
import com.mkdoc.schema.Schema;
import com.mkdoc.schema.UKeGMSProfile;

import com.mkdoc.store.AbstractFileStoreManager;
import com.mkdoc.store.MockStoreManager;
import com.mkdoc.store.StoreManager;

import com.mkdoc.util.ApplicationContext;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;

import java.net.URL;

import org.xml.sax.ContentHandler;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>XhtmlStoreWriterPlugin</code>.
 *  @version    0.8 2005-11-01
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class XhtmlStoreWriterPluginTest extends TestCase {

    /**
     *  The expected <code>IllegalArgumentException</code>.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String EXPECTED_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  The application profile property key name.
     *  @since  Version 0.5 2005-05-26
     */
    public static final String PROFILE = "profile";

    /**
     *  A test domain.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String DOMAIN = System.getProperty("testDomain");

    /**
     *  A test domain URL.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String U_DOMAIN = "http://" + DOMAIN;

    /**
     *  A test domain root directory URL.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String U_ROOT = U_DOMAIN + "/";

    /**
     *  A root index source file.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String INDEX = "index.html";

    /**
     *  A test domain root index URL.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String U_ROOT_IDX = U_ROOT + INDEX;

    /**
     *  The project source directory.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String HOME = System.getProperty("mkHome");

    /**
     *  The project test source directory.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String TEST = System.getProperty("testDir") +
                                         File.separator +
                                         "html";

    /**
     *  The default JSpider output directory.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String OUTPUT = "output";

    /**
     *  A custom output directory.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String CUSTOM = "custom";

    /**
     *  The test source directory path.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String PATH_TEST = HOME +
                                              File.separator +
                                              TEST;

    /**
     *  An invalid source file for testing.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String INVALID = "invalid-Basic.html";

    /**
     *  A file reference for invalid source for testing.
     *  @since  Version 0.4 2005-05-10
     */
    protected static final File SRC_INVALID = new File(PATH_TEST +
                                                       File.separator +
                                                       INVALID);

    /**
     *  An irreparably invalid source file for testing.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String BROKEN = "invalid-Irreparable.html";

    /**
     *  An invalid output file for testing.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String INVALID_NT = "invalid-Basic.nt";

    /**
     *  An irreparably invalid output file for testing.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String BROKEN_NT = "invalid-Irreparable.nt";

    /**
     *  A file reference for invalid source for testing.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final File FILE_INVALID = new File(PATH_TEST +
                                                        File.separator +
                                                        INVALID);

    /**
     *  A file reference for an irreparably invalid source for testing.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final File FILE_BROKEN = new File(PATH_TEST +
                                                       File.separator +
                                                       BROKEN);

    /**
     *  An valid source file for testing.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String VALID = "valid-Basic.html";

    /**
     *  An valid output file for testing.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String VALID_RDF =
    "com.mkdoc.jspider.XhtmlStoreWriterPlugin.rdf";


    /**
     *  A file reference for valid source for testing.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final File FILE_VALID = new File(PATH_TEST +
                                                      File.separator +
                                                      VALID);

    /**
     *  The default JSpider output path.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String PATH_OUT = HOME +
                                             File.separator +
                                             OUTPUT;


    /**
     *  The custom output path.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String PATH_CUSTOM = HOME +
                                                File.separator +
                                                OUTPUT +
                                                File.separator +
                                                CUSTOM;

    /**
     *  A non-existent output directory.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String NOT_EXIST = "bogus";

    /**
     *  A non-existent output path.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String PATH_BAD = HOME +
                                                   File.separator +
                                                   OUTPUT +
                                                   File.separator +
                                                   NOT_EXIST;


    /**
     *  A <code>File</code> representation of the default
     *  output path for comparisons.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final File DIR_OUT = new File(PATH_OUT);

    /**
     *  A <code>File</code> representation of the home
     *  directory for comparisons.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final File DIR_HOME = new File(HOME);

    /**
     *  A <code>File</code> representation of the custom
     *  output directory for comparisons.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final File DIR_CUSTOM = new File(PATH_CUSTOM);

    /**
     *  A test domain URL.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String URL_DOMAIN = "http://" + DOMAIN;

    /**
     *  An expected output directory for the test domain.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String PATH_DOMAIN = PATH_CUSTOM +
                                                File.separator +
                                                DOMAIN;

    /**
     *  A File reference for the expected output directory
     *  for the test domain.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final File DIR_DOMAIN = new File(PATH_DOMAIN);

    /**
     *  A test domain root directory URL.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String URL_ROOT = URL_DOMAIN + "/";

    /**
     *  A test domain root index URL.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String URL_INDEX = URL_ROOT + INDEX;

    /**
     *  A valid source URL.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String URL_VALID = URL_ROOT + VALID;

    /**
     *  An invalid source URL.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String URL_INVALID = URL_ROOT + INVALID;


    /**
     *  An irreparably invalid source URL.
     *  @since  Version 0.1 2005-03-29
     */
    protected static final String URL_BROKEN = URL_ROOT + BROKEN;

    /**
     *  A common property set for configuration.
     *  @since  Version 0.1 2005-03-29
     */
    protected MockPropertySet propertySet;

    /**
     *  A pre-configured plugin to use in multiple tests.
     *  @since  Version 0.1 2005-03-29
     */
    protected XhtmlStoreWriterPlugin commonPlugin;

    /**
     *  A Sesame repository for testing.
     *  @since  Version 0.1 2005-03-29
     */
    protected SesameRepository repository;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-03-29
     */
    public XhtmlStoreWriterPluginTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @exception  ConfigurationException  Should not be thrown in set up.
     *  @since  Version 0.1 2005-03-29
     */
    protected final void setUp() throws ConfigurationException {

        propertySet = new MockPropertySet();

        propertySet.setBoolean(AbstractFileStoreManager.ABSOLUTE, true);
        propertySet.setString(AbstractFileStoreManager.FOLDER, PATH_CUSTOM);

        commonPlugin = new XhtmlStoreWriterPlugin(propertySet);


        RepositoryConfig storeConfig = new RepositoryConfig("test");

        SailConfig sailConfig = new RdfSchemaRepositoryConfig();

        storeConfig.addSail(sailConfig);

        LocalService service = Sesame.getService();

        repository = service.createRepository(storeConfig);
    }

    /**
     *  Test constructor load custom profile with none configured.
     *  @since  Version 0.5 2005-05-26
     */
    public final void testConstructorNoCustomSchemaConfigured() {

        propertySet = new MockPropertySet();

        commonPlugin = new XhtmlStoreWriterPlugin(propertySet);

        assertTrue(commonPlugin.profile == null);

    }

    /**
     *  Test constructor load custom schema.
     *  @exception  ClassNotFoundException  Should not be thrown.
     *  @since  Version 0.5 2005-05-26
     */
    public final void testLoadCustomSchemaSuccess()
        throws ClassNotFoundException {

        String eGMS = "com.mkdoc.schema.UKeGMSProfile";

        propertySet = new MockPropertySet();

        propertySet.setClass(PROFILE, Class.forName(eGMS));

        commonPlugin = new XhtmlStoreWriterPlugin(propertySet);

        assertTrue(commonPlugin.profile instanceof UKeGMSProfile);

    }

    /**
     *  Test load custom profile that cannot be loaded.
     *  @exception  ClassNotFoundException  Should not be thrown.
     *  @since  Version 0.5 2005-05-26
     */
    public final void testConstructorCustomProfileException()
        throws ClassNotFoundException {

        String schemaClass = "java.lang.String";

        propertySet = new MockPropertySet();

        propertySet.setClass(PROFILE, Class.forName(schemaClass));

        commonPlugin = new XhtmlStoreWriterPlugin(propertySet);

        assertTrue(commonPlugin.profile == null);
    }

    /**
     *  Test get content handler with profile loaded.
     *  @exception  IOException  Should not be thrown by this test.
     *  @exception  ClassNotFoundException  Should not be thrown by this test.
     *  @exception  InstantiationException  Should not be thrown by this test.
     *  @exception  IllegalAccessException  Should not be thrown by this test.
     *  @since  Version 0.5 2005-05-26
     */
    public final void testGetContentHandlerProfileLoaded()
        throws IOException, ClassNotFoundException, InstantiationException,
        IllegalAccessException {

        String eGMS = "com.mkdoc.schema.UKeGMSProfile";

        propertySet = new MockPropertySet();

        propertySet.setClass(PROFILE, Class.forName(eGMS));

        ApplicationContext context = null;

        context = commonPlugin.getApplicationContext(propertySet);

        commonPlugin.loadApplicationProfile(context);

        StoreManager manager = new MockStoreManager();

        URL url = new URL(U_ROOT_IDX);


        ContentHandler handler = commonPlugin.getContentHandler(manager,
                                                                url);

        RdfContentHandler rdfHandler = (RdfContentHandler) handler;

        String predicate = "eGMS.Addressee";

        String expanded = rdfHandler.expandPredicate(predicate);

        Schema schema = new UKeGMS();

        assertTrue(expanded.equals(schema.expandPredicate(predicate)));

    }

    /**
     *  Test get content handler without profile loaded.
     *  @exception  IOException  Should not be thrown by this test.
     *  @since  Version 0.5 2005-05-26
     */
    public final void testGetContentHandlerProfileNotLoaded()
        throws IOException {

        StoreManager manager = new MockStoreManager();

        URL url = new URL(U_ROOT_IDX);

        ContentHandler handler = commonPlugin.getContentHandler(manager,
                                                                url);

        RdfContentHandler rdfHandler = (RdfContentHandler) handler;

        String predicate = "eGMS.Addressee";

        String expanded = rdfHandler.expandPredicate(predicate);

        Schema schema = new UKeGMS();

        assertFalse(expanded.equals(schema.expandPredicate(predicate)));

        assertTrue(expanded.equals(predicate));
    }

    /**
     *  Test get content handler illegal argument
     *  @exception  IOException  Should not be thrown by this test.
     *  @since  Version 0.1 2005-03-29
     */
    public final void testGetContentHandlerNullManager()
        throws IOException {

        URL url = new URL(U_ROOT_IDX);

        try {

            commonPlugin.getContentHandler(null, url);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Test get content handler illegal argument
     *  @since  Version 0.1 2005-03-29
     */
    public final void testGetContentHandlerNullUri() {

        StoreManager manager = new MockStoreManager();

        try {

            commonPlugin.getContentHandler(manager, null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Constructor should not accept a null argument.
     *  @since  Version 0.1 2005-03-29
     */
    public final void testConstructorNullPropertySet() {

        try {

            new XhtmlStoreWriterPlugin(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  With null event should throw an
     *  IllegalArgumentException.
     *  @since  Version 0.1 2005-03-29
     */
    public final void testNotifyNullEvent() {

        try {

            commonPlugin.notify(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  With wrong type should ignore.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2005-03-29
     */
    public final void testNotifyWrongType()
        throws IOException {

        MockFetchedResource resource = new MockFetchedResource();

        resource.setUrl(new URL(URL_DOMAIN));

        MockInputStream mockInStream = new MockInputStream();

        mockInStream.setFailOnRead(true);

        resource.setInputStream(mockInStream);

        ResourceDiscoveredEvent rde = new ResourceDiscoveredEvent(resource);

        MockLog log = new MockLog();

        commonPlugin.setLog(log);

        commonPlugin.notify(rde);

        String errorLog = log.getError();

        assertTrue(errorLog.equals(""));
    }

    /**
     *  With null input stream, should throw an
     *  <code>IllegalArgumentException</code>.
     *  @since  Version 0.1 2005-03-29
     */
/*
    public final void testAddResourceNullInputStream() {

        try {

            commonPlugin.addResource(null,
                                     URL_VALID);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }
*/

    /**
     *  With null URI, should throw an
     *  <code>IllegalArgumentException</code>.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2005-03-29
     */
/*
    public final void testAddResourceNullUri() throws IOException {

        InputStream inStream = new FileInputStream(FILE_VALID);

        try {

            commonPlugin.addResource(inStream,
                                     null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }
*/

    /**
     *  With an <code>IOException</code>, should log the
     *  problem and return.
     *  @since  Version 0.1 2005-03-29
     */
/*
    public final void testAddResourceIOException() {

        MockInputStream mockInStream = new MockInputStream();

        mockInStream.setFailOnRead(true);

        MockLog log = new MockLog();

        commonPlugin.setLog(log);

        commonPlugin.addResource(mockInStream, URL_VALID);

        String errorLog = log.getError();

        assertTrue(!errorLog.equals(""));
    }
*/

    /**
     *  An irreparable source should throw a
     *  <code>SAXException</code>, log the problem and
     *  return.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2005-03-29
     */
/*
    public final void testAddResourceSAXException() throws IOException {

        InputStream inStream = new FileInputStream(FILE_BROKEN);

        MockLog log = new MockLog();

        commonPlugin.setLog(log);

        commonPlugin.addResource(inStream, URL_BROKEN);

        String errorLog = log.getError();

        assertTrue(!errorLog.equals(""));
    }
*/

    /**
     *  Test parse with a null input stream.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.4 2005-05-10
     */
    public final void testParseNullInputStream() throws IOException {

        try {

            commonPlugin.parse(null, new URL(URL_VALID));

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test parse with a null URL.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.4 2005-05-10
     */
    public final void testParseNullUrl() throws IOException {

        try {

            InputStream fileInStream = new FileInputStream(SRC_INVALID);

            commonPlugin.parse(fileInStream, null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test parse throws IOException.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.4 2005-05-10
     */
    public final void testParseThrowsIOException() throws IOException {

        MockInputStream badInStream = new MockInputStream();

        badInStream.setFailOnRead(true);

        MockLog log = new MockLog();

        commonPlugin.setLog(log);

        InputStream fileInStream =  null;

        try {

            fileInStream = new FileInputStream(SRC_INVALID);

            commonPlugin.parse(fileInStream, new URL(URL_VALID));

            String message = log.getError();

            assertTrue("Should log the error, actual log was '" +
                       message + "'.",
                       message.indexOf("Error parsing input stream") > -1);
        }
        finally {

            fileInStream.close();
        }
    }

    /**
     *  Get description, no check for literal content.
     *  @since  Version 0.1 2005-03-29
     */
    public final void testGetDescription() {

        String description = commonPlugin.getDescription();

        assertTrue((description != null) &&
                   (!description.equals("")));

    }

    /**
     *  Get version, no check for literal content.
     *  @since  Version 0.1 2005-03-29
     */
    public final void testGetVersion() {

        String version = commonPlugin.getVersion();

        assertTrue((version != null) &&
                   (!version.equals("")));
    }

    /**
     *  Get Name.
     *  @since  Version 0.1 2005-03-29
     */
    public final void testGetName() {

        String name = commonPlugin.getName();

        assertTrue((name != null) &&
                   (name.equals(
                    "XHTML metadata RDF store plugin for JSpider")));
    }

    /**
     *  Tear down a test scenario as required.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2005-
    protected final void tearDown() throws IOException {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test coverage decorator.
     *  @since  Version 0.1 2005-03-29
     */
    public static Test suite() {

        return new TestSuite(XhtmlStoreWriterPluginTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-03-29
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
