package com.mkdoc.jspider;

/*
    Created:    2005-03-01
    Issued:     2005-03-01
    Modified:   2005-10-21

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.api.event.resource.ResourceFetchedEvent;
import net.javacoding.jspider.api.event.resource.ResourceDiscoveredEvent;

import com.mkdoc.sax.RdfContentHandler;

import com.mkdoc.schema.UKeGMS;
import com.mkdoc.schema.Schema;
import com.mkdoc.schema.UKeGMSProfile;

import com.mkdoc.store.AbstractFileStoreManager;
import com.mkdoc.store.MockFileStoreManager;
import com.mkdoc.store.MockVirtualFileStoreManager;
import com.mkdoc.store.StoreManagerFactory;

import com.mkdoc.util.ApplicationContext;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;

import java.net.URL;

import org.xml.sax.ContentHandler;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>XhtmlTripleWriterPlugin</code>.
 *  @version    0.8 2005-10-21
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class XhtmlTripleWriterPluginTest extends TestCase {

    /**
     *  A subject manager class name.
     *  @since  Version 0.3 2005-04-29
     */
    public static final String CLASS_NAME =
    "com.mkdoc.store.TripleFileStoreManager";

    /**
     *  An expected <code>IllegalArgumentException</code> message.
     *  @since  Version 0.1 2005-03-01
     */
    public static final String EXPECTED_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  The schema property key name.
     *  @since  Version 0.5 2005-05-26
     */
    public static final String PROFILE = "profile";

    /**
     *  The project source directory.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String HOME = System.getProperty("mkHome");

    /**
     *  The project test source directory.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String TEST = System.getProperty("testDir") +
                                         File.separator +
                                         "html";

    /**
     *  The default JSpider output directory.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String OUTPUT = "output";

    /**
     *  A custom output directory.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String CUSTOM = "custom";

    /**
     *  The test source directory path.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String PATH_TEST = HOME +
                                              File.separator +
                                              TEST;

    /**
     *  An invalid source file for testing.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String INVALID = "invalid-Basic.html";

    /**
     *  An irreparably invalid source file for testing.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String BROKEN = "invalid-Irreparable.html";

    /**
     *  An invalid output file for testing.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String INVALID_NT = "invalid-Basic.nt";

    /**
     *  An irreparably invalid output file for testing.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String BROKEN_NT = "invalid-Irreparable.nt";

    /**
     *  A file reference for invalid source for testing.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final File F_SRC_INVLD = new File(PATH_TEST +
                                                       File.separator +
                                                       INVALID);

    /**
     *  A file reference for an irreparably invalid source for testing.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final File F_SRC_BROKEN = new File(PATH_TEST +
                                                        File.separator +
                                                        BROKEN);

    /**
     *  An valid source file for testing.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String VALID = "valid-Basic.html";

    /**
     *  An valid output file for testing.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String VALID_NT = "valid-Basic.nt";


    /**
     *  A file reference for valid source for testing.
     *  @since  Version 0.4 2005-05-10
     */
    protected static final File SRC_VALID = new File(PATH_TEST +
                                                          File.separator +
                                                          VALID);
    /**
     *  A file reference for invalid source for testing.
     *  @since  Version 0.4 2005-05-10
     */
    protected static final File SRC_INVALID = new File(PATH_TEST +
                                                       File.separator +
                                                       INVALID);

    /**
     *  The default JSpider output path.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String PATH_OUT = HOME +
                                             File.separator +
                                             OUTPUT;


    /**
     *  The custom output path.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String PATH_CUSTOM = HOME +
                                                File.separator +
                                                OUTPUT +
                                                File.separator +
                                                CUSTOM;

    /**
     *  A <code>File</code> representation of the default
     *  output path for comparisons.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final File DIR_OUT = new File(PATH_OUT);

    /**
     *  A <code>File</code> representation of the home
     *  directory for comparisons.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final File DIR_HOME = new File(HOME);

    /**
     *  A <code>File</code> representation of the custom
     *  output directory for comparisons.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final File DIR_CUSTOM = new File(PATH_CUSTOM);

    /**
     *  A test domain.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String DOMAIN = System.getProperty("testDomain");


    /**
     *  A test domain URL.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String U_DOMAIN = "http://" + DOMAIN;

    /**
     *  An expected output directory for the test domain.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String PATH_DOMAIN = PATH_CUSTOM +
                                                File.separator +
                                                DOMAIN;

    /**
     *  A File reference for the expected output directory
     *  for the test domain.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final File DIR_DOMAIN = new File(PATH_DOMAIN);

    /**
     *  A test domain root directory URL.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String U_ROOT = U_DOMAIN + "/";

    /**
     *  A root index source file.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String INDEX = "index.html";

    /**
     *  A root index output file.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String INDEX_NT = "index.nt";

    /**
     *  A test domain root index URL.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String U_ROOT_IDX = U_ROOT + INDEX;

    /**
     *  A valid source URL.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String U_VALID = U_ROOT + VALID;

    /**
     *  An invalid source URL.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String U_INVALID = U_ROOT + INVALID;


    /**
     *  An irreparably invalid source URL.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final String U_BROKEN = U_ROOT + BROKEN;

    /**
     *  A file reference for the actual root index.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final File F_ROOT_INDEX = new File(PATH_DOMAIN +
                                                        File.separator +
                                                        INDEX_NT);

    /**
     *  A file reference for invalid output file.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final File F_OUT_INVLD = new File(PATH_DOMAIN +
                                                       File.separator +
                                                       INVALID_NT);

    /**
     *  A file reference for invalid output file.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final File F_OUT_BROKEN = new File(PATH_DOMAIN +
                                                           File.separator +
                                                           BROKEN_NT);

    /**
     *  A file reference for valid output file.
     *  @since  Version 0.1 2005-03-01
     */
    protected static final File F_OUT_VALID = new File(PATH_DOMAIN +
                                                          File.separator +
                                                          VALID_NT);

    /**
     *  A pre-configured plugin to use in multiple tests.
     *  @since  Version 0.1 2005-03-01
     */
    protected XhtmlTripleWriterPlugin commonPlugin;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-
     */
    public XhtmlTripleWriterPluginTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @exception  ClassNotFoundException  Should not be thrown.
     *  @since  Version 0.1 2005-
     */
    protected final void setUp() throws ClassNotFoundException {

        MockPropertySet propertySet = new MockPropertySet();

        Class managerClass = Class.forName(CLASS_NAME);

        propertySet.setClass(StoreManagerFactory.MANAGER_KEY,
                             managerClass);

        propertySet.setBoolean(AbstractFileStoreManager.ABSOLUTE, true);
        propertySet.setString(AbstractFileStoreManager.FOLDER, PATH_CUSTOM);

        commonPlugin = new XhtmlTripleWriterPlugin(propertySet);

    }

    /**
     *  Test constructor with custom profile exception.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @exception  ClassNotFoundException Should not be thrown by this test.
     *  @since  Version 0.5 2005-05-26
     */
    public final void testConstructorCustomProfileException()
        throws IOException, ClassNotFoundException {

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setBoolean(AbstractFileStoreManager.ABSOLUTE, false);
        propertySet.setString(AbstractFileStoreManager.FOLDER, CUSTOM);
        propertySet.setClass(PROFILE, Class.forName("java.lang.String"));

        XhtmlTripleWriterPlugin plugin = null;

        plugin = new XhtmlTripleWriterPlugin(propertySet);

        assertTrue((plugin != null) &&
                   (plugin.profile == null));
    }

    /**
     *  Test constructor with custom profile exception.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @exception  ClassNotFoundException  Should not be thrown.
     *  @since  Version 0.5 2005-05-26
     */
    public final void testConstructorCustomProfileSuccess()
        throws IOException, ClassNotFoundException {

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setBoolean(AbstractFileStoreManager.ABSOLUTE, false);
        propertySet.setString(AbstractFileStoreManager.FOLDER, CUSTOM);
        propertySet.setClass(PROFILE,
                             Class.forName("com.mkdoc.schema.UKeGMSProfile"));

        XhtmlTripleWriterPlugin plugin = null;

        plugin = new XhtmlTripleWriterPlugin(propertySet);

        assertTrue((plugin != null) &&
                   (plugin.profile != null) &&
                   (plugin.profile instanceof UKeGMSProfile));
    }

    /**
     *  Test load custom profile with null property set.
     *  @exception  ClassNotFoundException Should not be thrown by this test.
     *  @exception  InstantiationException Should not be thrown by this test.
     *  @exception  IllegalAccessException Should not be thrown by this test.
     *  @since  Version 0.5 2005-05-26
     */
    public final void testLoadCustomProfileNullPropertySet()
        throws ClassNotFoundException, InstantiationException,
        IllegalAccessException {

        try {

            commonPlugin.loadApplicationProfile(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test load custom profile with none configured.
     *  @exception  ClassNotFoundException Should not be thrown by this test.
     *  @exception  InstantiationException Should not be thrown by this test.
     *  @exception  IllegalAccessException Should not be thrown by this test.
     *  @since  Version 0.5 2005-05-26
     */
    public final void testLoadCustomProfileNoneConfigured()
        throws ClassNotFoundException, InstantiationException,
        IllegalAccessException {

        MockPropertySet propertySet = new MockPropertySet();

        ApplicationContext context = null;

        context = AbstractMKDocPlugin.getApplicationContext(propertySet);

        String log = commonPlugin.loadApplicationProfile(context);

        assertTrue(log.equals("No custom application profile configured."));

    }

    /**
     *  Test load custom profile with none configured.
     *  @exception  ClassNotFoundException Should not be thrown by this test.
     *  @exception  InstantiationException Should not be thrown by this test.
     *  @exception  IllegalAccessException Should not be thrown by this test.
     *  @since  Version 0.5 2005-05-26
     */
    public final void testLoadCustomProfileSuccess()
        throws ClassNotFoundException, InstantiationException,
        IllegalAccessException {

        String eGMS = "com.mkdoc.schema.UKeGMSProfile";

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setClass(PROFILE, Class.forName(eGMS));

        ApplicationContext context = null;

        context = AbstractMKDocPlugin.getApplicationContext(propertySet);

        String log = commonPlugin.loadApplicationProfile(context);

        assertTrue(log.indexOf(eGMS) > -1);

    }

    /**
     *  Test get content handler with profile loaded.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  ClassNotFoundException Should not be thrown by this test.
     *  @exception  InstantiationException Should not be thrown by this test.
     *  @exception  IllegalAccessException Should not be thrown by this test.
     *  @since  Version 0.1 2005-03-01
     */
    public final void testGetContentHandlerProfileLoaded()
        throws IOException, ClassNotFoundException, InstantiationException,
        IllegalAccessException {

        String eGMS = "com.mkdoc.schema.UKeGMSProfile";

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setClass(PROFILE, Class.forName(eGMS));

        ApplicationContext context = null;

        context = AbstractMKDocPlugin.getApplicationContext(propertySet);

        commonPlugin.loadApplicationProfile(context);

        MockFileStoreManager manager = new MockFileStoreManager();

        URL url = new URL(U_ROOT_IDX);

        ContentHandler handler = commonPlugin.getContentHandler(manager,
                                                                url);

        RdfContentHandler rdfHandler = (RdfContentHandler) handler;

        String predicate = "eGMS.Addressee";

        String expanded = rdfHandler.expandPredicate(predicate);

        Schema schema = new UKeGMS();

        assertTrue(expanded.equals(schema.expandPredicate(predicate)));

    }

    /**
     *  Test get content handler, null subject manager.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.5 2005-05-26
     */
    public final void testGetContentHandlerNullSubjectManager()
        throws IOException {

        try {

            commonPlugin.getContentHandler(
                                        null,
                                        new URL("http://www.remote.example/"));

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test get content handler, null URL.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.5 2005-05-26
     */
    public final void testGetContentHandlerNullUrl()
        throws IOException {

        try {

            commonPlugin.getContentHandler(new MockFileStoreManager(),
                                           null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }



    /**
     *  Test get content handler without profile loaded.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  ClassNotFoundException Should not be thrown by this test.
     *  @exception  InstantiationException Should not be thrown by this test.
     *  @exception  IllegalAccessException Should not be thrown by this test.
     *  @since  Version 0.5 2005-05-26
     */
    public final void testGetContentHandlerProfileNotLoaded()
        throws IOException, ClassNotFoundException, InstantiationException,
        IllegalAccessException {

        MockFileStoreManager manager = new MockFileStoreManager();

        URL url = new URL(U_ROOT_IDX);


        ContentHandler handler = commonPlugin.getContentHandler(manager,
                                                                url);

        RdfContentHandler rdfHandler = (RdfContentHandler) handler;

        String predicate = "eGMS.Addressee";

        String expanded = rdfHandler.expandPredicate(predicate);

        Schema schema = new UKeGMS();

        assertFalse(expanded.equals(schema.expandPredicate(predicate)));

        assertTrue(expanded.equals(predicate));

    }

    /**
     *  Constructor should not accept a null argument.
     *  @since  Version 0.1 2005-03-01
     */
    public final void testConstructorNullPropertySet() {

        try {

            new XhtmlTripleWriterPlugin(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Constructor should run successfully with empty
     *  property set.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2005-03-01
     */
/*
    public final void testConstructorEmptyPropertySet()
        throws IOException {

        MockPropertySet propertySet = new MockPropertySet();

        XhtmlTripleWriterPlugin plugin = null;

        plugin = new XhtmlTripleWriterPlugin(propertySet);

        assertTrue((plugin != null) &&
                   (plugin instanceof XhtmlTripleWriterPlugin));

        String outPath = plugin.outputDir.getCanonicalPath();

        assertTrue("Should use default output directory, '" +
                   DIR_OUT.getCanonicalPath() + "',\n" +
                   "actual output path =             '" +
                   outPath + "'",
                   outPath.equals(DIR_OUT.getCanonicalPath()));
    }
*/

    /**
     *  With absolute file path but no path specification,
     *  should try the default output directory.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2005-03-01
     */
/*
    public final void testConstructorAbsolutePathNullPath()
        throws IOException {

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setBoolean(XhtmlTripleWriterPlugin.ABSOLUTE, true);

        XhtmlTripleWriterPlugin plugin = null;

        plugin = new XhtmlTripleWriterPlugin(propertySet);

        assertTrue((plugin != null) &&
                   (plugin instanceof XhtmlTripleWriterPlugin));

        String outPath = plugin.outputDir.getCanonicalPath();

        assertTrue("Should use the default output directory, '" +
                   DIR_OUT.getCanonicalPath() + "',\n" +
                   "actual output path =             '" +
                   outPath + "'",
                   outPath.equals(DIR_OUT.getCanonicalPath()));
    }
*/

    /**
     *  With absolute file path and a valid path specification,
     *  should use the specified path.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2005-03-01
     */
/*
    public final void testConstructorAbsolutePathWithPath()
        throws IOException {

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setBoolean(XhtmlTripleWriterPlugin.ABSOLUTE, true);
        propertySet.setString(XhtmlTripleWriterPlugin.FOLDER, PATH_CUSTOM);

        XhtmlTripleWriterPlugin plugin = null;

        plugin = new XhtmlTripleWriterPlugin(propertySet);

        assertTrue((plugin != null) &&
                   (plugin instanceof XhtmlTripleWriterPlugin));

        String outPath = plugin.outputDir.getCanonicalPath();

        assertTrue("Should use test output directory, '" +
                   DIR_CUSTOM.getCanonicalPath() + "',\n" +
                   "actual output path =             '" +
                   outPath + "'",
                   outPath.equals(DIR_CUSTOM.getCanonicalPath()));
    }
*/

    /**
     *  With relative file path but no path specification,
     *  should try the default output directory.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2005-03-01
     */
/*
    public final void testConstructorRelativePathNullPath()
        throws IOException {

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setBoolean(XhtmlTripleWriterPlugin.ABSOLUTE, false);

        XhtmlTripleWriterPlugin plugin = null;

        plugin = new XhtmlTripleWriterPlugin(propertySet);

        assertTrue((plugin != null) &&
                   (plugin instanceof XhtmlTripleWriterPlugin));

        String outPath = plugin.outputDir.getCanonicalPath();

        assertTrue("Should use the default output directory, '" +
                   DIR_OUT.getCanonicalPath() + "',\n" +
                   "actual output path =             '" +
                   outPath + "'",
                   outPath.equals(DIR_OUT.getCanonicalPath()));
    }
*/

    /**
     *  With relative file path and a valid path specification,
     *  should use the specified path.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2005-03-01
     */
/*
    public final void testConstructorRelativePathWithPath()
        throws IOException {

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setBoolean(XhtmlTripleWriterPlugin.ABSOLUTE, false);
        propertySet.setString(XhtmlTripleWriterPlugin.FOLDER, CUSTOM);

        XhtmlTripleWriterPlugin plugin = null;

        plugin = new XhtmlTripleWriterPlugin(propertySet);

        assertTrue((plugin != null) &&
                   (plugin instanceof XhtmlTripleWriterPlugin));

        String outPath = plugin.outputDir.getCanonicalPath();

        assertTrue("Should use test output directory, '" +
                   DIR_CUSTOM.getCanonicalPath() + "',\n" +
                   "actual output path =             '" +
                   outPath + "'",
                   outPath.equals(DIR_CUSTOM.getCanonicalPath()));
    }
*/

    /**
     *  With null event should throw an
     *  IllegalArgumentException.
     *  @since  Version 0.1 2005-03-01
     */
    public final void testNotifyNullEvent() {

        ResourceDiscoveredEvent nullEvent = null;

        try {

            commonPlugin.notify(nullEvent);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  With wrong type should ignore.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2005-03-01
     */
    public final void testNotifyWrongType()
        throws IOException {

        MockFetchedResource resource = new MockFetchedResource();

        resource.setUrl(new URL(U_DOMAIN));

        InputStream fileInStream = new FileInputStream(F_SRC_INVLD);

        resource.setInputStream(fileInStream);

        ResourceDiscoveredEvent rde = new ResourceDiscoveredEvent(resource);

        commonPlugin.notify(rde);

        assertFalse("Should not output file to path for this event type'" +
                    F_ROOT_INDEX.getCanonicalPath() + "'",
                    F_ROOT_INDEX.exists());

    }

    /**
     *  Should use the default <code>index.html</code> name
     *  for the output file.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2005-03-01
     */
    public final void testNotifyResourceNoFile()
        throws IOException {

        MockFetchedResource resource = new MockFetchedResource();

        resource.setUrl(new URL(U_DOMAIN));

        InputStream fileInStream = new FileInputStream(F_SRC_INVLD);

        resource.setInputStream(fileInStream);

        ResourceFetchedEvent rfe = new ResourceFetchedEvent(resource);

        commonPlugin.notify(rfe);

        assertTrue("Should output file to path '" +
                   F_ROOT_INDEX.getCanonicalPath() + "'",
                   F_ROOT_INDEX.exists());

    }

    /**
     *  Should use the file part of the URL for the output file name.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2005-03-01
     */
    public final void testNotifyResourceWithFile()
        throws IOException {

        MockFetchedResource resource = new MockFetchedResource();

        resource.setUrl(new URL(U_VALID));

        InputStream fileInStream = new FileInputStream(F_SRC_INVLD);

        resource.setInputStream(fileInStream);

        ResourceFetchedEvent rfe = new ResourceFetchedEvent(resource);

        commonPlugin.notify(rfe);

        assertTrue("Should output file to path '" +
                   F_OUT_VALID.getCanonicalPath() + "'",
                   F_OUT_VALID.exists());
    }

    /**
     *  With null input stream, should throw an
     *  <code>IllegalArgumentException</code>.
     *  @since  Version 0.1 2005-03-01
     */
/*
    public final void testWriteFileNullInputStream() {

        try {

            commonPlugin.writeFile(null,
                                   F_OUT_VALID,
                                   U_VALID);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }
*/

    /**
     *  With null file, should throw an
     *  <code>IllegalArgumentException</code>.
     *  @exception  FileNotFoundException   If the input
     *          source file is not found.
     *  @since  Version 0.1 2005-03-01
     */
/*
    public final void testWriteFileNullFile()
        throws FileNotFoundException {

        InputStream inStream = new FileInputStream(SRC_VALID);

        try {

            commonPlugin.writeFile(inStream,
                                   null,
                                   U_VALID);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }
*/

    /**
     *  With null URI, should throw an
     *  <code>IllegalArgumentException</code>.
     *  @exception  FileNotFoundException   If the input
     *          source file is not found.
     *  @since  Version 0.1 2005-03-01
     */
/*
    public final void testWriteFileNullUri()
        throws FileNotFoundException {

        InputStream inStream = new FileInputStream(SRC_VALID);

        try {

            commonPlugin.writeFile(inStream,
                                   F_OUT_VALID,
                                   null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }
*/

    /**
     *  With file not found, should log the problem and
     *  return.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2005-03-01
     */
/*
    public final void testWriteFileNotFoundException()
        throws IOException {

        InputStream inStream = new FileInputStream(SRC_VALID);

        // Parent directories not created
        commonPlugin.writeFile(inStream, F_OUT_VALID, U_VALID);

        assertFalse("Should not output file to path '" +
                    F_OUT_VALID.getCanonicalPath() + "'",
                    F_OUT_VALID.exists());

    }
*/

    /**
     *  With an <code>IOException</code>, should log the
     *  problem and return.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2005-03-01
     */
/*
    public final void testWriteIOException()
        throws IOException {

        InputStream inStream = new FileInputStream(SRC_VALID);

        commonPlugin.createDirectories(F_OUT_VALID);

        assertTrue("Could not create a new output file.",
                   F_OUT_VALID.createNewFile());

        assertTrue("Could not set output file read only.",
                   F_OUT_VALID.setReadOnly());

        commonPlugin.writeFile(inStream, F_OUT_VALID, U_VALID);

        long fileSize = F_OUT_VALID.length();

        assertTrue(fileSize == 0L);
    }
*/

    /**
     *  An irreparable source should throw a
     *  <code>SAXException</code>, log the problem and
     *  return.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2005-03-01
     */
/*
    public final void testWriteSAXException()
        throws IOException {

        InputStream inStream = new FileInputStream(F_SRC_BROKEN);

        commonPlugin.createDirectories(F_OUT_BROKEN);

        commonPlugin.writeFile(inStream, F_OUT_BROKEN, U_BROKEN);

        assertTrue("Output file should still be written with no triples: '" +
                   F_OUT_BROKEN.getCanonicalPath() + "'",
                   F_OUT_BROKEN.exists());
    }
*/

    /**
     *  Test parse with a null input stream.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.4 2005-05-10
     */
    public final void testParseNullInputStream() throws IOException {

        try {

            commonPlugin.parse(null, new URL(U_VALID));

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test parse with a null URL.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.4 2005-05-10
     */
    public final void testParseNullUrl() throws IOException {

        try {

            InputStream fileInStream = new FileInputStream(SRC_INVALID);

            commonPlugin.parse(fileInStream, null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test parse throws IOException.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.4 2005-05-10
     */
    public final void testParseThrowsIOException() throws IOException {

        MockVirtualFileStoreManager badManager = null;

        badManager = new MockVirtualFileStoreManager();

        badManager.setFailOnGetOutput(true);

        commonPlugin.manager = badManager;

        MockLog log = new MockLog();

        commonPlugin.setLog(log);

        InputStream fileInStream =  null;

        try {

            fileInStream = new FileInputStream(SRC_INVALID);

            commonPlugin.parse(fileInStream, new URL(U_VALID));

            String message = log.getError();

            assertTrue("Should log the error, actual log was '" +
                       message + "'.",
                       message.indexOf("Could not write output") > -1);
        }
        finally {

            fileInStream.close();
        }
    }


    /**
     *  Get description, no check for literal content.
     *  @since  Version 0.1 2005-03-01
     */
    public final void testGetDescription() {

        String description = commonPlugin.getDescription();

        assertTrue((description != null) &&
                   (!description.equals("")));

    }

    /**
     *  Get version, no check for literal content.
     *  @since  Version 0.1 2005-03-01
     */
    public final void testGetVersion() {

        String version = commonPlugin.getVersion();

        assertTrue((version != null) &&
                   (!version.equals("")));
    }

    /**
     *  Get Name.
     *  @since  Version 0.1 2005-03-01
     */
    public final void testGetName() {

        String name = commonPlugin.getName();

        assertTrue((name != null) &&
                   (name.equals(
                    "XHTML metadata triple writer plugin for JSpider")));
    }

    /**
     *  Tear down a test scenario as required.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2005-12-03
     */
    protected final void tearDown() throws IOException {

        if (F_ROOT_INDEX.exists()) {

            assertTrue("Could not delete root index file '" +
                       F_ROOT_INDEX.getCanonicalPath() + "'",
                       F_ROOT_INDEX.delete());
        }
        if (F_OUT_VALID.exists()) {

            assertTrue("Could not delete valid output file '" +
                       F_OUT_VALID.getCanonicalPath() + "'",
                       F_OUT_VALID.delete());
        }
        if (F_OUT_INVLD.exists()) {

            assertTrue("Could not delete invalid output file '" +
                       F_OUT_INVLD.getCanonicalPath() + "'",
                       F_OUT_INVLD.delete());
        }
        if (F_OUT_BROKEN.exists()) {

            assertTrue("Could not delete irreparable output file '" +
                       F_OUT_BROKEN.getCanonicalPath() + "'",
                       F_OUT_BROKEN.delete());
        }
        if (DIR_DOMAIN.exists()) {

            assertTrue("Could not delete the test domain output directory '" +
                       DIR_DOMAIN.getCanonicalPath() + "'",
                       DIR_DOMAIN.delete());
        }
        if (DIR_CUSTOM.exists()) {

            assertTrue("Could not delete the custom output directory '" +
                       DIR_CUSTOM.getCanonicalPath() + "'",
                       DIR_CUSTOM.delete());
        }
    }

    /**
     *  Create a test suite for this test case.
     *  @return A test coverage decorator.
     *  @since  Version 0.1 2005-03-01
     */
    public static Test suite() {

        return new TestSuite(XhtmlTripleWriterPluginTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-03-01
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
