package com.mkdoc.query;

/*
    Created:    2005-07-26
    Issued:     2005-07-26
    Modified:   2005-10-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>QueryFactory</code>.
 *  @version    0.3 2005-10-28
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class QueryFactoryTest extends TestCase {

    /**
     *  An expected <code>IllegalArgumentException</code>.
     *  @since  Version 0.1 2005-07-26
     */
    public static final String EXPECTED_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  Results per page constant.
     *  @since  Version 0.2 2005-08-02
     */
    public static final int RESULTS_PP = 10;

    /**
     *  Dummy query string.
     *  @since  Version 0.1 2005-07-26
     */
    public static final String QUERY = "Test query";

    /**
     *  Dummy query URI.
     *  @since  Version 0.1 2005-07-26
     */
    public static final String URI = "http://www.remote.example/uri/test";

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-07-26
     */
    public QueryFactoryTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-07-26
    protected void setUp() {

    }
     */

    /**
     *  Get a result renderer.
     *  @since  Version 0.1 2005-07-26
     */
    public final void testGetResultRendererXhtmlType() {

        QueryContext context = new XhtmlQueryContext(1, RESULTS_PP);

        SimpleQuery query = new SimpleQuery(QUERY,
                                            URI,
                                            context);

        ResultRenderer renderer = QueryFactory.getResultRenderer(query);

        assertTrue((renderer != null) &&
                   (renderer instanceof XhtmlResultRenderer));
    }

    /**
     *  Get a result renderer.
     *  @since  Version 0.1 2005-07-26
     */
    public final void testGetResultRendererTextType() {

        QueryContext context = new TextQueryContext(1, RESULTS_PP);

        SimpleQuery query = new SimpleQuery(QUERY,
                                            URI,
                                            context);

        ResultRenderer renderer = QueryFactory.getResultRenderer(query);

        assertTrue((renderer != null) &&
                   (renderer instanceof TextResultRenderer));
    }

    /**
     *  Get a result renderer.
     *  @since  Version 0.1 2005-07-26
     */
    public final void testGetResultRendererRssType() {

        QueryContext context = new RssQueryContext(1, RESULTS_PP);

        SimpleQuery query = new SimpleQuery(QUERY,
                                            URI,
                                            context);

        ResultRenderer renderer = QueryFactory.getResultRenderer(query);

        assertTrue((renderer != null) &&
                   (renderer instanceof RssResultRenderer));
    }

    /**
     *  Get a result renderer.
     *  @since  Version 0.1 2005-07-26
     */
    public final void testGetResultRendererNullQuery() {

        SimpleQuery query = null;

        try {

            QueryFactory.getResultRenderer(query);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-07-26
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-07-26
     */
    public static Test suite() {

        return new TestSuite(QueryFactoryTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-07-26
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
