package com.mkdoc.query;

/*
    Created:    2005-10-28
    Issued:     2005-10-28
    Modified:   2005-10-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>RssResultRenderer</code>.
 *  @version    0.1 2005-10-28
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class RssResultRendererTest extends TestCase {

    /**
     *  The &quot;actual markup&quot; test failure report.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String ACTUAL = "', actual output was:\n\n";

    /**
     *  The default page index.
     *  @since  Version 0.1 2005-10-28
     */
    public static final int PAGE_INDEX = 1;

    /**
     *  The default results per page limit.
     *  <p>
     *    Low to make &quot;extreme&quot; paging possible with fewer results.
     *  </p>
     *  @since  Version 0.1 2005-10-28
     */
    public static final int RESULT_LIMIT = 2;

    /**
     *  The number of results pages limit tests.
     *  @since  Version 0.1 2005-10-28
     */
    public static final int LIMIT_RESULTS = 22;

    /**
     *  The first page number.
     *  @since  Version 0.1 2005-10-28
     */
    public static final int FIRST = 1;

    /**
     *  The second page number.
     *  @since  Version 0.1 2005-10-28
     */
    public static final int SECOND = 2;

    /**
     *  The third page number.
     *  @since  Version 0.1 2005-10-28
     */
    public static final int THIRD = 3;

    /**
     *  The fourth page number.
     *  @since  Version 0.1 2005-10-28
     */
    public static final int FOURTH = 4;

    /**
     *  The fifth page number.
     *  @since  Version 0.1 2005-10-28
     */
    public static final int FIFTH = 5;

    /**
     *  The sixth page number.
     *  @since  Version 0.1 2005-10-28
     */
    public static final int SIXTH = 6;

    /**
     *  The tenth page number.
     *  @since  Version 0.1 2005-10-28
     */
    public static final int TENTH = 10;

    /**
     *  The eleventh page number.
     *  @since  Version 0.1 2005-10-28
     */
    public static final int ELEVENTH = 11;

    /**
     *  An SeRQL query.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String QUERY_STRING =
    "SELECT DISTINCT targetUri, createdDate, dcSubject, targetTitle, " +
    "targetSummary FROM {targetUri} dcterms:created {createdDate}, " +
    "     {targetUri} dc:subject {dcSubject}," +
    "     [ {targetUri} dc:title {targetTitle} ], " +
    "     [ {targetUri} dc:description {targetSummary} ] " +
    "WHERE (label(dcSubject) LIKE \"GCJ\" OR " +
    "       label(dcSubject) LIKE \"JSpider\") " +
    "AND label(createdDate) LIKE \"2005-*\" " +
    "UNION " +
    "SELECT DISTINCT targetUri, createdDate, dcSubject, targetTitle,  " +
    "targetSummary FROM {targetUri} dcterms:created {} rdf:value  " +
    "{createdDate}, {targetUri} dc:subject {dcSubject}, " +
    "     [ {targetUri} dc:title {targetTitle} ], " +
    "     [ {targetUri} dc:description {targetSummary} ] " +
    "WHERE (label(dcSubject) LIKE \"GCJ\" OR " +
    "       label(dcSubject) LIKE \"JSpider\") " +
    "AND label(createdDate) LIKE \"2005-*\" " +
    "USING NAMESPACE dc = <http://purl.org/dc/elements/1.1/>, " +
    "            dcterms = <http://purl.org/dc/terms/> ";

    /**
     *  A result key for testing.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String RESULT_KEY =
    "http://localhost/HttpQuery?dcterms.created=2005-*&amp;dc.subject=" +
    "GCJ+JSpider";

    /**
     *  A result item title.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String TITLE = "Item title";

    /**
     *  A result item summary.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String SUMMARY = "Item summary";

    /**
     *  A result item URI
     *  @since  Version 0.1 2005-10-28
     */
    public static final String URI = "http://remote.example/uri/result-item";

    /**
     *  An XHTML query context.
     *  @since  Version 0.1 2005-10-28
     */
    protected RssQueryContext context;

    /**
     *  A query object.
     *  @since  Version 0.1 2005-10-28
     */
    protected Query query;

    /**
     *  A renderer to test.
     *  @since  Version 0.1 2005-10-28
     */
    protected RssResultRenderer renderer;

    /**
     *  A query result to test.
     *  @since  Version 0.1 2005-10-28
     */
    protected MockQueryResult result;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-10-28
     */
    public RssResultRendererTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-10-28
     */
    protected final void setUp() {

        context = new RssQueryContext(PAGE_INDEX,
                                       RESULT_LIMIT);

        query = new SimpleQuery(QUERY_STRING,
                                RESULT_KEY,
                                context);

        renderer = new RssResultRenderer(query);

    }

    /**
     *  Test basic result listing.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testGetQueryResultFullMode() {

        result = new MockQueryResult(TITLE, SUMMARY, URI, FIRST);

        StringBuffer buffer = renderer.getQueryResult(result, renderer.FULL);

        String markup = buffer.toString();

        assertTrue("Markup should include the document header " +
                   ACTUAL +
                   markup,
                   markup.indexOf(RssResultRenderer.HEADER) > -1);

        assertTrue("Markup should include the document footer " +
                   ACTUAL +
                   markup,
                   markup.indexOf(RssResultRenderer.FOOTER) > -1);

    }

    /**
     *  Test query result with null result.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testGetQueryResultNullResult() {

        try {

            renderer.getQueryResult(null,
                                    renderer.INSERT);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }


    /**
     *  Test basic result listing.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testGetQueryResultInsertMode() {

        result = new MockQueryResult(TITLE, SUMMARY, URI, FIRST);

        StringBuffer buffer = renderer.getQueryResult(result, renderer.INSERT);

        String markup = buffer.toString();

        assertTrue("Markup should not include the document header " +
                   ACTUAL +
                   markup,
                   markup.indexOf(RssResultRenderer.HEADER) == -1);

        assertTrue("Markup should not include the document footer " +
                   ACTUAL +
                   markup,
                   markup.indexOf(RssResultRenderer.FOOTER) == -1);
    }

    /**
     *  Test the constructor with a null argument.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testConstructorNullQuery() {

        try {

            new RssResultRenderer(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test the content type.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testGetContentType() {

        String type = renderer.getContentType();

        assertTrue(type.equals("application/rss+xml"));
    }

    /**
     *  Test basic result listing.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testGetQueryResultNoItems() {

        result = new MockQueryResult(TITLE, SUMMARY, URI, 0);

        StringBuffer buffer = renderer.getQueryResult(result, renderer.INSERT);

        String markup = buffer.toString();

        String expected = "No results";

        assertTrue("Text should include the summary statement '" +
                   expected + ACTUAL +
                   markup,
                   markup.indexOf(expected) > -1);
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-10-28
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-10-28
     */
    public static Test suite() {

        return new TestSuite(RssResultRendererTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-10-28
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
