package com.mkdoc.query;

/*
    Created:    2005-10-28
    Issued:     2005-10-28
    Modified:   2005-10-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>XhtmlQueryContext</code>.
 *  @version    0.1 2005-10-28
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class XhtmlQueryContextTest extends TestCase {

    /**
     *  Custom page limit.
     *  @since  Version 0.1 2005-10-28
     */
    public static final int PAGE_LIMIT = 5;

    /**
     *  Custom page number.
     *  @since  Version 0.1 2005-10-28
     */
    public static final int PAGE_NO = 5;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-10-28
     */
    public XhtmlQueryContextTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-10-28
    protected final void setUp() {

    }
     */

    /**
     *  Test the constructor.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testConstructorNegativePage() {

        XhtmlQueryContext context = new XhtmlQueryContext(-1, PAGE_LIMIT);

        int page = context.getPageNumber();

        assertTrue(page == 1);
    }

    /**
     *  Test the constructor.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testConstructorNegativeLimit() {

        XhtmlQueryContext context = new XhtmlQueryContext(PAGE_NO, -1);

        int limit = context.getResultsPerPage();

        assertTrue(limit == context.LIMIT);
    }

    /**
     *  Test the constructor.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testConstructorValid() {

        XhtmlQueryContext context = new XhtmlQueryContext(PAGE_NO, PAGE_LIMIT);

        int limit = context.getResultsPerPage();

        assertTrue(limit == PAGE_LIMIT);

        int page = context.getPageNumber();

        assertTrue(page == PAGE_NO);
    }

      /**
     *  Test the output type.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testGetOutputType() {

        XhtmlQueryContext context = new XhtmlQueryContext(PAGE_NO, PAGE_LIMIT);

        String type = context.getOutputType();

        assertTrue("Output type should be 'html', actual type was '" +
                   type + "'.",
                   type.equals("html"));
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-10-28
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-10-28
     */
    public static Test suite() {

        return new TestSuite(XhtmlQueryContextTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-10-28
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
