package com.mkdoc.sax;

/*
    Created:    2004-10-25
    Issued:     2004-10-25
    Modified:   2005-01-07

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.io.IOException;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 *  A mock <code>XMLReader</code> for testing only.
 *  @version    0.1 2004-10-25
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockXMLReader extends AbstractXMLReader implements XMLReader {

    /**
     *  Test feature number 1, locked true.
     *  @since  version 0.1 2004-10-25
     */
    public static final String TEST1 =
    "http://www.mkdoc.com/uri/sax/feature/test1";

    /**
     *  Test feature number 2, locked false.
     *  @since  version 0.1 2004-10-25
     */
    public static final String TEST2 =
    "http://www.mkdoc.com/uri/sax/feature/test2";

    /**
     *  Test property number 1, locked true.
     *  @since  version 0.1 2004-10-25
     */
    public static final String TEST3 =
    "http://www.mkdoc.com/uri/sax/property/test3";

    /**
     *  Test property number 2, un-locked false.
     *  @since  version 0.1 2004-10-25
     */
    public static final String TEST4 =
    "http://www.mkdoc.com/uri/sax/property/test4";

    /**
     *  An exception to be thrown by the parse methods.
     *  @since  Version 0.1 2004-10-25
     */
    private String exception = null;

    /**
     *  Set test configuration.
     *  @exception  SAXException    Should not be thrown.
     *  @since  Version 0.1 2004-10-25
     */
    public MockXMLReader() throws SAXException {

        features.put(NAMESPACES,
                     new Feature(false));

        features.put(PREFIXES,
                     new Feature(false));


        Feature testFeature1 = new Feature();

        testFeature1.setSupported(false);

        features.put(TEST1, testFeature1);

        Feature testFeature2 = new Feature(false);

        testFeature2.setSupported(false);

        features.put(TEST2, testFeature2);


        String value1 = new String("Property 1 value");

        Property testProperty1 = new Property(value1);

        testProperty1.setSupported(false);

        properties.put(TEST3, testProperty1);


        String value2 = new String("Property 2 value");

        Property testProperty2 = new Property(value2);

        // Support enabled by default

        properties.put(TEST4, testProperty2);

    }

    /**
     *  Parse an input source.
     *  <p>
     *    Throws an exception if requested, else does
     *    nothing.
     *  </p>
     *  @param  input   The source to parse.
     *  @exception  IOException If there is a problem reading the
     *          input source.
     *  @exception  SAXException    If an error occurs
     *          parsing the input.
     *  @since  Version 0.1 2004-10-25
     */
    public final void parse(final InputSource input)
        throws IOException, SAXException {

        if (exception != null) {

            if (exception.equals("SAXException")) {

                throw new SAXException("MockXMLReader throws SAXException.");
            }
            else {

                throw new IOException("MockXMLReader throws IOException.");
            }
        }
        // Else do nothing
    }

    /**
     *  Set an exception to be thrown by the parse methods.
     *  @param  name    The class name of the exception to
     *          be thrown, either <code>IOException</code>
     *          or <code>SAXException</code>.
     *  @since  Version 0.1 2004-10-25
     */
    public final void setException(final String name) {

        if (name != null) {

            if ((name.equals("IOException")) ||
                (name.equals("SAXException"))) {

                exception = name;

            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null exception name not permitted.");
        }
    }

}
