package com.mkdoc.schema;

/*
    Created:    2005-10-19
    Issued:     2005-10-19
    Modified:   2005-10-19

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>AbstractSchemaProperty</code>.
 *  @version    0.1 2005-10-19
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class AbstractSchemaPropertyTest extends TestCase {

    /**
     *  The resource URI.
     *  @since  Version 0.1 2005-10-19
     */
    protected static final String URI =
    "http://test.mksearch.mkdoc.org/uri/AbstractSchemaPropertyTest";

    /**
     *  The resource label.
     *  @since  Version 0.1 2005-10-19
     */
    protected static final String LABEL = "Abstract schema property test";

    /**
     *  The unique resource name.
     *  @since  Version 0.1 2005-10-19
     */
    protected static final String NAME = "abstractSchemaPropertyTest";

    /**
     *  A name name variable.
     *  @since  Version 0.1 2005-10-19
     */
    protected static final String NAME_VAR = "testAbstractSchemaPropertyTest";

    /**
     *  A bNode name variable.
     *  @since  Version 0.1 2005-10-19
     */
    protected static final String BNODE_VAR =
    "testAbstractSchemaPropertyTestBNode";

    /**
     *  A parameter name.
     *  @since  Version 0.1 2005-10-19
     */
    protected static final String PARAM = "test.abstractSchemaPropertyTest";

    /**
     *  A qualified name.
     *  @since  Version 0.1 2005-10-19
     */
    protected static final String QNAME = "test:abstractSchemaPropertyTest";

    /**
     *  Whether the property has encoding schemes that may apply to it.
     *  @since  Version 0.5 2005-10-19
     */
    protected static final boolean ENCODING = true;

    /**
     *  Whether the property may have a URI encoding schemes.
     *  @since  Version 0.6 2005-10-19
     */
    protected static final boolean URI_ENCODING = true;

    /**
     *  A schema property to test.
     *  @since  Version 0.6 2005-10-19
     */
    protected MockSchemaProperty property;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-10-19
     */
    public AbstractSchemaPropertyTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-10-19
     */
    protected final void setUp() {

        property = new MockSchemaProperty(URI,
                                          LABEL,
                                          NAME,
                                          ENCODING,
                                          URI_ENCODING);
    }


    /**
     *  Test the accessor method.
     *  @since  Version 0.1 2005-10-19
     */
    public final void testGetUri() {

        String uri = property.getUri();

        assertTrue(uri.equals(URI));
    }

    /**
     *  Test the accessor method.
     *  @since  Version 0.1 2005-10-19
     */
    public final void testGetLabel() {

        String label = property.getLabel();

        assertTrue(label.equals(LABEL));
    }

    /**
     *  Test the accessor method.
     *  @since  Version 0.1 2005-10-19
     */
    public final void testGetName() {

        String name = property.getName();

        assertTrue(name.equals(NAME));
    }

    /**
     *  Test the accessor method.
     *  @since  Version 0.1 2005-10-19
     */
    public final void testHasEncoding() {

        assertTrue(property.hasEncodingScheme());
    }

    /**
     *  Test the accessor method.
     *  @since  Version 0.1 2005-10-19
     */
    public final void testHasUriEncoding() {

        assertTrue(property.hasUriEncodingScheme());
    }

    /**
     *  Test the name variable.
     *  @since  Version 0.1 2005-10-19
     */
    public final void testGetNameVariable() {

        String nameVar = property.getNameVariable();

        assertTrue(nameVar.equals(NAME_VAR));
    }

    /**
     *  Test the bNode name variable.
     *  @since  Version 0.1 2005-10-19
     */
    public final void testGetBNodeNameVariable() {

        String bNodeVar = property.getBNodeVariable();

        assertTrue(bNodeVar.equals(BNODE_VAR));
    }

    /**
     *  Test the parameter name variable.
     *  @since  Version 0.1 2005-10-19
     */
    public final void testGetParameterName() {

        String param = property.getParameterName();

        assertTrue(param.equals(PARAM));
    }

    /**
     *  Test the qualified name variable.
     *  @since  Version 0.1 2005-10-19
     */
    public final void testGetQualifiedName() {

        String qName = property.getQualifiedName();

        assertTrue(qName.equals(QNAME));
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-10-19
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-10-19
     */
    public static Test suite() {

        return new TestSuite(AbstractSchemaPropertyTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-10-19
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
