package com.mkdoc.servlet;

/*
    Created:    2005-07-22
    Issued:     2005
    Modified:   2005-07-25

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.RequestDispatcher;

/**
 *  Mock request dispatcher for testing.
 *  @version    0.1 2005-07-22
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockRequestDispatcher implements RequestDispatcher {

    /**
     *  The path to which the request will be dispatched.
     *  @since  Verison 0.1 2005-07-22
     */
    protected String path;

    /**
     *  Get a dispatcher for a resource path.
     *  @param  path    The resource path.
     *  @since  Version 0.1 2005-07-22
     */
    public MockRequestDispatcher(final String path) {

        if (path != null) {

            this.path = path;
        }
        else {

            throw new IllegalArgumentException(
                        "Null path not permitted.");
        }
    }

    /**
     *  Forward a servlet request.
     *  @param  request The servlet request object.
     *  @param  response    The servlet response object.
     *  @exception  IOException May be thrown by the servlet resource to which
     *          the request is forwarded.
     *  @exception  ServletException May be thrown by the servlet resource to
     *          which the request is forwarded.
     *  @since  Version 0.1 2005-07-22
     */
    public final void forward(final ServletRequest request,
                              final ServletResponse response)
        throws ServletException, IOException {

        try {

            MockServletResponse mockResponse = (MockServletResponse) response;

            mockResponse.setPath(path);
        }
        catch (ClassCastException cce) {

            // Ignore
        }
    }

    /**
     *  Include a servlet resource.
     *  @param  request The servlet request object.
     *  @param  response    The servlet response object.
     *  @exception  IOException May be thrown by the servlet resource that is
     *          included.
     *  @exception  ServletException May be thrown by the servlet resource that
     *          is included.
     *  @since  Version 0.1 2005-07-22
     */
    public final void include(final ServletRequest request,
                              final ServletResponse response)
        throws ServletException, IOException {

        try {

            MockServletResponse mockResponse = (MockServletResponse) response;

            mockResponse.setPath(path);
        }
        catch (ClassCastException cce) {

            // Ignore
        }
    }


}
