package com.mkdoc.store;

/*
    Created:    2005-05-11
    Issued:     2005-05-11
    Modified:   2005-07-12

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.core.util.config.PropertySet;

import com.mkdoc.jspider.MockPropertySet;

import com.mkdoc.jspider.AbstractMKDocPlugin;

import com.mkdoc.util.ApplicationContext;

import java.io.File;
import java.io.OutputStream;
import java.io.IOException;

import java.net.MalformedURLException;
import java.net.URL;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>HtmlFileStoreManagerPlugin</code>.
 *  @version    0.4 2005-07-12
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class HtmlFileStoreManagerTest extends TestCase {

    /**
     *  The project source directory.
     *  @since  Version 0.1 2005-05-11
     */
    protected static final String HOME = System.getProperty("mkHome");

    /**
     *  A test domain.
     *  @since  Version 0.1 2005-05-11
     */
    protected static final String DOMAIN = System.getProperty("testDomain");

    /**
     *  The default JSpider output directory.
     *  @since  Version 0.1 2005-05-11
     */
    protected static final String OUTPUT = "output";

    /**
     *  A custom output directory.
     *  @since  Version 0.1 2005-05-11
     */
    protected static final String CUSTOM = "custom";

    /**
     *  The custom output path.
     *  @since  Version 0.1 2005-05-11
     */
    protected static final String PATH_CUSTOM = HOME +
                                                File.separator +
                                                OUTPUT +
                                                File.separator +
                                                CUSTOM;

    /**
     *  The default JSpider output directory.
     *  @since  Version 0.1 2004-12-16
     */
    protected static final File DIR_TEST = new File(HOME +
                                                    File.separator +
                                                    OUTPUT);

    /**
     *  A test file.
     *  @since  Version 0.1 2004-12-16
     */
    protected static final String INDEX = "/index.html";

    /**
     *  A test URL.
     *  @since  Version 0.1 2004-12-16
     */
    protected static final String URL_TEST = "http://" +
                                             DOMAIN +
                                             INDEX;


    /**
     *  A file reference for invalid output file.
     *  @since  Version 0.1 2004-12-16
     */
    protected static final File FILE_OUT = new File(DIR_TEST,
                                                    DOMAIN + INDEX);

    /**
     *  A <code>File</code> representation of the custom
     *  output directory for comparisons.
     *  @since  Version 0.1 2005-05-11
     */
    protected static final File DIR_CUSTOM = new File(PATH_CUSTOM);

    /**
     *  A test range of bytes to write.
     *  @since  Version 0.1 2005-05-11
     */
    public static final int BYTE_RANGE = 256;

    /**
     *  A subject manager to test.
     *  @since  Version 0.1 2005-05-11
     */
    protected HtmlFileStoreManager manager;

    /**
     *  A subject manager to test.
     *  @since  Version 0.3 2005-05-31
     */
    protected HtmlFileStoreManager managerInit;

    /**
     *  An expected <code>IllegalArgumentException</code> message.
     *  @since  Version 0.3 2005-05-31
     */
    public static final String EXPECTED_IAE =
    "Should throw an IllegalArgumentException.";


    /**
     *  A URL without file extension.
     *  @since  Version 0.3 2005-05-31
     */
    protected static final String URL_NO_EXT = "http://" +
                                               DOMAIN +
                                               "/";

    /**
     *  A URL with file extension.
     *  @since  Version 0.3 2005-05-31
     */
    protected static final String URL_EXT = "http://" +
                                            DOMAIN +
                                            "/test.html";

    /**
     *  The domain root trimmed.
     *  @since  Version 0.3 2005-05-31
     */
    protected static final String ROOT_TRIM = "/";

    /**
     *  A URL with file extension trimmed.
     *  @since  Version 0.3 2005-05-31
     */
    protected static final String EXT_TRIM = "/test";


    /**
     *  The default JSpider output path.
     *  @since  Version 0.3 2005-05-31
     */
    protected static final String PATH_OUT = HOME +
                                             File.separator +
                                             OUTPUT;
    /**
     *  A temporary output file.
     *  @since  Version 0.3 2005-05-31
     */
    protected static final String INDEX_TMP = "index.html";

    /**
     *  A bad test URL.
     *  @since  Version 0.3 2005-05-31
     */
    protected static final String URL_BAD = "http://" +
                                             DOMAIN +
                                             "/\\:*?\"<>|.BAD";


    /**
     *  A test file path.
     *  @since  Version 0.3 2005-05-31
     */
    protected static final String PATH_TEST = PATH_OUT +
                                              File.separator +
                                              DOMAIN +
                                              File.separator +
                                              INDEX_TMP;

    /**
     *  A <code>File</code> representation of the default
     *  output path for comparisons.
     *  @since  Version 0.3 2005-05-31
     */
    protected static final File DIR_OUT = new File(PATH_OUT);

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-05-11
     */
    public HtmlFileStoreManagerTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-05-11
     */
    protected final void setUp() {

        manager = new HtmlFileStoreManager();

        managerInit = new HtmlFileStoreManager();

        PropertySet propertySet = new MockPropertySet();

        ApplicationContext context = null;

        context = AbstractMKDocPlugin.getApplicationContext(propertySet);

        managerInit.configureOutput(context);
    }

    /**
     *  With relative file path and a valid path specification,
     *  should use the specified path.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2005-05-11
     */
    public final void testInitializeOutputPathAbsolute()
        throws IOException {

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setBoolean(AbstractFileStoreManager.ABSOLUTE, false);
        propertySet.setString(AbstractFileStoreManager.FOLDER, CUSTOM);

        ApplicationContext context = null;

        context = AbstractMKDocPlugin.getApplicationContext(propertySet);

        manager.initialize(context);

        String outPath = manager.outputDir.getCanonicalPath();

        assertTrue("Should use test output directory, '" +
                   DIR_CUSTOM.getCanonicalPath() + "',\n" +
                   "actual output path =             '" +
                   outPath + "'",
                   outPath.equals(DIR_CUSTOM.getCanonicalPath()));
    }

    /**
     *  Get the extension.
     *  @since  Version 0.1 2005-05-11
     */
    public final void testGetExtension() {

        assertTrue(".html".equals(manager.getExtension()));
    }

    /**
     *  Get shutdown, pure coverage test.
     *  @since  Version 0.1 2005-05-11
     */
    public final void testShutdown() {

        manager.shutdown();

        // Not implemented, no assertion
    }

    /**
     *  Configure output should run successfully with empty
     *  property set.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.3 2005-05-31
     */
    public final void testConfigureOutputEmptyPropertySet()
        throws IOException {

        MockPropertySet propertySet = new MockPropertySet();

        ApplicationContext context = null;

        context = AbstractMKDocPlugin.getApplicationContext(propertySet);

        manager.configureOutput(context);

        String outPath = manager.outputDir.getCanonicalPath();

        assertTrue("Should use default output directory, '" +
                   DIR_OUT.getCanonicalPath() + "',\n" +
                   "actual output path =             '" +
                   outPath + "'",
                   outPath.equals(DIR_OUT.getCanonicalPath()));
    }

    /**
     *  With absolute file path but no path specification,
     *  should try the default output directory.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.3 2005-05-31
     */
    public final void testConfigureOutputAbsolutePathNullPath()
        throws IOException {

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setBoolean(AbstractFileStoreManager.ABSOLUTE, true);

        ApplicationContext context = null;

        context = AbstractMKDocPlugin.getApplicationContext(propertySet);

        manager.configureOutput(context);

        String outPath = manager.outputDir.getCanonicalPath();

        assertTrue("Should use the default output directory, '" +
                   DIR_OUT.getCanonicalPath() + "',\n" +
                   "actual output path =             '" +
                   outPath + "'",
                   outPath.equals(DIR_OUT.getCanonicalPath()));
    }

    /**
     *  With absolute file path and a valid path specification,
     *  should use the specified path.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.3 2005-05-31
     */
    public final void testConfigureOutputAbsolutePathWithPath()
        throws IOException {

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setBoolean(AbstractFileStoreManager.ABSOLUTE, true);
        propertySet.setString(AbstractFileStoreManager.FOLDER, PATH_CUSTOM);

        ApplicationContext context = null;

        context = AbstractMKDocPlugin.getApplicationContext(propertySet);

        manager.configureOutput(context);

        String outPath = manager.outputDir.getCanonicalPath();

        assertTrue("Should use test output directory, '" +
                   DIR_CUSTOM.getCanonicalPath() + "',\n" +
                   "actual output path =             '" +
                   outPath + "'",
                   outPath.equals(DIR_CUSTOM.getCanonicalPath()));
    }

    /**
     *  With relative file path but no path specification,
     *  should try the default output directory.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.3 2005-05-31
     */
    public final void testConfigureOutputRelativePathNullPath()
        throws IOException {

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setBoolean(AbstractFileStoreManager.ABSOLUTE, false);

        ApplicationContext context = null;

        context = AbstractMKDocPlugin.getApplicationContext(propertySet);

        manager.configureOutput(context);

        String outPath = manager.outputDir.getCanonicalPath();

        assertTrue("Should use the default output directory, '" +
                   DIR_OUT.getCanonicalPath() + "',\n" +
                   "actual output path =             '" +
                   outPath + "'",
                   outPath.equals(DIR_OUT.getCanonicalPath()));
    }

    /**
     *  With relative file path and a valid path specification,
     *  should use the specified path.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.3 2005-05-31
     */
    public final void testConfigureOutputRelativePathWithPath()
        throws IOException {

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setBoolean(AbstractFileStoreManager.ABSOLUTE, false);
        propertySet.setString(AbstractFileStoreManager.FOLDER, CUSTOM);

        ApplicationContext context = null;

        context = AbstractMKDocPlugin.getApplicationContext(propertySet);

        manager.configureOutput(context);

        String outPath = manager.outputDir.getCanonicalPath();

        assertTrue("Should use test output directory, '" +
                   DIR_CUSTOM.getCanonicalPath() + "',\n" +
                   "actual output path =             '" +
                   outPath + "'",
                   outPath.equals(DIR_CUSTOM.getCanonicalPath()));
    }

    /**
     *  Configure output should not accept a null argument.
     *  @since  Version 0.3 2005-05-31
     */
    public final void testConfigureOutputNullPropertySet() {

        try {

            manager.configureOutput(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Get an output stream and check it is attached to a file.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  StoreException Should not be thrown by this test.
     *  @since  Version 0.3 2005-05-31
     */
    public final void testGetOutputStream()
        throws IOException, StoreException {

        File testFile = new File(PATH_TEST);

        if (testFile.exists()) {

            File parent = testFile.getParentFile();

            if (!testFile.delete()) {

                fail("Could not delete test file, " +
                     testFile.getAbsolutePath() + ".");
            }
            if (!parent.delete()) {

                fail("Could not delete test directory, " +
                     parent.getAbsolutePath() + ".");
            }
        }

        assertFalse(testFile.exists());

        URL url = new URL(URL_TEST);

        OutputStream outStream = managerInit.getOutputStream(url);

        try {

            for (int i = 0; i < BYTE_RANGE; i++) {

                outStream.write(i);
            }
        }
        catch (IOException ioe) {

            fail("Should not throw an IOException " +
                 ioe.getMessage() + ".");
        }
        finally {

            outStream.close();
        }

        assertTrue(testFile.exists());

        if (!testFile.delete()) {

            fail("Could not delete test file, " +
                 testFile.getAbsolutePath() + ".");
        }
    }

    /**
     *  Get an output stream and throw an exception.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  StoreException Should not be thrown by this test.
     *  @since  Version 0.3 2005-05-31
     */
    public final void testGetOutputStreamThrowsException()
        throws IOException, StoreException {

        URL url = new URL(URL_BAD);

        try {

            managerInit.getOutputStream(url);

            fail("Should throw a StoreException.");
        }
        catch (StoreException se) { }

    }


    /**
     *  Get an output file for a subject URL without file part.
     *  @exception  MalformedURLException   Should not be thrown by this test.
     *  @since  Version 0.3 2005-05-31
     */
    public final void testGetStorageFileWithoutExtension()
        throws MalformedURLException {

        File expected = new File(PATH_TEST);

        URL url = new URL(URL_NO_EXT);

        File file = managerInit.getStorageFile(url);

        String path = file.getAbsolutePath();

        assertTrue("Storage file path should be '" +
                   expected.getAbsolutePath() + ", actual path was '" +
                   path + "'.",
                   path.equals(expected.getAbsolutePath()));

    }

    /**
     *  If URL has a dot file extension, should return
     *  <code>true</code>.
     *  @exception  MalformedURLException   If the test URL
     *          is malformed, should not be thrown.
     *  @since  Version 0.3 2005-05-31
     */
    public final void testIncludesFileTrue() throws MalformedURLException {

        URL url = new URL(URL_EXT);

        assertTrue(manager.includesFile(url));
    }

    /**
     *  If URL has no dot file extension, should return
     *  <code>false</code>.
     *  @exception  MalformedURLException   If the test URL
     *          is malformed, should not be thrown.
     *  @since  Version 0.3 2005-05-31
     */
    public final void testIncludesFileFalse() throws MalformedURLException {

        URL url = new URL(URL_NO_EXT);

        assertFalse(manager.includesFile(url));
    }

    /**
     *  Domain root URL should truncate to just "/".
     *  @exception  MalformedURLException   If the test URL
     *          is malformed, should not be thrown.
     *  @since  Version 0.3 2005-05-31
     */
    public final void testTrimExtensionNoExtension()
        throws MalformedURLException {

        URL url = new URL(URL_NO_EXT);

        String trimmed = manager.trimExtension(url);

        assertTrue("Trimmed root URL should equal '" + ROOT_TRIM + "', " +
                   "actually '" + trimmed + "'.",
                   trimmed.equals(ROOT_TRIM));

    }

    /**
     *  Should truncate to file part without the extension.
     *  @exception  MalformedURLException   If the test URL
     *          is malformed, should not be thrown.
     *  @since  Version 0.3 2005-05-31
     */
    public final void testTrimExtensionWithExtension()
        throws MalformedURLException {

        URL url = new URL(URL_EXT);

        String trimmed = manager.trimExtension(url);

        assertTrue("Trimmed file URL should equal '" + EXT_TRIM + "', " +
                   "actually '" + trimmed + "'.",
                   trimmed.equals(EXT_TRIM));

    }

    /**
     *  If parent does not exist, should create.
     *  @since  Version 0.3 2005-05-31
     */
    public final void testCreateDirectoriesNotExist() {

        File parent = FILE_OUT.getParentFile();

        if ((parent.exists()) &&
            (!parent.delete())) {

            fail("Could not delete parent directory, " +
                 parent.getAbsolutePath() + ".");
        }

        assertFalse("Parent directory should not exist but does, '" +
                    parent.getAbsolutePath() + "'.",
                    parent.exists());

        managerInit.createDirectories(FILE_OUT);

        assertTrue("Parent directory should exist but does not, '" +
                   parent.getAbsolutePath() + "'.",
                   parent.exists());

        if (!parent.delete()) {

            fail("Could not delete parent directory, " +
                 parent.getAbsolutePath() + ".");
        }
    }

    /**
     *  Create directories null file.
     *  @since  Version 0.3 2005-05-31
     */
    public final void testCreateDirectoriesNullFile() {

        try {

            managerInit.createDirectories(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Create directories null parent.
     *  @since  Version 0.3 2005-05-31
     */
    public final void testCreateDirectoriesNullParent() {

        File file = new File("NoParent.txt");

        try {

            managerInit.createDirectories(file);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  If parent already exists, should return immediately.
     *  @since  Version 0.3 2005-05-31
     */
    public final void testCreateDirectoriesParentExists() {

        managerInit.createDirectories(FILE_OUT);

        File parent = FILE_OUT.getParentFile();

        assertTrue(parent.exists());

        long modified1 = parent.lastModified();

        managerInit.createDirectories(FILE_OUT);

        long modified2 = parent.lastModified();

        assertTrue(modified1 == modified2);

        if (!parent.delete()) {

            fail("Could not delete parent directory, " +
                 parent.getAbsolutePath() + ".");
        }
    }


    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-05-11
     */
    public static Test suite() {

        return new TestSuite(HtmlFileStoreManagerTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-05-11
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
