package com.mkdoc.store;

/*
    Created:    2005-10-26
    Issued:     2005-10-26
    Modified:   2005-10-26

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.openrdf.model.Resource;
import org.openrdf.model.Statement;
import org.openrdf.model.URI;
import org.openrdf.model.Value;

import org.openrdf.model.impl.StatementImpl;
import org.openrdf.model.impl.URIImpl;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>ImportListener</code>.
 *  <p>
 *    Draft class so many placeholder tests
 *  </p>
 *  @version    0.1 2005-10-26
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class ImportListenerTest extends TestCase {

    /**
     *  A test error message.
     *  @since  Version 0.1 2005-10-26
     */
    public static final String ERROR = "I didn't bring the jump leads.";

    /**
     *  A test warning message.
     *  @since  Version 0.1 2005-10-26
     */
    public static final String WARNING = "It may get too cold soon.";

    /**
     *  A test notice message.
     *  @since  Version 0.1 2005-10-26
     */
    public static final String NOTICE = "See how deep the snow is.";

    /**
     *  A subject URI.
     *  @since  Version 0.1 2005-10-26
     */
    public static final String SUBJECT_URI =
    "http://www.remote.example/uri/ImportListener";

    /**
     *  A predicate URI.
     *  @since  Version 0.1 2005-10-26
     */
    public static final String PRED_URI =
    "http://www.remote.example/uri/Audible";

    /**
     *  An object URI.
     *  @since  Version 0.1 2005-10-26
     */
    public static final String OBJECT_URI =
    "http://www.remote.example/uri/True";


    /**
     *  A listener to test.
     *  @since  Version 0.1 2005-10-26
     */
    protected ImportListener listener;

    /**
     *  A statement for testing.
     *  @since  Version 0.1 2005-10-26
     */
    protected Statement statement;

    /**
     *  A subject resource for testing.
     *  @since  Version 0.1 2005-10-26
     */
    protected Resource subject;

    /**
     *  A predicate URI for testing.
     *  @since  Version 0.1 2005-10-26
     */
    protected URI predicate;

    /**
     *  An object value for testing.
     *  @since  Version 0.1 2005-10-26
     */
    protected Value object;


    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-10-26
     */
    public ImportListenerTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-10-26
     */
    protected final void setUp() {

        listener = new ImportListener();

        subject = new URIImpl(SUBJECT_URI);

        predicate = new URIImpl(PRED_URI);

        object = new URIImpl(OBJECT_URI);

        statement = new StatementImpl(subject, predicate, object);
    }

    /**
     *  Test the empty methods, pure coverage, no assertions.
     *  @since  Version 0.1 2005-10-26
     */
    public final void testTransactions() {

        listener.transactionStart();

        listener.transactionEnd();

        listener.status("Testing", 1, 2);
    }

    /**
     *  Test the notification method.
     *  @since  Version 0.1 2005-10-26
     */
    public final void testNotification() {

        listener.notification(NOTICE, 1, 2, statement);

        assertTrue(listener.notices.contains(statement));
        assertFalse(listener.errors.contains(statement));
        assertFalse(listener.warnings.contains(statement));
    }

    /**
     *  Test the error method.
     *  @since  Version 0.1 2005-10-26
     */
    public final void testError() {

        listener.error(ERROR, 1, 2, statement);

        assertTrue(listener.errors.contains(statement));
        assertFalse(listener.notices.contains(statement));
        assertFalse(listener.warnings.contains(statement));
    }

    /**
     *  Test the warning method.
     *  @since  Version 0.1 2005-10-26
     */
    public final void testWarning() {

        listener.warning(WARNING, 1, 2, statement);

        assertTrue(listener.warnings.contains(statement));
        assertFalse(listener.errors.contains(statement));
        assertFalse(listener.notices.contains(statement));
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-10-26
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-10-26
     */
    public static Test suite() {

        return new TestSuite(ImportListenerTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-10-26
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
